/*
 * Copyright (C) oStorybook Team
 *
 * This program is free software, see <http://www.gnu.org/licenses/>.
 *
 * Parts of this code are from the NovaWorx project. Other parts are
 * from the SHEF project developed and published by Bob Tantlinger.
 */
package shef.actions;

import i18n.I18N;
import java.awt.HeadlessException;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.IOException;
import javax.swing.Action;
import javax.swing.JEditorPane;
import javax.swing.KeyStroke;
import javax.swing.text.html.HTMLDocument;
import resources.icons.ICONS;
import shef.editors.wys.HTMLTextEditAction;
import shef.editors.wys.WysiwygEditor;
import storybook.toolkit.html.Html;

public class EditPasteAction extends HTMLTextEditAction {

	private static final String TT = "EditPasteAction";

	public EditPasteAction(WysiwygEditor editor) {
		super(editor, I18N.getMsg("shef.paste"));
		putValue(MNEMONIC_KEY, I18N.getMnem("shef.paste"));
		setSmallIcon(ICONS.K.EDIT_PASTE);
		setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, InputEvent.CTRL_DOWN_MASK));
		addShouldBeEnabledDelegate((Action a) -> true);
		setShortDescription(I18N.getMsg("shef.paste"));
		enabled = true;
	}

	@Override
	protected void updateWysiwygContextState(JEditorPane wysEditor) {
		//LOG.trace(TT + ".updateWysiwygContextState(editor)");
		this.updateEnabledState();
	}

	@Override
	protected void wysiwygEditPerformed(ActionEvent e, JEditorPane editor) {
		//LOG.trace(TT + ".wysiwygEditPerformed(e=" + e.toString() + ", editor)");
		HTMLDocument document = (HTMLDocument) editor.getDocument();
		try {
			Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
			Transferable content = clip.getContents(this);
			String txt = content.getTransferData(
					new DataFlavor(String.class, "String")).toString();
			txt = Html.htmlToText(txt);
			if (txt == null || txt.isEmpty()) {
				return;
			}
			int caret = editor.getCaretPosition();
			if (caret < 1) {
				return;
			}
			CompoundUndoManager.beginCompoundEdit(document);
			String x = editor.getSelectedText();
			if (x == null || x.isEmpty()) {
				editor.setSelectionStart(caret);
				editor.setSelectionEnd(caret);
			}
			editor.replaceSelection(txt);
			CompoundUndoManager.endCompoundEdit(document);
		} catch (HeadlessException | UnsupportedFlavorException | IOException ex) {
			ex.printStackTrace(System.err);
		}
	}

}
