/*
 * Copyright (C) 2019 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.edit;

import storybook.ui.Ui;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JTextField;
import org.miginfocom.swing.MigLayout;
import resources.icons.ICONS;
import storybook.Const;
import static storybook.ui.Ui.initIntegerField;
import i18n.I18N;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.DB_DATA;
import storybook.model.hbn.entity.Event;
import storybook.toolkit.LOG;
import storybook.toolkit.SbDuration;
import storybook.toolkit.html.Html;
import storybook.toolkit.swing.js.JSDateChooser;
import storybook.ui.MIG;
import storybook.ui.dialog.chooser.ColorPicker;
import static storybook.ui.panel.AbstractPanel.INFO;
import static storybook.ui.panel.AbstractPanel.MANDATORY;

/**
 *
 * @author favdb
 */
public class EditEvent extends AbstractEditor {

	private JComboBox cbCategory;
	private JSDateChooser dDate;
	private JTextField fDuration;
	//private ColorChooserPanel tfColor;
	private ColorPicker cbColor;
	private static String INITIALES = I18N.getMsg("duration.initiales");
	private JTextField tfTimestep;
	private JButton btColor;

	public EditEvent(Editor m, AbstractEntity e) {
		super(m, e, "090");
		initAll();
	}

	@Override
	public void initUpper() {
		Event event = (Event) entity;
		cbCategory = Ui.initAutoCombo(panel, "category",
				Event.findCategories(mainFrame),
				event.getCategory(),
				MANDATORY, !EMPTY, GROW);
		JPanel p = new JPanel(new MigLayout(MIG.INS0));
		cbColor = new ColorPicker(event.getJColor());
		p.add(cbColor);
		btColor = Ui.initButton("btColor", "...", ICONS.K.COLOR, "color.other", e -> changeColor());
		p.add(btColor);
		if (cbColor.getSelectedIndex() < 1) {
			btColor.setBackground(event.getJColor());
		}
		Ui.addField(panel, "color", "", p, !GROW, MANDATORY, null);
		dDate = initSbDate(panel, "date", event.getEventTime(), MANDATORY, 0);
		String tDuration = "";
		if (!event.getDuration().isEmpty()) {
			SbDuration dur = new SbDuration(event.getDuration());
			tDuration = dur.toText(INITIALES);
		}
		fDuration = Ui.initStringField(panel, "duration", 16, tDuration, !MANDATORY, !INFO);
		fDuration.setToolTipText(I18N.getMsg("duration.format"));
		tfTimestep = initIntegerField(panel, DB_DATA.DATA.EVENT_TIMESTEP, 5, event.getTimeStep(), !MANDATORY, !INFO);
		tfTimestep.setToolTipText(I18N.getMsg("event.timestep_tip"));
	}

	@Override
	public boolean verifier() {
		JTextField tf = new JTextField();
		resetError();
		dDate.setBorder(tf.getBorder());
		if (dDate.hasError() || dDate.getDate() == null) {
			errorMsg(dDate, Const.ERROR_MISSING);
		}
		if (!fDuration.getText().isEmpty()) {
			String str = SbDuration.check(fDuration.getText(), INITIALES);
			if (!str.isEmpty()) {
				errorString(fDuration, str);
			}
		}
		/*if (!tfTimestep.getText().isEmpty() && !StringUtil.isNumeric(tfTimestep.getText())) {
			errorMsg(tfTimestep, Const.ERROR_NOTNUMERIC);
		}*/
		if (Html.htmlToText(hNotes.getText()).isEmpty()) {
			errorMsg(hNotes, Const.ERROR_MISSING);
		}
		return (msgError.isEmpty());
	}

	@Override
	public void apply() {
		Event event = (Event) entity;
		event.setCategory((String) cbCategory.getSelectedItem());
		event.setEventTime(dDate.getTimestamp());
		SbDuration dur = SbDuration.getFromText(fDuration.getText(), INITIALES);
		event.setDuration(dur.toString());
		/*if (!tfTimestep.getText().isEmpty()) {
			event.setTimeStep(Integer.parseInt(tfTimestep.getText()));
		} else {
			event.setTimeStep(0);
		}*/
		dur = SbDuration.getFromText(tfTimestep.getText());
		event.setTimeStep(dur.toMinutes());
		if (cbColor.getSelectedIndex() == -1) {
			event.setJColor(btColor.getBackground());
		} else {
			event.setJColor((Color) cbColor.getSelectedItem());
		}
		super.apply();
	}

	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		// empty
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		// empty
	}

	private void changeColor() {
		LOG.trace("changeColor()");
		Color color = JColorChooser.showDialog(this, I18N.getMsg("color.choose"), ((Event) entity).getJColor());
		btColor.setBackground(color);
	}

}
