/*
 * Copyright (C) 2022 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.edit;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.util.ArrayList;
import java.util.List;
import javax.swing.ButtonGroup;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.ListCellRenderer;
import javax.swing.SwingConstants;
import org.miginfocom.swing.MigLayout;
import storybook.App;
import storybook.toolkit.html.Html;
import storybook.toolkit.swing.ColorIcon;
import storybook.toolkit.swing.ColorUtil;
import storybook.ui.MIG;

/**
 *
 * @author favdb
 */
public class Intensity extends JPanel {

    private static final String TT = "IntensityPanel";

    public static final boolean RADIO = false, LIST = true;

    private List<JRadioButton> btc = new ArrayList<>();
    private List<Color> colors = new ArrayList<>();
    private boolean mode;
    private JComboBox cbList;

    public Intensity(boolean mode, int value) {
	super();
	this.mode = mode;
	initColors();
	initAll();
	setValue(value);
    }

    private void initColors() {
	colors.add(App.getInstance().preferences.intensityGet(0));
	colors.add(App.getInstance().preferences.intensityGet(1));
	colors.add(App.getInstance().preferences.intensityGet(2));
	colors.add(App.getInstance().preferences.intensityGet(3));
	colors.add(App.getInstance().preferences.intensityGet(4));
    }

    private void initAll() {
	init();
	initUi();
    }

    private void init() {
	// empty
    }

    private void initRadioButton() {
	ButtonGroup btg = new ButtonGroup();
	for (int i = 0; i < 5; i++) {
	    JRadioButton bt = new JRadioButton(String.format(" %d ", i + 1));
	    bt.setBackground(getColor(i + 1));
	    if (ColorUtil.isDark(getColor(i + 1))) {
		bt.setForeground(Color.WHITE);
	    }
	    btc.add(bt);
	    btg.add(bt);
	}
    }

    @SuppressWarnings("unchecked")
    private void initUi() {
	setLayout(new MigLayout(MIG.INS0));
	if (mode == LIST) {
	    cbList = new JComboBox();
	    cbList.setRenderer(new ComboBoxRenderer(this, cbList));
	    initRadioButton();
	    for (JRadioButton b : btc) {
		Font f = b.getFont();
		b.setIcon(new ColorIcon(b.getBackground(), f.getSize(), f.getSize()));
		b.setHorizontalTextPosition(SwingConstants.LEFT);
		cbList.addItem(b);
	    }
	    add(cbList);
	} else {
	    initRadioButton();
	    for (JRadioButton b : btc) {
		add(b);
	    }
	}
    }

    public int getValue() {
	if (mode == LIST) {
	    return cbList.getSelectedIndex() + 1;
	}
	for (int i = 0; i < 5; i++) {
	    if (btc.get(i).isSelected()) {
		return i + 1;
	    }
	}
	return 1;
    }

    public void setValue(int value) {
	//App.trace(TT + ".setValue(value=" + value + ")");
	if (mode == LIST) {
	    cbList.setSelectedIndex(value - 1);
	}
	btc.get(value - 1).setSelected(true);
    }

    public Color getColor(int i) {
	return (i >= 1 && i <= 5 ? colors.get(i - 1) : colors.get(0));
    }

    public Color getColor() {
	if (mode == LIST) {
	    return colors.get(cbList.getSelectedIndex());
	}
	return colors.get(getValue() - 1);
    }

    public static Color getColorValue(int val) {
	Intensity intensity = new Intensity(LIST, val);
	return intensity.getColor();
    }

    public String getHtmlValue() {
	Integer v = getValue();
	String html = v.toString() + " " + Html.getColorSpan(getColor(), 3);
	return html;
    }

    private static class ComboBoxRenderer implements ListCellRenderer {

	private final Intensity panel;

	public ComboBoxRenderer(Intensity p, JComboBox cbList) {
	    this.panel = p;
	}

	@Override
	public Component getListCellRendererComponent(JList list, Object value, int index,
		boolean isSelected, boolean cellHasFocus) {
	    if (value instanceof JRadioButton) {
		JRadioButton rb = (JRadioButton) value;
		rb.setSelected(isSelected);
		Color c = panel.getColor(index + 1);
		rb.setBackground(c);
		if (ColorUtil.isDark(c)) {
		    rb.setForeground(Color.WHITE);
		}
		return rb;
	    }
	    return (Component) value;
	}
    }

}
