/*
 * Copyright (C) 2020 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.exim;

import i18n.I18N;
import java.io.File;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import storybook.exim.exporter.Exporter;
import storybook.exim.importer.Importer;
import storybook.ui.MainFrame;
import storybook.ui.SbView;

/**
 *
 * @author favdb
 */
public class EXIM {

	private static String TT = "EXIM";

	private EXIM() {
		throw new IllegalStateException("Utility class");
	}

	public static boolean exporter(MainFrame mainFrame, SbView view) {
		if (view == null) {
			return (false);
		}
		Exporter exp = new Exporter(mainFrame);
		return (exp.exec(view));
	}

	public static boolean exporter(MainFrame mainFrame, JPanel panel) {
		if (panel == null) {
			return (false);
		}
		Exporter exp = new Exporter(mainFrame);
		return (exp.exportPanel(panel));
	}

	public static boolean importer(MainFrame mainFrame, File xml) {
		if (xml == null) {
			return (false);
		}
		Importer exp = new Importer(mainFrame, xml);
		return exp.exec(xml.getAbsolutePath());
	}

	public static String getDirExport(MainFrame mainFrame) {
		String name = mainFrame.getBook().getParamExport().getDirectory();
		File file = new File(name);
		if (name.isEmpty()) {
			name = mainFrame.getH2File().getPath();
		}
		if (!file.exists()) {
			if (!file.mkdir()) {
				return mainFrame.getH2File().getPath();
			}
		}
		return name;
	}

	public static boolean askExists(JComponent p, String name) {
		File file = new File(name);
		if (file.exists() && JOptionPane.showConfirmDialog(p,
				I18N.getMsg("export.replace", name),
				I18N.getMsg("export"),
				JOptionPane.OK_CANCEL_OPTION) == JOptionPane.CANCEL_OPTION) {
			return (false);
		}
		return (true);
	}

	/**
	 * for export to a file
	 */
	public static String[] extFile = {
		"docx,MS Word",
		"odt,LibreOffice",
		"other," + I18N.getMsg("other")
	};

	public static String getEditor(String ext) {
		int n = findExt(ext);
		if (n > -1) {
			String xname[] = extFile[n].split(",");
			return xname[1];
		}
		return "";
	}

	public static String getEditor(int ext) {
		if (ext < extFile.length) {
			String xname[] = extFile[ext].split(",");
			return xname[1];
		}
		return "???";
	}

	public static int findExt(String ext) {
		//LOG.trace(TT + ".findExt(ext=" + ext + ")");
		for (int i = 0; i < extFile.length; i++) {
			String xname[] = extFile[i].split(",");
			if (xname[0].equals(ext)) {
				return i;
			}
		}
		return extFile.length - 1;
	}

}
