/*
 * Copyright (C) 2016 favdb
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package storybook.exim.exporter;

import i18n.I18N;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JOptionPane;
import org.hibernate.Session;
import storybook.App;
import storybook.Pref;
import storybook.model.Model;
import storybook.model.book.Book;
import storybook.model.book.BookParamExport;
import storybook.model.hbn.dao.*;
import storybook.model.hbn.entity.*;
import storybook.model.hbn.entity.DB_DATA.DATA;
import storybook.toolkit.LOG;
import storybook.ui.MainFrame;
import storybook.ui.SbView;

/**
 * class for exporting a table
 *
 * @author favdb
 */
public class ExportTable extends AbstractExport {
	//TODO à réécrire pour profiter pleinement de getColumns et getRow

	/**
	 * export a table
	 *
	 * @param mainFrame
	 * @param format
	 */
	public ExportTable(MainFrame mainFrame, String format) {
		super(mainFrame, format);
	}

	enum KW {
		ID,
		NAME,
		TITLE,
		DESCRIPTION,
		NOTES,
		ASSISTANT,
		STATUS,
		CATEGORY,
		EXPORT,
		DATABASE,
		XML,
		HTML,
		KEY,
		VALUE,
		STEP,
		POLTI,
		PARAM,
		BACKUP,
		DIRECTORY,
		AUTO,
		INCREMENT,
		EDITOR,
		MODLESS,
		XUSE,
		EXTEND,
		TEMPLATE,
		IMPORT, FILE,
		SCENE_START, SCENE_END,
		LOCATION,
		PERSON,
		ADRESS,
		CITY,
		COUNTRY,
		ALTITUDE,
		NUMBER,
		DATE,
		COLOR,
		SORT;

		@Override
		public String toString() {
			return this.name().toLowerCase().replace("_", ".");
		}
	}

	public static void exportDB(MainFrame m, String format) {
		ExportTable export = new ExportTable(m, format);
		if (export.askFileExists("DataBase") && JOptionPane.showConfirmDialog(m,
				I18N.getMsg("export.replace", export.param.getFileName()),
				I18N.getMsg(KW.EXPORT.toString()),
				JOptionPane.OK_CANCEL_OPTION) == JOptionPane.CANCEL_OPTION) {
			return;
		}
		export.writeDataBase();
	}

	public static void exportDB(MainFrame m) {
		// if export directory is empty then set it to default
		BookParamExport p = m.getBook().getParamExport();
		boolean b = false;
		if (p.getDirectory().isEmpty()) {
			b = true;
		} else {
			File dir = new File(p.getDirectory());
			if (!dir.exists() || !dir.isDirectory()) {
				b = true;
			}
		}
		if (b) {
			p.setDirectory(m.getH2File().getPath());
			p.save();
		}
		exportDB(m, KW.XML.toString());
	}

	public static void exportTable(MainFrame m, SbView view) {
		exportTable(m, view, App.getPref().getString(Pref.KEY.EXP_FORMAT));
	}

	public static void exportTable(MainFrame m, SbView view, String format) {
		//App.trace("ExportTable.exportTable(mainFrame, view="+view.toString()+", f="+f+")");
		exportTable(m, view.getName(), format);
	}

	public static void exportTable(MainFrame m, String viewName, String format) {
		//App.trace("ExportTable.exportTable(mainFrame, v="+v+", f="+f+")");
		ExportTable export = new ExportTable(m, format);
		if (export.askFileExists(viewName)
				&& JOptionPane.showConfirmDialog(m,
						I18N.getMsg("export.replace", export.param.getFileName()),
						I18N.getMsg(KW.EXPORT.toString()),
						JOptionPane.OK_CANCEL_OPTION) == JOptionPane.CANCEL_OPTION) {
			return;
		}
		if (!export.openFile("table_" + viewName)) {
			return;
		}
		export.writeTable(viewName);
		export.closeFile(true);
	}

	public void setDir(String d) {
		param.setDirectory(d);
	}

	public void setFormat(String f) {
		param.setFormat(f);
	}

	public void writeTable(String name) {
		//App.trace("ExportTable.writeTable(name="+name+")");
		if (!isOpened) {
			return;
		}
		switch (Book.getTYPE(name)) {
			case ATTRIBUTE:
			case ATTRIBUTES:
				writeAttributes();
				break;
			case CATEGORY:
			case CATEGORIES:
				writeCategories();
				break;
			case CHAPTER:
			case CHAPTERS:
				writeChapters();
				break;
			case ENDNOTE:
			case ENDNOTES:
				writeEndnotes();
				break;
			case EVENT:
			case EVENTS:
				writeEvents();
				break;
			case GENDER:
			case GENDERS:
				writeGenders();
				break;
			case IDEA:
			case IDEAS:
				writeIdeas();
				break;
			case INTERNAL:
			case INTERNALS:
				writeInternal();
				break;
			case ITEM:
			case ITEMS:
				writeItems();
				break;
			case ITEMLINK:
			case ITEMLINKS:
				writeItemlinks();
				break;
			case LOCATION:
			case LOCATIONS:
				writeLocations();
				break;
			case MEMO:
			case MEMOS:
				writeMemos();
				break;
			case PART:
			case PARTS:
				writeParts();
				break;
			case PERSON:
			case PERSONS:
				writePersons();
				break;
			case RELATION:
			case RELATIONS:
				writeRelations();
				break;
			case SCENE:
			case SCENES:
				writeScenes();
				break;
			case STRAND:
			case STRANDS:
				writeStrands();
				break;
			case TAG:
			case TAGS:
				writeTags();
				break;
			case TAGLINK:
			case TAGLINKS:
				writeTaglinks();
				break;
			default:
				LOG.err("unknown type to export " + name);
				break;
		}
		if (KW.HTML.toString().equals(param.getFormat())) {
			writeText("    </table>\n");
		}
	}

	private void writeAttributes() {
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.ATTRIBUTE_KEY.i18n()), 10));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.ATTRIBUTE_VALUE.i18n()), 80));
		writeHeaderColumn(headers);
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		AttributeDAOImpl dao = new AttributeDAOImpl(session);
		List<Attribute> entities = dao.findAll();
		for (Attribute e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writeCategories() {
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.NAME.i18n()), 32));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.CATEGORY_SORT.i18n()), 5));
		headers.add(new ColumnHeader(I18N.getMsg("category.value"), 80));
		writeHeaderColumn(headers);
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		CategoryDAOImpl dao = new CategoryDAOImpl(session);
		List<Category> entities = dao.findAll();
		for (Category e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writeChapters() {
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.PART.i18n()), 5));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.CHAPTER_CHAPTERNO.i18n()), 2));
		headers.add(new ColumnHeader(I18N.getMsg(KW.TITLE.toString()), 60));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.CREATION.i18n()), 10));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.OBJECTIVE_DATE.i18n()), 10));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.OBJECTIVE_DONE.i18n()), 10));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.OBJECTIVE_SIZE.i18n()), 6));
		headers.add(new ColumnHeader(I18N.getMsg(KW.DESCRIPTION.toString()), 30));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NOTES.toString()), 30));
		writeHeaderColumn(headers);
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		ChapterDAOImpl dao = new ChapterDAOImpl(session);
		List<Chapter> entities = dao.findAll();
		for (Chapter e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writeGenders() {
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NAME.toString()), 32));
		headers.add(new ColumnHeader(I18N.getMsg("gender.childhood"), 3));
		headers.add(new ColumnHeader(I18N.getMsg("gender.adolescence"), 3));
		headers.add(new ColumnHeader(I18N.getMsg("gender.adulthood"), 3));
		headers.add(new ColumnHeader(I18N.getMsg("gender.retirement"), 3));
		headers.add(new ColumnHeader(I18N.getMsg("icone"), 32));
		writeHeaderColumn(headers);
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		GenderDAOImpl dao = new GenderDAOImpl(session);
		List<Gender> entities = dao.findAll();
		for (Gender e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writeIdeas() {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		IdeaDAOImpl dao = new IdeaDAOImpl(session);
		List<Idea> entities = dao.findAll();
		//writeText("nb ideas="+entities.size());
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg(KW.STATUS.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.CATEGORY.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NOTES.toString()), 32));
		writeHeaderColumn(headers);
		for (Idea e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writeInternal() {
		if (KW.XML.toString().equals(param.getFormat())) {
			writeInfo();
			writeParam();
		} else {
			List<ColumnHeader> headers = new ArrayList<>();
			headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
			headers.add(new ColumnHeader(I18N.getMsg(KW.KEY.toString()), 16));
			headers.add(new ColumnHeader(I18N.getMsg(KW.VALUE.toString()), 32));
			writeHeaderColumn(headers);
		}
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		InternalDAOImpl dao = new InternalDAOImpl(session);
		List<Internal> entities = dao.findAll();
		for (Internal e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writeInfo() {
		writeText("    <info \n");
		StringBuilder b = new StringBuilder();
		b.append(stringAttribute(2, KW.TITLE.toString(), book.getTitle())).append("\n");
		b.append(stringAttribute(2, "subtitle", book.getSubtitle())).append("\n");
		b.append(stringAttribute(2, "author", book.getAuthor())).append("\n");
		b.append(stringAttribute(2, "copyright", book.getCopyright())).append("\n");
		writeText(b.toString() + "    >\n");
		//todo check if assistant XML is ok
		String v = book.info.assistantGet();
		if (!v.isEmpty()) {
			writeXml(2, KW.ASSISTANT.toString(), "");
			writeXml(3, v, "");
		}
		//blurb
		v = book.getBlurb();
		if (!v.isEmpty()) {
			writeXml(2, "blurb", v);
		}
		//notes
		v = book.getNotes();
		if (!v.isEmpty()) {
			writeXml(2, KW.NOTES.toString(), v);
		}
		writeXml(1, "/info", "");
	}

	private void writeParam() {
		//begin param
		writeXml(1, KW.PARAM.toString(), "");

		writeParamBackup();
		writeParamEditor();
		writeParamExport();
		writeParamImport();

		//end param
		writeXml(1, "/" + KW.PARAM.toString(), "");
	}

	private void writeParamBackup() {
		StringBuilder b;
		b = new StringBuilder();
		b.append(stringAttribute(0, KW.DIRECTORY.toString(), mainFrame.getPref().getString(Pref.KEY.BACKUP_DIR, "")));
		b.append(stringAttribute(0, KW.AUTO.toString(), mainFrame.getPref().getBoolean(Pref.KEY.BACKUP_OPEN)));
		b.append(stringAttribute(0, KW.INCREMENT.toString(), mainFrame.getPref().getBoolean(Pref.KEY.BACKUP_INCREMENT)));
		writeXmlAttribute(2, KW.BACKUP.toString(), b.toString());
	}

	private void writeParamEditor() {
		StringBuilder b;
		b = new StringBuilder();
		b.append(stringAttribute(0, KW.MODLESS.toString(), mainFrame.getPref().editorGetModless()));
		if (book.isUseXeditor()) {
			b.append(stringAttribute(0, KW.XUSE.toString(), book.isUseXeditor()));
			b.append(stringAttribute(0, KW.EXTEND.toString(), book.getParamEditor().getExtension()));
			//b.append(stringAttribute(0, KW.TEMPLATE.toString(), book.param.getParamEditor().getTemplate()));
		}
		writeXmlAttribute(2, KW.EDITOR.toString(), b.toString());
	}

	private void writeParamExport() {
		BookParamExport paramBook = mainFrame.getBook().getParamExport();
		StringBuilder b = new StringBuilder();
		b.append(stringAttribute(0, KW.DIRECTORY.toString(), paramBook.getDirectory()));
		writeXml(2, KW.EXPORT.toString() + " " + b.toString(), "");

		//export part options
		b = new StringBuilder();
		b.append(stringAttribute(0, KW.TITLE.toString(), param.getLayout().getPartTitle()));
		writeXmlAttribute(3, "part", b.toString());

		//export layout options
		b = new StringBuilder();
		b.append(stringAttribute(0, "booktitle", paramBook.getChapterBookTitle()));
		b.append(stringAttribute(0, "booklayout", param.getLayout().getBookLayout()));
		b.append(stringAttribute(4, "breakpage", paramBook.getChapterBreakPage()));
		writeXmlAttribute(3, "layout", b.toString());

		//export csv and txt options
		b = new StringBuilder();
		b.append(stringAttribute(0, "csvcomma", param.getCsvComma()));
		b.append(stringAttribute(0, "csvquote", param.getCsvQuote()));
		b.append(stringAttribute(0, "txttab", param.getTxtSeparator()));
		writeXmlAttribute(3, "csv", b.toString());

		//export html options
		b = new StringBuilder();
		b.append(stringAttribute(0, "css", paramBook.getHtmlCss()));
		b.append(stringAttribute(0, "multichapter", paramBook.getHtmlMultiChapter()));
		b.append(stringAttribute(0, "multiscene", paramBook.getHtmlMultiScene()));
		b.append(stringAttribute(0, "highlight", paramBook.getHighlight()));
		b.append(stringAttribute(0, "nav", paramBook.getHtmlNav()));
		b.append(stringAttribute(0, "navimage", paramBook.getHtmlNavImage()));
		writeXmlAttribute(3, KW.HTML.toString(), b.toString());

		//end export
		writeXml(2, "/" + KW.EXPORT.toString(), "");
	}

	private void writeParamImport() {
		StringBuilder b;
		//import
		b = new StringBuilder();
		b.append(stringAttribute(0, KW.DIRECTORY.toString(), mainFrame.getPref().getString(Pref.KEY.IMP_DIRECTORY)));
		b.append("\n");
		b.append(stringAttribute(3, KW.FILE.toString(), mainFrame.getPref().getString(Pref.KEY.IMP_FILE)));
		writeXmlAttribute(2, KW.IMPORT.toString(), b.toString());
	}

	private void writeItems() {
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NAME.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.CATEGORY.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.STATUS.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.DESCRIPTION.toString()), 32));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NOTES.toString()), 32));
		writeHeaderColumn(headers);
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		ItemDAOImpl dao = new ItemDAOImpl(session);
		List<Item> entities = dao.findAll();
		for (Item e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writeItemlinks() {
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg(KW.SCENE_START.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.SCENE_END.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.LOCATION.toString()), 32));
		headers.add(new ColumnHeader(I18N.getMsg(KW.PERSON.toString()), 32));
		writeHeaderColumn(headers);
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		ItemlinkDAOImpl dao = new ItemlinkDAOImpl(session);
		List<Itemlink> entities = dao.findAll();
		for (Itemlink e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writeLocations() {
		//App.trace("ExportTable.writeLocations()");
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NAME.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.ADRESS.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.CITY.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.COUNTRY.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.ALTITUDE.toString()), 8));
		headers.add(new ColumnHeader(I18N.getMsg("location.site"), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.DESCRIPTION.toString()), 32));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NOTES.toString()), 32));
		writeHeaderColumn(headers);
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		LocationDAOImpl dao = new LocationDAOImpl(session);
		List<Location> entities = dao.findAll();
		for (Location e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writeMemos() {
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NAME.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NOTES.toString()), 32));
		writeHeaderColumn(headers);
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		MemoDAOImpl dao = new MemoDAOImpl(session);
		List<Memo> entities = dao.findAll();
		for (Memo e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writeParts() {
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NUMBER.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NAME.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg("part"), 16));
		headers.add(new ColumnHeader(I18N.getMsg("manage.date.creation"), 10));
		headers.add(new ColumnHeader(I18N.getMsg("manage.date.objective"), 10));
		headers.add(new ColumnHeader(I18N.getMsg("manage.date.done"), 10));
		headers.add(new ColumnHeader(I18N.getMsg("manage.size.objective"), 6));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NOTES.toString()), 32));
		writeHeaderColumn(headers);
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		PartDAOImpl dao = new PartDAOImpl(session);
		List<Part> entities = dao.findAll();
		for (Part e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writePersons() {
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg("gender"), 16));
		headers.add(new ColumnHeader(I18N.getMsg("person.firstname"), 16));
		headers.add(new ColumnHeader(I18N.getMsg("person.lastname"), 16));
		headers.add(new ColumnHeader(I18N.getMsg("person.abbr"), 5));
		headers.add(new ColumnHeader(I18N.getMsg("person.birthday"), 10));
		headers.add(new ColumnHeader(I18N.getMsg("person.death"), 10));
		headers.add(new ColumnHeader(I18N.getMsg("person.occupation"), 16));
		headers.add(new ColumnHeader(I18N.getMsg("person.color"), 10));
		headers.add(new ColumnHeader(I18N.getMsg(KW.CATEGORY.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg("attributes"), 10));
		headers.add(new ColumnHeader(I18N.getMsg(KW.DESCRIPTION.toString()), 32));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NOTES.toString()), 32));
		writeHeaderColumn(headers);
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		PersonDAOImpl dao = new PersonDAOImpl(session);
		List<Person> entities = dao.findAll();
		for (Person e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writeRelations() {
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg("person.first"), 16));
		headers.add(new ColumnHeader(I18N.getMsg("person.second"), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.SCENE_START.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.SCENE_END.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg("persons"), 32));
		headers.add(new ColumnHeader(I18N.getMsg("items"), 32));
		headers.add(new ColumnHeader(I18N.getMsg("locations"), 32));
		headers.add(new ColumnHeader(I18N.getMsg(KW.DESCRIPTION.toString()), 32));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NOTES.toString()), 32));
		writeHeaderColumn(headers);
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		RelationDAOImpl dao = new RelationDAOImpl(session);
		List<Relationship> entities = dao.findAll();
		for (Relationship e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writeScenes() {
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg(KW.TITLE.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.CHAPTER.i18n()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.STRAND.i18n()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NUMBER.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.DATE.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.STATUS.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.INFORMATIVE.i18n()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.DURATION.i18n()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.INTENSITY.i18n()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.SCENE_NARRATOR.i18n()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.PERSONS.i18n()), 32));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.ITEMS.i18n()), 32));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.LOCATIONS.i18n()), 32));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.PLOTS.i18n()), 32));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.STRANDS.i18n()), 32));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.SCENE_RELATIVETIME.i18n()), 16));
		headers.add(new ColumnHeader(I18N.getMsg("scene.relative"), 16));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.SCENE_SUMMARY.i18n()), 32));
		headers.add(new ColumnHeader(I18N.getMsg(DATA.SCENE_XEDITOR.i18n()), 32));
		if (book.info.scenarioGet()) {
			headers.add(new ColumnHeader(I18N.getMsg(DATA.SCENARIO_STAGE.i18n()), 16));
			headers.add(new ColumnHeader(I18N.getMsg(DATA.SCENARIO_PITCH.i18n()), 16));
			headers.add(new ColumnHeader(I18N.getMsg(DATA.SCENARIO_LOC.i18n()), 16));
			headers.add(new ColumnHeader(I18N.getMsg(DATA.SCENARIO_MOMENT.i18n()), 16));
			headers.add(new ColumnHeader(I18N.getMsg(DATA.SCENARIO_START.i18n()), 16));
			headers.add(new ColumnHeader(I18N.getMsg(DATA.SCENARIO_END.i18n()), 16));
		}
		headers.add(new ColumnHeader(I18N.getMsg(DATA.DESCRIPTION.i18n()), 32));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NOTES.toString()), 32));
		writeHeaderColumn(headers);
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> entities = dao.findAll();
		for (Scene e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writeStrands() {
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg("strand.abbr"), 5));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NAME.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.COLOR.toString()), 8));
		headers.add(new ColumnHeader(I18N.getMsg(KW.SORT.toString()), 10));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NOTES.toString()), 32));
		writeHeaderColumn(headers);
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		StrandDAOImpl dao = new StrandDAOImpl(session);
		List<Strand> entities = dao.findAll();
		for (Strand e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writeTags() {
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NAME.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.CATEGORY.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.STATUS.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.DESCRIPTION.toString()), 32));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NOTES.toString()), 32));
		writeHeaderColumn(headers);
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		TagDAOImpl dao = new TagDAOImpl(session);
		List<Tag> entities = dao.findAll();
		for (Tag e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writeTaglinks() {
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg(KW.SCENE_START.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.SCENE_END.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.LOCATION.toString()), 32));
		headers.add(new ColumnHeader(I18N.getMsg(KW.PERSON.toString()), 32));
		writeHeaderColumn(headers);
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		TaglinkDAOImpl dao = new TaglinkDAOImpl(session);
		List<Taglink> entities = dao.findAll();
		for (Taglink e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writeEndnotes() {
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NAME.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg("endnote.type"), 8));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NUMBER.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NOTES.toString()), 32));
		writeHeaderColumn(headers);
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		EndnoteDAOImpl dao = new EndnoteDAOImpl(session);
		List<Endnote> entities = dao.findAll();
		for (Endnote e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	private void writeEvents() {
		List<ColumnHeader> headers = new ArrayList<>();
		headers.add(new ColumnHeader(I18N.getMsg(KW.ID.toString()), 5));
		headers.add(new ColumnHeader(I18N.getMsg(KW.TITLE.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg("timeevent.date"), 16));
		headers.add(new ColumnHeader(I18N.getMsg("timeevent.combo.label"), 8));
		headers.add(new ColumnHeader(I18N.getMsg(KW.CATEGORY.toString()), 16));
		headers.add(new ColumnHeader(I18N.getMsg(KW.NOTES.toString()), 32));
		writeHeaderColumn(headers);
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		EventDAOImpl dao = new EventDAOImpl(session);
		List<Event> entities = dao.findAll();
		for (Event e : entities) {
			switch (param.getFormat()) {
				case F_XML:
					writeText(e.toXml());
					break;
				case F_HTML:
					writeHtml(e.toHtml());
					break;
				case F_CSV:
					writeText(e.toCsv(param.getCsvQuote(), param.getCsvQuote(), param.getCsvComma()));
					break;
				case F_TXT:
					writeText(e.toText());
					break;
				default:
					break;
			}
		}
		model.commit();
	}

	public void writeDataBase() {
		String viewName = "DataBase";
		if (!openFile(viewName)) {
			return;
		}
		writeInternal();
		writeAttributes();
		writeCategories();
		writeGenders();
		writeItems();
		writeItemlinks();
		writeLocations();
		writePersons();
		writeTags();
		writeTaglinks();
		writeIdeas();
		writeMemos();
		writeRelations();
		writeEvents();
		writeStrands();
		writeParts();
		writeChapters();
		writeScenes();
		writeEndnotes();
		closeFile(VERBOSE);
	}

	private void writeHeaderColumn(List<ColumnHeader> headers) {
		//App.trace("ExportTable.writeHeaderColumn(headers="+headers.size()+" columns");
		if (headers == null || headers.isEmpty()) {
			return;
		}
		switch (param.getFormat()) {
			case F_HTML:
				writeText("<table border=\"1\" cellspacing=\"0\" cellpadding=\"0\">\n<tr>\n");
				for (ColumnHeader header : headers) {
					writeColumn(header.getName(), header.getSize());
				}
				writeText("</tr>\n");
				break;
			case F_XML: //no headers
				break;
			case F_CSV:
				for (ColumnHeader header : headers) {
					writeText(param.getCsvQuote() + header.getName() + param.getCsvQuote() + param.getCsvComma());
				}
				writeText("\n");
				break;
			case F_TXT:
				for (ColumnHeader header : headers) {
					writeText(header.getName() + param.getTxtSeparator());
				}
				writeText("\n");
				break;
			default:
				break;
		}
	}

	private void writeColumn(String name, int size) {
		switch (param.getFormat()) {
			case F_HTML:
				if (size > 0) {
					writeText("    <td width=\"" + size + "%\">" + ("".equals(name) ? "&nbsp" : name) + "</td>\n");
				} else {
					writeText("    <td>" + name + "</td>\n");
				}
				break;
			case F_CSV:
				writeText("\"" + name + "\";");
				break;
			case F_TXT:
				writeText("\t" + name + "");
				break;
			default:
				break;
		}
	}

	public class ColumnHeader {

		private final String name;
		private final int size;

		ColumnHeader(String n, int s) {
			name = n;
			size = s;
		}

		public String getName() {
			return (name);
		}

		public int getSize() {
			return (size);
		}

	}
}
