/*
 * Copyright (C) 2023 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.exim.importer;

import i18n.I18N;
import java.io.File;
import javax.swing.JOptionPane;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import storybook.model.hbn.entity.Endnote;
import storybook.toolkit.LOG;
import storybook.toolkit.xml.XmlFile;
import storybook.ui.MainFrame;

/**
 * import Comments in the current project
 *
 * @author favdb
 */
public class ImportComments {

	private static final String TT = "ImportComments";
	private final MainFrame mainFrame;
	private final File file;

	public static void exec(MainFrame mainFrame, File file) {
		LOG.trace(TT + ".exec(mainFrame, file=" + file.getAbsolutePath() + ")");
		ImportComments imp = new ImportComments(mainFrame, file);
		if (!imp.exec()) {
			JOptionPane.showMessageDialog(null,
					I18N.getMsg("import.msg.notok", file.getAbsolutePath()),
					I18N.getMsg("import"),
					JOptionPane.ERROR_MESSAGE);
		} else {
			JOptionPane.showMessageDialog(null,
					I18N.getMsg("import.msg.ok", file.getAbsolutePath()),
					I18N.getMsg("import"),
					JOptionPane.INFORMATION_MESSAGE);
		}
	}
	private XmlFile xml;

	public ImportComments(MainFrame mainFrame, File file) {
		this.mainFrame = mainFrame;
		this.file = file;
	}

	public boolean exec() {
		LOG.trace(TT + ".exec()");
		boolean rc = false;
		xml = new XmlFile(file.getAbsolutePath());
		if (xml != null && xml.openXml()) {
			NodeList nodes = xml.rootNode.getElementsByTagName("endnote");
			if (nodes.getLength() > 0) {
				rc = getComments(nodes);
			} else {
				LOG.err(TT + ".exec() error: No Xml nodes for Comments");
			}
			xml.closeXml();
		} else {
			LOG.err(TT + ".exec() error: xml is null");
		}
		return rc;
	}

	private boolean getComments(NodeList nodes) {
		LOG.trace(TT + ".getComments(nodes lenght=" + nodes.getLength() + ")");
		boolean rc = true;
		for (int i = 0; i < nodes.getLength(); i++) {
			addComment(nodes.item(i));
		}
		return rc;
	}

	private void addComment(Node node) {
		Endnote comment = Endnote.fromXml(mainFrame, node);
	}

}
