/*
 Storybook: Open Source software for novelists and authors.
 Copyright (C) 2008 - 2015 Martin Mustun, Pete Keller

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.model;

import i18n.I18N;
import java.awt.Component;
import java.awt.event.ActionListener;
import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import org.apache.commons.beanutils.BeanUtils;
import static org.apache.commons.beanutils.BeanUtils.copyProperties;
import org.apache.commons.beanutils.ConvertUtils;
import org.apache.commons.beanutils.converters.DateConverter;
import org.apache.commons.beanutils.converters.IntegerConverter;
import org.apache.commons.beanutils.converters.LongConverter;
import org.apache.commons.beanutils.converters.SqlTimestampConverter;
import org.apache.commons.beanutils.converters.StringConverter;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;
import storybook.App;
import storybook.action.ChapterOrderByTimestampAction;
import storybook.action.ChapterReSortAction;
import storybook.action.DeleteEntityAction;
import storybook.action.EditEntityAction;
import storybook.action.EditSceneXeditorAction;
import storybook.action.NewEntityAction;
import storybook.action.ShowInBookViewAction;
import storybook.action.ShowInChronoViewAction;
import storybook.action.ShowInManageViewAction;
import storybook.action.ShowInMemoriaAction;
import storybook.action.ShowInOSMAction;
import storybook.action.ShowInfoAction;
import storybook.ctrl.Ctrl;
import storybook.model.book.Book;
import storybook.model.handler.AbstractEntityHandler;
import storybook.model.handler.AttributeHandler;
import storybook.model.handler.CategoryHandler;
import storybook.model.handler.ChapterHandler;
import storybook.model.handler.EventHandler;
import storybook.model.handler.GenderHandler;
import storybook.model.handler.IdeaHandler;
import storybook.model.handler.InternalHandler;
import storybook.model.handler.ItemHandler;
import storybook.model.handler.ItemlinkHandler;
import storybook.model.handler.LocationHandler;
import storybook.model.handler.MemoHandler;
import storybook.model.handler.PartHandler;
import storybook.model.handler.PersonHandler;
import storybook.model.handler.PlotHandler;
import storybook.model.handler.RelationHandler;
import storybook.model.handler.SceneHandler;
import storybook.model.handler.StrandHandler;
import storybook.model.handler.TagHandler;
import storybook.model.handler.TaglinkHandler;
import storybook.model.hbn.dao.AttributeDAOImpl;
import storybook.model.hbn.dao.CategoryDAOImpl;
import storybook.model.hbn.dao.ChapterDAOImpl;
import storybook.model.hbn.dao.EndnoteDAOImpl;
import storybook.model.hbn.dao.EpisodeDAOImpl;
import storybook.model.hbn.dao.EventDAOImpl;
import storybook.model.hbn.dao.GenderDAOImpl;
import storybook.model.hbn.dao.IdeaDAOImpl;
import storybook.model.hbn.dao.InternalDAOImpl;
import storybook.model.hbn.dao.ItemDAOImpl;
import storybook.model.hbn.dao.ItemlinkDAOImpl;
import storybook.model.hbn.dao.LocationDAOImpl;
import storybook.model.hbn.dao.MemoDAOImpl;
import storybook.model.hbn.dao.PartDAOImpl;
import storybook.model.hbn.dao.PersonDAOImpl;
import storybook.model.hbn.dao.PlotDAOImpl;
import storybook.model.hbn.dao.RelationDAOImpl;
import storybook.model.hbn.dao.SceneDAOImpl;
import storybook.model.hbn.dao.StrandDAOImpl;
import storybook.model.hbn.dao.TagDAOImpl;
import storybook.model.hbn.dao.TaglinkDAOImpl;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.AbstractTag;
import storybook.model.hbn.entity.Attribute;
import storybook.model.hbn.entity.Category;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.Endnote;
import storybook.model.hbn.entity.Episode;
import storybook.model.hbn.entity.Event;
import storybook.model.hbn.entity.Gender;
import storybook.model.hbn.entity.Idea;
import storybook.model.hbn.entity.Internal;
import storybook.model.hbn.entity.Item;
import storybook.model.hbn.entity.Itemlink;
import storybook.model.hbn.entity.Location;
import storybook.model.hbn.entity.Memo;
import storybook.model.hbn.entity.Part;
import storybook.model.hbn.entity.Person;
import storybook.model.hbn.entity.Plot;
import storybook.model.hbn.entity.Relationship;
import storybook.model.hbn.entity.Scene;
import storybook.model.hbn.entity.Status;
import storybook.model.hbn.entity.Strand;
import storybook.model.hbn.entity.Tag;
import storybook.model.hbn.entity.Taglink;
import storybook.toolkit.DateUtil;
import storybook.toolkit.LOG;
import storybook.toolkit.ListUtil;
import storybook.toolkit.Period;
import storybook.toolkit.TextUtil;
import storybook.toolkit.comparator.ObjectComparator;
import storybook.toolkit.html.Html;
import storybook.toolkit.swing.js.JSLabel;
import storybook.ui.MainFrame;
import storybook.ui.renderer.lcr.LCREntity;

public class EntityUtil {

	private static final String TT = "EntityUtil",
			HTML_P_START = "<p style=\"padding-top:10px\">";

	public static Part getFirstPart(MainFrame mainFrame) {
		@SuppressWarnings("unchecked")
		List<Part> lst = findEntities(mainFrame, Book.TYPE.PART);
		return (lst.isEmpty() ? (Part) null : lst.get(0));
	}

	public static Strand getFirstStrand(MainFrame mainFrame) {
		@SuppressWarnings("unchecked")
		List<Strand> lst = findEntities(mainFrame, Book.TYPE.STRAND);
		return (lst.isEmpty() ? (Strand) null : lst.get(0));
	}

	public static void createEntity(MainFrame mainFrame, AbstractEntity entity) {
		Session session = mainFrame.getBookModel().beginTransaction();
		session.save(entity);
		session.getTransaction().commit();
		if (session.isOpen()) {
			session.close();
		}
		mainFrame.getBookModel().fireAgain();
	}

	/**
	 * get the last entity
	 *
	 * @param mainFrame
	 * @param type
	 * @return
	 */
	public static AbstractEntity getFirst(MainFrame mainFrame, Book.TYPE type) {
		List entities = findEntities(mainFrame, type);
		if (entities.isEmpty()) {
			return null;
		}
		return (AbstractEntity) entities.get(0);
	}

	/**
	 * get the last entity
	 *
	 * @param mainFrame
	 * @param type
	 * @return
	 */
	public static AbstractEntity getLast(MainFrame mainFrame, Book.TYPE type) {
		List entities = findEntities(mainFrame, type);
		if (entities.isEmpty()) {
			return null;
		}
		return (AbstractEntity) entities.get(entities.size() - 1);
	}

	public static void removeEntity(MainFrame mainFrame, Book.TYPE type, long l) {
		AbstractEntity entity = findEntity(mainFrame, type, l);
		mainFrame.getBookModel().deleteEntity(entity);
	}

	private EntityUtil() {
		// empty
	}

	public static boolean WITH_CHRONO = true;

	public static void convertPlainTextToHtml(MainFrame mainFrame) {
		boolean useHtmlScenes = mainFrame.getPref().editorGetUseHtml();
		boolean useHtmlDescr = false;
		if (!useHtmlScenes && !useHtmlDescr) {
			return;
		}
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();

		// scenes
		SceneDAOImpl sceneDao = new SceneDAOImpl(session);
		List<Scene> scenes = sceneDao.findAll();
		for (Scene scene : scenes) {
			if (useHtmlScenes) {
				scene.setSummary(Html.textToHTML(scene.getSummary()));
			}
			if (useHtmlDescr) {
				scene.setNotes(Html.textToHTML(scene.getNotes()));
			}
			session.update(scene);
		}

		// chapters
		ChapterDAOImpl chapterDao = new ChapterDAOImpl(session);
		List<Chapter> chapters = chapterDao.findAll();
		for (Chapter chapter : chapters) {
			if (useHtmlDescr) {
				chapter.setDescription(Html.textToHTML(chapter.getDescription()));
				chapter.setNotes(Html.textToHTML(chapter.getNotes()));
			}
		}

		// entities
		PersonDAOImpl personDao = new PersonDAOImpl(session);
		List<Person> persons = personDao.findAll();
		for (Person person : persons) {
			if (useHtmlDescr) {
				person.setDescription(Html.textToHTML(person.getDescription()));
				person.setNotes(Html.textToHTML(person.getNotes()));
			}
		}

		// locations
		LocationDAOImpl locationDao = new LocationDAOImpl(session);
		List<Location> locations = locationDao.findAll();
		for (Location location : locations) {
			if (useHtmlDescr) {
				location.setDescription(Html.textToHTML(location.getDescription()));
				location.setNotes(Html.textToHTML(location.getNotes()));
			}
		}

		// tags
		TagDAOImpl tagDao = new TagDAOImpl(session);
		List<Tag> tags = tagDao.findAll();
		for (Tag tag : tags) {
			if (useHtmlDescr) {
				tag.setDescription(Html.textToHTML(tag.getDescription()));
				tag.setNotes(Html.textToHTML(tag.getNotes()));
			}
		}

		// items
		ItemDAOImpl itemDao = new ItemDAOImpl(session);
		List<Item> items = itemDao.findAll();
		for (Item item : items) {
			if (useHtmlDescr) {
				item.setDescription(Html.textToHTML(item.getDescription()));
				item.setNotes(Html.textToHTML(item.getNotes()));
			}
		}

		// ideas
		IdeaDAOImpl ideaDao = new IdeaDAOImpl(session);
		List<Idea> ideas = ideaDao.findAll();
		for (Idea idea : ideas) {
			if (useHtmlDescr) {
				idea.setNotes(Html.textToHTML(idea.getNotes()));
			}
		}

		model.commit();

	}

	public static void convertHtmlToPlainText(MainFrame mainFrame) {
		boolean usePlainTextScenes = !mainFrame.getPref().editorGetUseHtml();
		boolean usePlainTextDescr = true;
		if (!usePlainTextScenes && !usePlainTextDescr) {
			return;
		}
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();

		// scenes
		SceneDAOImpl sceneDao = new SceneDAOImpl(session);
		List<Scene> scenes = sceneDao.findAll();
		for (Scene scene : scenes) {
			if (usePlainTextScenes) {
				scene.setSummary(Html.htmlToText(scene.getSummary(), true));
			}
			if (usePlainTextDescr) {
				scene.setNotes(Html.htmlToText(scene.getNotes()));
			}
			session.update(scene);
		}

		// chapters
		ChapterDAOImpl chapterDao = new ChapterDAOImpl(session);
		List<Chapter> chapters = chapterDao.findAll();
		for (Chapter chapter : chapters) {
			if (usePlainTextDescr) {
				chapter.setDescription(Html.htmlToText(chapter.getDescription(), true));
				chapter.setNotes(Html.htmlToText(chapter.getNotes(), true));
			}
		}

		// entities
		PersonDAOImpl personDao = new PersonDAOImpl(session);
		List<Person> persons = personDao.findAll();
		for (Person person : persons) {
			if (usePlainTextDescr) {
				person.setDescription(Html.htmlToText(person.getDescription(), true));
				person.setNotes(Html.htmlToText(person.getNotes(), true));
			}
		}

		// locations
		LocationDAOImpl locationDao = new LocationDAOImpl(session);
		List<Location> locations = locationDao.findAll();
		for (Location location : locations) {
			if (usePlainTextDescr) {
				location.setDescription(Html.htmlToText(location.getDescription(), true));
				location.setNotes(Html.htmlToText(location.getNotes(), true));
			}
		}

		// tags
		TagDAOImpl tagDao = new TagDAOImpl(session);
		List<Tag> tags = tagDao.findAll();
		for (Tag tag : tags) {
			if (usePlainTextDescr) {
				tag.setDescription(Html.htmlToText(tag.getDescription(), true));
				tag.setNotes(Html.htmlToText(tag.getNotes(), true));
			}
		}

		// items
		ItemDAOImpl itemDao = new ItemDAOImpl(session);
		List<Item> items = itemDao.findAll();
		for (Item item : items) {
			if (usePlainTextDescr) {
				item.setDescription(Html.htmlToText(item.getDescription(), true));
				item.setNotes(Html.htmlToText(item.getNotes(), true));
			}
		}

		// ideas
		IdeaDAOImpl ideaDao = new IdeaDAOImpl(session);
		List<Idea> ideas = ideaDao.findAll();
		for (Idea idea : ideas) {
			if (usePlainTextDescr) {
				idea.setNotes(Html.htmlToText(idea.getNotes(), true));
			}
		}

		model.commit();

	}

	public static List<Long> getReadOnlyIds(AbstractEntity entity) {
		ArrayList<Long> ret = new ArrayList<>();
		if (entity instanceof Category || entity instanceof Gender) {
			ret.add(1L);
			ret.add(2L);
		} else if (entity instanceof Part || entity instanceof Strand) {
			ret.add(1L);
		}
		return ret;
	}

	public static int getLastPartno(MainFrame mainFrame) {
		@SuppressWarnings("unchecked")
		List<Part> lst = findEntities(mainFrame, Book.TYPE.PART);
		int n = 0;
		for (Part scene : lst) {
			if ((scene.getNumber() != null) && (scene.getNumber() > n)) {
				n = scene.getNumber();
			}
		}
		return n;
	}

	public static int getLastChapterno(MainFrame mainFrame) {
		@SuppressWarnings("unchecked")
		List<Chapter> lst = findEntities(mainFrame, Book.TYPE.CHAPTER);
		int n = 0;
		for (Chapter scene : lst) {
			if ((scene.getChapterno() != null) && (scene.getChapterno() > n)) {
				n = scene.getChapterno();
			}
		}
		return n;
	}

	public static int getLastSceneno(MainFrame mainFrame) {
		@SuppressWarnings("unchecked")
		List<Scene> lst = findEntities(mainFrame, Book.TYPE.SCENE);
		int n = 0;
		for (Scene scene : lst) {
			if ((scene.getSceneno() != null) && (scene.getSceneno() > n)) {
				n = scene.getSceneno();
			}
		}
		return n;
	}

	public static Date findFirstDate(MainFrame mainFrame) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		Date date = dao.findFirstDate();
		model.commit();
		return date;
	}

	public static Date findLastDate(MainFrame mainFrame) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		Date date = dao.findLastDate();
		model.commit();
		return date;
	}

	public static void deleteTagAndItemlinks(Model model, AbstractEntity entity) {
		Session session = model.beginTransaction();
		TaglinkDAOImpl tagLinkDao = new TaglinkDAOImpl(session);
		List<Taglink> tagLinks = null;
		if (entity instanceof Scene) {
			tagLinks = tagLinkDao.findByStartOrEndScene((Scene) entity);
		} else if (entity instanceof Person) {
			tagLinks = tagLinkDao.findByPerson((Person) entity);
		} else if (entity instanceof Location) {
			tagLinks = tagLinkDao.findByLocation((Location) entity);
		}
		session.close();
		if (tagLinks != null && !tagLinks.isEmpty()) {
			for (Taglink link : tagLinks) {
				model.setTAGLINK_Delete(link);
			}
		}
		session = model.beginTransaction();
		ItemlinkDAOImpl itemLinkDao = new ItemlinkDAOImpl(session);
		List<Itemlink> itemLinks = null;
		if (entity instanceof Scene) {
			itemLinks = itemLinkDao.findByStartOrEndScene((Scene) entity);
		} else if (entity instanceof Person) {
			itemLinks = itemLinkDao.findByPerson((Person) entity);
		} else if (entity instanceof Location) {
			itemLinks = itemLinkDao.findByLocation((Location) entity);
		}
		session.close();
		if (itemLinks != null && !itemLinks.isEmpty()) {
			for (Itemlink link : itemLinks) {
				model.setITEMLINK_Delete(link);
			}
		}

	}

	public static void copyEntityProperties(MainFrame mainFrame, AbstractEntity entity, AbstractEntity newEntity) {
		try {
			if (entity == null) {
				LOG.err("EntityUtils.copyEntityProperties entity is NULL");
				return;
			}
			if (newEntity == null) {
				LOG.err("EntityUtils.copyEntityProperties newEntity is NULL");
				return;
			}
			ConvertUtils.register(new DateConverter(null), Date.class);
			ConvertUtils.register(new SqlTimestampConverter(null), Timestamp.class);
			ConvertUtils.register(new IntegerConverter(), Integer.class);
			ConvertUtils.register(new LongConverter(), Long.class);
			ConvertUtils.register(new StringConverter(), String.class);
			copyProperties(newEntity, entity);
		} catch (IllegalAccessException | InvocationTargetException e) {
			if (e.getMessage() == null) {
				return;
			}
			LOG.err(e.getMessage(), e);
			//LOG.error("EntityUtil.copyEntityProperties()\nException : ", e);
		}
	}

	public static AbstractEntity cloneEntity(MainFrame mainFrame, AbstractEntity entity) {
		if (entity != null) try {
			ConvertUtils.register(new DateConverter(null), Date.class);
			ConvertUtils.register(new SqlTimestampConverter(null), Timestamp.class);
			ConvertUtils.register(new IntegerConverter(), Integer.class);
			ConvertUtils.register(new StringConverter(), String.class);
			return (AbstractEntity) BeanUtils.cloneBean(entity);
		} catch (IllegalAccessException
				| InstantiationException
				| InvocationTargetException
				| NoSuchMethodException e) {
			LOG.err("EntityUtil.cloneEntityProperties() Exception : ", e);
		}
		return null;
	}

	public static void copyEntity(MainFrame mainFrame, AbstractEntity entity) {
		AbstractEntity newEntity = createNewEntity(entity);
		if (entity == null) {
			return;
		}
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		session.refresh(entity);
		copyEntityProperties(mainFrame, entity, newEntity);
		markCopiedEntity(mainFrame, newEntity);
		List<Plot> plots = new ArrayList<>();
		List<Item> items = new ArrayList<>();
		List<Location> locations = new ArrayList<>();
		List<Person> persons = new ArrayList<>();
		List<Strand> strands = new ArrayList<>();
		switch (Book.getTYPE(entity)) {
			case SCENE:
				//empty all links to others
				Scene scene = (Scene) entity;
				Scene newScene = (Scene) newEntity;
				if (newScene == null) {
					break;
				}
				if (!scene.hasRelativescene()) {
					newScene.removeRelativescene();
				}
				newScene.setPersons(persons);
				newScene.setLocations(locations);
				newScene.setItems(items);
				newScene.setPlots(plots);
				newScene.setStrands(strands);
				if (!scene.hasRelativescene()) {
					newScene.removeRelativescene();
				}
				break;
			default:
				break;
		}
		model.commit();
		Ctrl ctrl = mainFrame.getBookController();
		ctrl.newEntity(newEntity);
	}

	private static void markCopiedEntity(MainFrame mainFrame, AbstractEntity entity) {
		String copyStr = "(" + I18N.getMsg("copy") + ") ";
		switch (Book.getTYPE(entity)) {
			case SCENE:
				Scene s = (Scene) entity;
				s.setTitle(copyStr + s.getTitle());
				break;
			case CHAPTER:
				Chapter c = (Chapter) entity;
				c.setTitle(copyStr + c.getName());
				break;
			case ATTRIBUTE:
			case CATEGORY:
			case GENDER:
			case ITEM:
			case LOCATION:
			case PART:
			case PLOT:
			case RELATION:
			case STRAND:
			case TAG:
				AbstractTag t = (AbstractTag) entity;
				t.setName(copyStr + t.getName());
				break;
			case IDEA:
				Idea i = (Idea) entity;
				i.setName(copyStr + i.getName());
				i.setNotes(copyStr + i.getNotes());
				break;
			default:
				break;
		}
	}

	@SuppressWarnings("null")
	public static boolean hasHierarchyChanged(AbstractEntity oldEntity, AbstractEntity updEntity) {
		if (oldEntity == null || updEntity == null) {
			return false;
		}
		switch (Book.getTYPE(oldEntity)) {
			case IDEA:
				return !Objects.equals(((Idea) oldEntity).getStatus(), ((Idea) updEntity).getStatus());
			case SCENE:
				Scene oldC = (Scene) oldEntity;
				Scene updC = (Scene) updEntity;
				if (oldC.getChapter() == null && updC.getChapter() != null) {
					return true;
				}
				if (oldC.getChapter() != null && updC.getChapter() == null) {
					return true;
				}
				return oldC.getChapter() != null && updC.getChapter() != null
						&& (!Objects.equals(oldC.getChapter().getId(), updC.getChapter().getId()));
			case PERSON:
				Person oldP = (Person) oldEntity;
				Person updP = (Person) updEntity;
				if (!Objects.equals(oldP.getCategory(), updP.getCategory())) {
					return true;
				}
				return !Objects.equals(oldP.getGender(), updP.getGender());
			case RELATION:
				return false;
			case LOCATION:
				Location oldL = (Location) oldEntity;
				Location updL = (Location) updEntity;
				Location oldSite = (Location) oldL.getSite();
				Location updSite = (Location) updL.getSite();
				String oldCity = oldL.getCity();
				String updCity = updL.getCity();
				String oldCountry = oldL.getCountry();
				String updCountry = updL.getCountry();
				if (oldSite == null && updSite != null) {
					return true;
				}
				if (oldSite != null) {
					if (updSite == null) {
						return true;
					}
					if (!oldSite.equals(updSite)) {
						return true;
					}
				}
				if (oldCity == null && updCity != null) {
					return true;
				}
				if (oldCity != null) {
					if (updCity == null) {
						return true;
					}
					if (!oldCity.equals(updCity)) {
						return true;
					}
				}
				if (oldCountry == null && updCountry != null) {
					return true;
				}
				if (oldCountry != null && updCountry == null) {
					return true;

				} else {
					return !oldCountry.equals(updCountry);
				}
			case ITEM:
			case TAG:
				AbstractTag oldT = (AbstractTag) oldEntity;
				AbstractTag updT = (AbstractTag) updEntity;
				if (oldT.getCategory() == null && updT.getCategory() == null) {
					return false;
				}
				if (oldT.getCategory() != null || updT.getCategory() != null) {
					return true;
				}
				return !oldT.getCategory().equals(updT.getCategory());
			default:
				break;
		}
		return false;
	}

	public static JPopupMenu createPopupMenu(MainFrame mainFrame, AbstractEntity entity, boolean withChrono) {
		JPopupMenu menu = new JPopupMenu();
		if (entity == null) {
			return null;
		}
		if (entity.isTransient()) {
			return null;
		}
		if (Book.getTYPE(entity) == Book.TYPE.ENDNOTE) {
			menu.add(new EditEntityAction(mainFrame, entity, false));
			return menu;
		}
		menu.add(new EditEntityAction(mainFrame, entity, false));
		if (entity instanceof Scene && mainFrame.getBook().isUseXeditor()) {
			Scene scene = (Scene) entity;
			boolean b = new File(scene.getOdf()).exists();
			EditSceneXeditorAction item = new EditSceneXeditorAction(mainFrame, entity);
			item.setEnabled(b);
			menu.add(item);
		}
		menu.add(new JPopupMenu.Separator());
		if (entity instanceof Scene || entity instanceof Chapter) {
			if (withChrono) {
				menu.add(new ShowInChronoViewAction(mainFrame, entity));
			}
			menu.add(new ShowInBookViewAction(mainFrame, entity));
			menu.add(new ShowInManageViewAction(mainFrame, entity));
		}
		menu.add(new ShowInfoAction(mainFrame, entity));
		if (isAvailableInMemoria(entity)) {
			menu.add(new ShowInMemoriaAction(mainFrame, entity));
		}
		menu.add(new JPopupMenu.Separator());
		if (entity instanceof Chapter) {
			menu.add(new ChapterOrderByTimestampAction(mainFrame, (Chapter) entity));
			menu.add(new ChapterReSortAction(mainFrame, (Chapter) entity));
			menu.add(new JPopupMenu.Separator());
		}

		menu.add(new NewEntityAction(mainFrame, entity));
		menu.add(new DeleteEntityAction(mainFrame, entity));

		if (entity instanceof Location) {
			menu.add(new JPopupMenu.Separator());
			menu.add(new ShowInOSMAction((Location) entity));
		}
		if (menu.getComponents().length == 0) {
			return null;
		}
		for (Component comp : menu.getComponents()) {
			if (comp instanceof JMenuItem) {
				((JMenuItem) comp).setFont(App.getInstance().fontGetDefault());
			}
		}
		return menu;
	}

	public static boolean isAvailableInMemoria(AbstractEntity entity) {
		return entity instanceof Person
				|| entity instanceof Plot
				|| entity instanceof Location
				|| entity instanceof Scene
				|| entity instanceof Tag
				|| entity instanceof Item;
	}

	public static List<Attribute> getEntityAttributes(MainFrame mainFrame, AbstractEntity entity) {
		if (entity.isTransient()) {
			return new ArrayList<>();
		}
		if (entity instanceof Person) {
			Model model = mainFrame.getBookModel();
			Session session = model.beginTransaction();
			Person person = (Person) entity;
			session.refresh(person);
			List<Attribute> ret = person.getAttributes();
			model.commit();
			return ret;
		}
		return new ArrayList<>();
	}

	public static List<String> getListAttributes(MainFrame mainFrame) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		AttributeDAOImpl dao = new AttributeDAOImpl(session);
		List<String> lst = dao.findKeys();
		model.commit();
		return lst;
	}

	public static void setEntityAttributes(MainFrame mainFrame, AbstractEntity entity, List<Attribute> attributes) {
		if (entity.isTransient()) {
			return;
		}
		if (entity instanceof Person) {
			try {
				Person person = (Person) entity;
				Model model = mainFrame.getBookModel();
				// delete attributes
				Session session = model.beginTransaction();
				person.setAttributes(new ArrayList<>());
				session.saveOrUpdate(person);
				// delete orphans
				// add attributes
				for (Attribute property : attributes) {
					session.save(property);
				}
				model.commit();
				// update person
				person.setAttributes(attributes);
				mainFrame.getBookController().updateEntity(person);
			} catch (HibernateException e) {
				LOG.err("EntityUtil.copyEntityProperties()", e);
			}
		}
	}

	public static AbstractEntityHandler getEntityHandler(MainFrame mainFrame, AbstractEntity entity) {
		switch (Book.getTYPE(entity)) {
			case ATTRIBUTE:
				return new AttributeHandler(mainFrame);
			case CATEGORY:
				return new CategoryHandler(mainFrame);
			case CHAPTER:
				return new ChapterHandler(mainFrame);
			case EVENT:
				return new EventHandler(mainFrame);
			case GENDER:
				return new GenderHandler(mainFrame);
			case IDEA:
				return new IdeaHandler(mainFrame);
			case INTERNAL:
				return new InternalHandler(mainFrame);
			case ITEM:
				return new ItemHandler(mainFrame);
			case ITEMLINK:
				return new ItemlinkHandler(mainFrame);
			case LOCATION:
				return new LocationHandler(mainFrame);
			case MEMO:
				return new MemoHandler(mainFrame);
			case PART:
				return new PartHandler(mainFrame);
			case PERSON:
				return new PersonHandler(mainFrame);
			//case PHOTO: return new PhotoHandler(mainFrame);
			case PLOT:
				return new PlotHandler(mainFrame);
			case RELATION:
				return new RelationHandler(mainFrame);
			case SCENARIO:
			/*if (mainFrame.getBook().isScenario()) {
					return new ScenarioHandler(mainFrame);
				}*/
			case SCENE:
				return new SceneHandler(mainFrame);
			case STRAND:
				return new StrandHandler(mainFrame);
			case TAG:
				return new TagHandler(mainFrame);
			case TAGLINK:
				return new TaglinkHandler(mainFrame);
			default:
				return null;
		}
	}

	public static Class<?> getEntityClass(AbstractEntity entity) {
		//App.trace("entity class=" + entity.getClass().getSimpleName());
		return getEntityClass(Book.getTYPE(entity));
	}

	public static Class getEntityClass(Book.TYPE type) {
		//App.trace("entity class=" + entity.getClass().getSimpleName());
		switch (type) {
			case ATTRIBUTE:
				return Attribute.class;
			case CATEGORY:
				return Category.class;
			case CHAPTER:
				return Chapter.class;
			case EPISODE:
				return Episode.class;
			case EVENT:
				return Event.class;
			case GENDER:
				return Gender.class;
			case IDEA:
				return Idea.class;
			case INTERNAL:
				return Internal.class;
			case ITEM:
				return Item.class;
			case ITEMLINK:
				return Itemlink.class;
			case LOCATION:
				return Location.class;
			case MEMO:
				return Memo.class;
			case PART:
				return Part.class;
			case PERSON:
				return Person.class;
			case PLOT:
				return Plot.class;
			case RELATION:
				return Relationship.class;
			case SCENE:
				return Scene.class;
			case STRAND:
				return Strand.class;
			case TAG:
				return Tag.class;
			case TAGLINK:
				return Taglink.class;
			default:
				return null;
		}
	}

	public static List<JCheckBox> createCategoryCheckBoxes(MainFrame mainFrame, ActionListener comp) {
		List<JCheckBox> list = new ArrayList<>();
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		CategoryDAOImpl dao = new CategoryDAOImpl(session);
		List<Category> categories = dao.findAllOrderBySort();
		model.commit();
		for (Category category : categories) {
			JCheckBox cb = new JCheckBox(category.getName());
			cb.putClientProperty("CbCategory", category);
			cb.setOpaque(false);
			cb.addActionListener(comp);
			cb.setSelected(true);
			list.add(cb);
		}
		return list;
	}

	public static List<JCheckBox> createCountryCheckBoxes(MainFrame mainFrame, ActionListener comp) {
		List<JCheckBox> list = new ArrayList<>();
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		LocationDAOImpl dao = new LocationDAOImpl(session);
		List<String> countries = dao.findCountries();
		model.commit();
		for (String country : countries) {
			JCheckBox chb = new JCheckBox(country);
			chb.setOpaque(false);
			chb.addActionListener(comp);
			chb.setSelected(true);
			list.add(chb);
		}
		return list;
	}

	public static List<JCheckBox> createItemCategoryCheckBoxes(MainFrame mainFrame, ActionListener comp) {
		List<JCheckBox> list = new ArrayList<>();
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		ItemDAOImpl dao = new ItemDAOImpl(session);
		List<String> categories = dao.findCategories();
		model.commit();
		for (String category : categories) {
			JCheckBox chb = new JCheckBox(category);
			chb.setOpaque(false);
			chb.addActionListener(comp);
			chb.setSelected(true);
			list.add(chb);
		}
		return list;
	}

	public static List<JCheckBox> createPersonCheckBoxes(MainFrame mainFrame, List<JCheckBox> cbl, ActionListener comp) {
		List<JCheckBox> list = new ArrayList<>();
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		PersonDAOImpl dao = new PersonDAOImpl(session);
		for (JCheckBox cb : cbl) {
			if (cb.isSelected()) {
				Category category = (Category) cb.getClientProperty("CbCategory");
				List<Person> persons = dao.findByCategory(category);
				for (Person person : persons) {
					JCheckBox cbPerson = new JCheckBox(person.getFullNameAbbr());
					cbPerson.setOpaque(false);
					cbPerson.putClientProperty("CbPerson", person);
					cbPerson.addActionListener(comp);
					list.add(cbPerson);
				}
			}
		}
		model.commit();
		return list;
	}

	public static List<JCheckBox> createItemCheckBoxes(MainFrame m, ActionListener comp) {
		List<JCheckBox> list = new ArrayList<>();
		Model model = m.getBookModel();
		Session session = model.beginTransaction();
		ItemDAOImpl dao = new ItemDAOImpl(session);
		List<Item> items = dao.findAll();
		for (Item item : items) {
			JCheckBox cbItem = new JCheckBox(item.getName());
			cbItem.setOpaque(false);
			cbItem.putClientProperty("CbItem", item);
			cbItem.addActionListener(comp);
			list.add(cbItem);
		}
		model.commit();
		return list;
	}

	public static Scene createScene(Strand strand, Chapter chapter) {
		Scene scene = new Scene();
		scene.setStrand(strand);
		scene.setStatus(1);
		scene.setChapter(chapter);
		scene.setSceneno(1);
		scene.setDate(new Date());
		scene.setTitle(I18N.getMsg("scene") + " 1");
		scene.setSummary("<p></p>");
		scene.setNotes("");
		return scene;
	}

	public static void abandonEntityChanges(MainFrame mainFrame, AbstractEntity entity) {
		try {
			if (entity.isTransient()) {
				return;
			}
			Model model = mainFrame.getBookModel();
			Session session = model.getSession();
			if (session != null && session.isOpen()) {
				Transaction transaction = session.beginTransaction();
				session.refresh(entity);
				transaction.commit();
			}
		} catch (HibernateException e) {
			LOG.err("EntityUtil.abandonEntityChanges(...)", e);
		}
	}

	public static String getToolTip(AbstractEntity entity) {
		return getToolTip(entity, null, false);
	}

	public static String getToolTip(AbstractEntity entity, Date date, boolean withDay) {
		StringBuilder buf = new StringBuilder();
		buf.append(Html.HTML_B);
		buf.append(Html.HEAD_B)
				.append(Html.STYLE_B).append(Html.FONT_SIZE_DEFAULT).append(Html.STYLE_E)
				.append(Html.HEAD_E);
		buf.append(Html.BODY_B);
		if (date != null) {
			buf.append(Html.intoP(
					Html.intoB(
							Html.intoI(DateUtil.timeToString(date, false)))));
		}
		if (entity.getName().trim().isEmpty()) {
			buf.append(Html.intoH(2, I18N.getMsg(entity.getObjType().toString())));
		} else {
			buf.append("<table width=\"300\">");
			buf.append(Html.TR_B).append(Html.TD_B);
			buf.append(Html.intoB(I18N.getColonMsg(entity.getObjType().toString())))
					.append(" ")
					.append(entity.getName());
			buf.append(Html.BR);
			switch (Book.getTYPE(entity)) {
				case LOCATION:
					getToolTipLocation(buf, (Location) entity);
					break;
				case PERSON:
					getToolTipPerson(buf, (Person) entity, date);
					break;
				case PLOT:
					getToolTipPlot(buf, (Plot) entity);
					break;
				case RELATION:
					getToolTipRelation(buf, (Relationship) entity);
					break;
				case SCENE:
					getToolTipScene(buf, (Scene) entity, withDay);
					break;
				case ITEM:
				case TAG:
					getToolTipTag(buf, (AbstractTag) entity);
					break;
				case STRAND:
					getToolTipStrand(buf, (Strand) entity);
					break;
				default:
					break;
			}
			buf.append(Html.TD_E)
					.append(Html.TR_E)
					.append(Html.TABLE_E);
		}
		buf.append(Html.BODY_E);
		buf.append(Html.HTML_E);
		return buf.toString();
	}

	private static void getToolTipLocation(StringBuilder buf, Location location) {
		if (location == null) {
			return;
		}
		if (!location.getCity().isEmpty()) {
			buf.append(I18N.getColonMsg("location.city"));
			buf.append(" ");
			buf.append(location.getCity());
			buf.append(Html.BR);
		}
		if (!location.getCity().isEmpty()) {
			buf.append(I18N.getColonMsg("location.country"));
			buf.append(" ");
			buf.append(location.getCountry());
		}
		buf.append(Html.intoP(TextUtil.truncateText(location.getDescription()), "\"margin-top:5px\""));
	}

	private static void getToolTipPerson(StringBuilder buf, Person person, Date date) {
		if (person == null) {
			return;
		}
		if (date != null && person.getBirthday() != null) {
			buf.append(I18N.getColonMsg("person.age"));
			buf.append(" ");
			buf.append(person.calculateAge(date));
			if (person.isDead(date)) {
				buf.append("+");
			}
			buf.append(Html.BR);
		}
		if (person.getGender() != null) {
			buf.append(I18N.getColonMsg("manage.persons.gender"));
			buf.append(" ");
			buf.append(person.getGender());
			buf.append(Html.BR);
		}
		if (person.getCategory() != null) {
			buf.append(I18N.getColonMsg("manage.persons.category"));
			buf.append(" ");
			buf.append(person.getCategory());
			buf.append(Html.BR);
		}
		if (person.getDescription() != null && !person.getDescription().isEmpty()) {
			buf.append(Html.intoP(TextUtil.truncateText(person.getDescription()), "'margin-top:5px'"));
		}
	}

	private static void getToolTipPlot(StringBuilder buf, Plot plot) {
		if (plot == null) {
			return;
		}
		if (plot.getCategory() != null && !plot.getCategory().isEmpty()) {
			buf.append(Html.intoB(I18N.getMsg("category"))).append(": ")
					.append(plot.getCategory()).append(Html.BR);
		}
		if (plot.getDescription() != null && !plot.getDescription().isEmpty()) {
			buf.append(plot.getDescription());
		}
	}

	private static void getToolTipRelation(StringBuilder buf, Relationship relation) {
		List<Person> persons = relation.getPersons();
		if (persons != null && !persons.isEmpty()) {
			buf.append(Html.P_B);
			buf.append(Html.intoB(Html.intoI(I18N.getColonMsg("persons")))).append(" ");
			for (Person p : persons) {
				buf.append(p.getAbbr()).append(" ");
			}
			buf.append(Html.P_E);
		}
		List<Item> items = relation.getItems();
		if (items != null && !items.isEmpty()) {
			buf.append(Html.P_B);
			buf.append(Html.intoB(Html.intoI(I18N.getColonMsg("items")))).append(" ");
			for (Item p : items) {
				buf.append(p.getName()).append(" ");
			}
			buf.append(Html.P_E);
		}
		List<Location> locations = relation.getLocations();
		if (locations != null && !locations.isEmpty()) {
			buf.append(Html.P_B);
			buf.append(Html.intoB(Html.intoI(I18N.getColonMsg("locations")))).append(" ");
			for (Location p : locations) {
				buf.append(p.getName()).append(" ");
			}
			buf.append(Html.P_E);
		}
	}

	private static void getToolTipScene(StringBuilder buf, Scene scene, boolean withDay) {
		buf.append(Html.P_B).append(Html.intoB(I18N.getColonMsg("chapter") + " "));
		buf.append((scene.getChapter() == null ? "" : scene.getChapter().getName()));
		buf.append(Html.P_E);
		if (scene.getSummary() != null && !scene.getSummary().isEmpty()) {
			buf.append(scene.getSummary(80));
		}
	}

	private static void getToolTipStrand(StringBuilder buf, Strand strand) {
		//buf.append(strand.getName());
	}

	private static void getToolTipTag(StringBuilder buf, AbstractTag tag) {
		if (tag.getDescription() != null && !tag.getDescription().isEmpty()) {
			buf.append(TextUtil.truncateText(tag.getDescription()));
		}
	}

	public static String getDeleteInfo(MainFrame mainFrame, AbstractEntity entity) {
		StringBuilder buf = new StringBuilder();
		boolean warnings = addDeletionInfo(mainFrame, entity, buf);
		if (warnings) {
			buf.append(Html.getHr());
		}
		Model model = mainFrame.getBookModel();
		Session session = model.getSession();
		session.beginTransaction();
		session.refresh(entity);
		buf.append(entity.toDetail(false));
		model.commit();
		return buf.toString();
	}

	private static boolean addDeletionInfo(MainFrame mainFrame, AbstractEntity entity, StringBuilder buf) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		boolean warnings = false;
		switch (entity.getObjType()) {
			case CATEGORY: {
				Category category = (Category) entity;
				List<Person> persons = new CategoryDAOImpl(session).findPersons(category);
				if (!persons.isEmpty()) {
					buf.append(HTML_P_START);
					buf.append(Html.getWarning(I18N.getMsg("category.delete.warning")));
					buf.append(Html.P_E);
					buf.append(Html.BR);
					buf.append(I18N.getColonMsg("persons"));
					buf.append(Html.BR);
					buf.append(Html.UL_B);
					for (Person person : persons) {
						buf.append(Html.intoLI(person.toString()));
					}
					buf.append(Html.UL_E);
					warnings = true;
				}
				break;
			}
			case CHAPTER: {
				Chapter chapter = (Chapter) entity;
				List<Scene> scenes = new ChapterDAOImpl(session).findScenes(chapter);
				if (!scenes.isEmpty()) {
					buf.append(HTML_P_START);
					buf.append(Html.getWarning(I18N
							.getMsg("warning.chapter.has.assigned.scenes")));
					buf.append(Html.P_E);
					buf.append(Html.BR);
					buf.append(I18N.getColonMsg("scenes"));
					buf.append(Html.BR);
					buf.append(Html.UL_B);
					for (Scene scene : scenes) {
						buf.append(Html.intoLI(scene.toString()));
					}
					buf.append(Html.UL_E);
					warnings = true;
				}
				break;
			}
			case GENDER: {
				Gender gender = (Gender) entity;
				List<Person> persons = new GenderDAOImpl(session).findPersons(gender);
				if (!persons.isEmpty()) {
					buf.append(HTML_P_START);
					buf.append(Html.getWarning(I18N.getMsg("gender.delete.warning")));
					buf.append(Html.P_E);
					buf.append(Html.BR);
					buf.append(I18N.getColonMsg("persons"));
					buf.append(Html.BR);
					buf.append(Html.UL_B);
					for (Person person : persons) {
						buf.append(Html.intoLI(person.toString()));
					}
					buf.append(Html.UL_E);
					warnings = true;
				}
				break;
			}
			case ITEM: {
				Item item = (Item) entity;
				List<Itemlink> links = new ItemlinkDAOImpl(session).findByItem(item);
				if (!links.isEmpty()) {
					buf.append(HTML_P_START);
					buf.append(Html.getWarning(I18N.getMsg("itemlink.warning.delete.all")));
					buf.append(Html.P_E);
					buf.append(Html.BR);
					buf.append(I18N.getColonMsg("itemlink"));
					buf.append(Html.BR);
					buf.append(Html.UL_B);
					for (Itemlink link : links) {
						buf.append(Html.intoLI(link.toString()));
					}
					buf.append(Html.UL_E);
					warnings = true;
				}
				break;
			}
			case PART: {
				Part part = (Part) entity;
				List<Chapter> chapters = new PartDAOImpl(session).findChapters(part);
				if (!chapters.isEmpty()) {
					buf.append(HTML_P_START);
					buf.append(Html.getWarning(I18N.getMsg("manage.parts.delete.warning")));
					buf.append(Html.P_E);
					buf.append(Html.BR);
					buf.append(I18N.getColonMsg("chapters"));
					buf.append(Html.BR);
					buf.append(Html.UL_B);
					for (Chapter chapter : chapters) {
						buf.append(Html.intoLI(chapter.toString()));
					}
					buf.append(Html.UL_E);
					warnings = true;
				}
				break;
			}
			case SCENE: {
				Scene scene = (Scene) entity;
				List<Scene> scenes = new SceneDAOImpl(session).findScenesWithRelativeSceneId(scene);
				if (!scenes.isEmpty()) {
					buf.append(HTML_P_START);
					buf.append(Html.getWarning(I18N.getMsg("scene.relativedate.delete.warning")));
					buf.append(Html.P_E);
					buf.append(Html.BR);
					buf.append(I18N.getColonMsg("scenes"));
					buf.append(Html.BR);
					buf.append(Html.UL_B);
					for (Scene scene2 : scenes) {
						buf.append(Html.intoLI(scene2.toString()));
					}
					buf.append(Html.UL_E);
					warnings = true;
				}
				break;
			}
			case STRAND: {
				Strand strand = (Strand) entity;
				List<Episode> episodes = new EpisodeDAOImpl(session).findByStrand(strand);
				if (!episodes.isEmpty()) {
					buf.append(HTML_P_START);
					buf.append(Html.getWarning(I18N.getMsg("manage.episodes.delete.warning")));
					buf.append(Html.P_E);
					warnings = true;
				}
				List<Scene> scenes = new StrandDAOImpl(session).findScenes(strand);
				if (!scenes.isEmpty()) {
					buf.append(HTML_P_START);
					buf.append(Html.getWarning(I18N.getMsg("manage.strands.delete.warning")));
					buf.append(Html.P_E);
					buf.append(Html.BR);
					buf.append(I18N.getColonMsg("scenes"));
					buf.append(Html.BR);
					buf.append(Html.UL_B);
					for (Scene scene : scenes) {
						buf.append(Html.intoLI(scene.toString()));
					}
					buf.append(Html.UL_E);
					warnings = true;
				}
				break;
			}
			case TAG: {
				Tag tag = (Tag) entity;
				List<Taglink> links = new TaglinkDAOImpl(session).findByTag(tag);
				if (!links.isEmpty()) {
					buf.append(HTML_P_START);
					buf.append(Html.getWarning(I18N.getMsg("taglink.warning.delete.all")));
					buf.append(Html.P_E);
					buf.append(Html.BR);
					buf.append(I18N.getColonMsg("taglink"));
					buf.append(Html.BR);
					buf.append(Html.UL_B);
					for (Taglink link : links) {
						buf.append(Html.intoLI(link.toString()));
					}
					buf.append(Html.UL_E);
					warnings = true;
				}
				break;
			}
			default:
				break;
		}
		model.commit();
		return warnings;
	}

	public static AbstractEntity get(MainFrame mainFrame, Class<? extends AbstractEntity> c, Long entityId) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		AbstractEntity entity = (AbstractEntity) session.get(c, entityId);
		model.commit();
		refresh(mainFrame, entity);
		return entity;
	}

	public static void refresh(MainFrame mainFrame, AbstractEntity entity) {
		if (entity == null) {
			return;
		}
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		session.refresh(entity);
		model.commit();
	}

	@SuppressWarnings("unchecked")
	public static void fillCB(MainFrame mainFrame, JComboBox cb,
			Book.TYPE type, AbstractEntity toSel, boolean isNew, boolean addEmptyItem) {
		cb.removeAllItems();
		cb.setRenderer(new LCREntity());
		if (addEmptyItem) {
			cb.addItem("");
		}
		List<AbstractEntity> entities = EntityUtil.findEntities(mainFrame, type);
		entities.forEach(entity -> cb.addItem(entity));
		if (toSel != null) {
			cb.setSelectedItem(toSel);
		}
		cb.revalidate();
	}

	public static String getEntityFullTitle(AbstractEntity entity) {
		StringBuilder buf = new StringBuilder();
		buf.append("<span ")
				.append(getCSSTitle1())
				.append(">\n")
				.append(getEntityTitle(entity))
				.append("</span>\n");
		if (entity != null && !entity.isTransient()) {
			buf.append("&nbsp;&nbsp;")
					.append("<span ")
					.append(getCSSTitle2())
					.append(">\n")
					.append(entity.toString())
					.append("</span>");
		}
		return buf.toString();
	}

	public static JSLabel getEntityIconLabel(AbstractEntity entity) {
		return new JSLabel(getEntityIcon(entity));
	}

	public static Icon getEntityIcon(AbstractEntity entity) {
		if (entity == null) {
			return new ImageIcon();
		}
		return (entity.getIcon());
	}

	public static String getTypeName(AbstractEntity entity) {
		return (entity.getClass().getName().replace("storybook.model.hbn.entity.", ""));
	}

	private static String getCSSTitle1() {
		return getCSSTitle1("");
	}

	private static String getCSSTitle1(String styles) {
		return "style=\"font-weight:bold;" + Html.FONT_SIZE + ":12px;" + styles + "\"";
	}

	private static String getCSSTitle2() {
		return getCSSTitle2("");
	}

	private static String getCSSTitle2(String styles) {
		return "style=\"font-weight:bold;" + Html.FONT_SIZE + ":10px;" + styles + "\"";
	}

	public static String getEntityTitle(AbstractEntity entity) {
		return getEntityTitle(entity, null);
	}

	public static String getEntityTitle(AbstractEntity entity, Boolean setIsTransient) {
		if (entity == null) {
			return ("");
		}
		boolean isTransient = entity.isTransient();
		if (setIsTransient != null) {
			isTransient = setIsTransient;
		}
		String str = entity.getObjType() + (isTransient ? ".new" : "");
		return (I18N.getMsg(str));
	}

	public static String getClassName(AbstractEntity entity) {
		return entity.getClass().getSimpleName().toLowerCase();
	}

	public static List<String> findCategories(MainFrame mainFrame, Book.TYPE type) {
		List<String> list = new ArrayList<>();
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		switch (type) {
			case EVENT:
				EventDAOImpl eventDAO = new EventDAOImpl(session);
				return (eventDAO.findCategories());
			case IDEA:
				IdeaDAOImpl ideaDAO = new IdeaDAOImpl(session);
				return (ideaDAO.findCategories());
			case ITEM:
				ItemDAOImpl itemDAO = new ItemDAOImpl(session);
				return (itemDAO.findCategories());
			case PLOT:
				PlotDAOImpl plotDAO = new PlotDAOImpl(session);
				return (plotDAO.findCategories());
			case TAG:
				TagDAOImpl tagDAO = new TagDAOImpl(session);
				return (tagDAO.findCategories());
			default:
				break;
		}
		return (list);
	}

	public static AbstractEntity findEntity(MainFrame mainFrame, Book.TYPE key, Long id) {
		List list = findEntities(mainFrame, key);
		if (list != null && !list.isEmpty()) {
			for (Object obj : list) {
				AbstractEntity entity = (AbstractEntity) obj;
				if (entity.getId().equals(id)) {
					return (entity);
				}
			}
		}
		return (null);
	}

	public static List<Strand> findStrands(MainFrame mainFrame) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		List<Strand> list = new StrandDAOImpl(session).findAll();
		model.commit();
		return list;
	}

	public static List<Part> findParts(MainFrame mainFrame) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		List<Part> list = new PartDAOImpl(session).findAll();
		model.commit();
		return list;
	}

	public static List<Chapter> findChapters(MainFrame mainFrame, Part part) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		List<Chapter> list = new ChapterDAOImpl(session).findAll(part);
		model.commit();
		return list;
	}

	public static Strand getDefaultStrand(MainFrame mainFrame) {
		return (Strand) findEntities(mainFrame, Book.TYPE.STRAND).get(0);
	}

	public static List findEntities(MainFrame mainFrame, Book.TYPE key) {
		return (findEntities(mainFrame, key.toString()));
	}

	@SuppressWarnings("unchecked")
	public static List findEntities(MainFrame mainFrame, String key) {
		//App.trace(TT + ".findEntities(mainFrame, key=" + key + ")");
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		List list = new ArrayList();
		switch (Book.getTYPE(key)) {
			case ATTRIBUTE:
			case ATTRIBUTES:
				list = new AttributeDAOImpl(session).findAll();
				break;
			case CATEGORY:
			case CATEGORIES:
				list = new CategoryDAOImpl(session).findAll();
				break;
			case CHAPTER:
			case CHAPTERS:
				list = new ChapterDAOImpl(session).findAll();
				break;
			case EPISODE:
			case EPISODES:
				list = new EpisodeDAOImpl(session).findAll();
				break;
			case ENDNOTE:
			case ENDNOTES:
				list = new EndnoteDAOImpl(session).findAll();
				break;
			case EVENT:
			case EVENTS:
				list = new EventDAOImpl(session).findAll();
				break;
			case GENDER:
			case GENDERS:
				list = new GenderDAOImpl(session).findAll();
				break;
			case IDEA:
			case IDEAS:
				list = new IdeaDAOImpl(session).findAll();
				break;
			case INTERNAL:
			case INTERNALS:
				list = new InternalDAOImpl(session).findAll();
				break;
			case ITEM:
			case ITEMS:
				list = new ItemDAOImpl(session).findAll();
				break;
			case ITEMLINK:
			case ITEMLINKS:
				list = new ItemlinkDAOImpl(session).findAll();
				break;
			case LOCATION:
			case LOCATIONS:
				list = new LocationDAOImpl(session).findAll();
				break;
			case MEMO:
			case MEMOS:
				list = new MemoDAOImpl(session).findAll();
				break;
			case PART:
			case PARTS:
				list = new PartDAOImpl(session).findAll();
				break;
			case PERSON:
			case PERSONS:
				list = new PersonDAOImpl(session).findAll();
				break;
			case PLOT:
			case PLOTS:
				list = new PlotDAOImpl(session).findAll();
				break;
			case RELATION:
			case RELATIONS:
				list = new RelationDAOImpl(session).findAll();
				break;
			case SCENE:
			case SCENES:
				list = new SceneDAOImpl(session).findAll();
				break;
			case STRAND:
			case STRANDS:
				list = new StrandDAOImpl(session).findAll();
				break;
			case TAG:
			case TAGS:
				list = new TagDAOImpl(session).findAll();
				break;
			case TAGLINK:
			case TAGLINKS:
				list = new TaglinkDAOImpl(session).findAll();
				break;
			default:
				LOG.log("** EntityUtil.findEntities unable to find " + key);
		}
		model.commit();
		return (list);
	}

	public static AbstractEntity findEntityByName(MainFrame mainFrame, String key, String name) {
		List list = findEntities(mainFrame, key);
		if (list != null) {
			for (Object entity : list) {
				if (((AbstractEntity) entity).getName().equals(name)) {
					return ((AbstractEntity) entity);
				}
			}
		}
		return (null);
	}

	public static AbstractEntity findEntityByName(MainFrame mainFrame, Book.TYPE key, String name) {
		List list = findEntities(mainFrame, key);
		if (list != null) {
			for (Object entity : list) {
				if (((AbstractEntity) entity).getName().equals(name)) {
					return ((AbstractEntity) entity);
				}
			}
		}
		return (null);
	}

	public static Attribute findAttribute(MainFrame mainFrame, String key, String val) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		AttributeDAOImpl dao = new AttributeDAOImpl(session);
		List<Attribute> attributes = dao.findAll();
		model.commit();
		for (Attribute attribute : attributes) {
			if (key.equals(attribute.getKey()) && val.equals(attribute.getValue())) {
				return (attribute);
			}
		}
		return (null);
	}

	public static Category findCategory(MainFrame mainFrame, String sel) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		CategoryDAOImpl dao = new CategoryDAOImpl(session);
		List<Category> categories = dao.findAll();
		model.commit();
		for (Category c : categories) {
			if (sel.equals(c.getName())) {
				return (c);
			}
		}
		return (null);
	}

	public static List<String> findEventCategories(MainFrame mainFrame) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		EventDAOImpl dao = new EventDAOImpl(session);
		List<String> categories = dao.findCategories();
		model.commit();
		return (categories);
	}

	public static List<Scene> findScenes(MainFrame mainFrame, Part part) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes = dao.findByPart(part);
		model.commit();
		return (scenes);
	}

	public static List<Scene> findScenes(MainFrame mainFrame) {
		List<Scene> scenes = new ArrayList<>();
		if (mainFrame == null) {
			return scenes;
		}
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		scenes = dao.findAll();
		model.commit();
		return scenes;
	}

	public static List<Scene> findScenes(MainFrame mainFrame, Chapter chapter) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes;
		if (chapter == null) {
			scenes = dao.findAll();
		} else {
			scenes = dao.findByChapter(chapter);
		}
		model.commit();
		return (scenes);
	}

	public static List<Scene> findScenes(MainFrame mainFrame, Item item) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes = dao.findByItem(item);
		model.commit();
		return (scenes);
	}

	public static List<Scene> findScenes(MainFrame mainFrame, Location location) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes = dao.findByLocationLink(location);
		model.commit();
		return (scenes);
	}

	public static List<Scene> findScenes(MainFrame mainFrame, Person person) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes = dao.findByPerson(person);
		model.commit();
		return (scenes);
	}

	public static List<Scene> findScenes(MainFrame mainFrame, Plot plot) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes = dao.findByPlot(plot);
		model.commit();
		return (scenes);
	}

	public static List<Scene> findScenes(MainFrame mainFrame, Strand strand) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes = dao.findByStrands(strand);
		scenes.addAll(dao.findByStrand(strand));
		model.commit();
		return (scenes);
	}

	@SuppressWarnings("unchecked")
	public static List<Scene> findScenes(MainFrame mainFrame, Relationship relation) {
		List<Scene> ret = new ArrayList<>();
		if (relation.hasStartScene()) {
			ret.add(relation.getStartScene());
		}
		if (relation.hasEndScene()) {
			ret.add(relation.getEndScene());
		}
		return ret;
	}

	public static List<Scene> findScenes(MainFrame mainFrame, Itemlink itemlink) {
		List<Scene> scenes = new ArrayList<>();
		Date date1 = null;
		if (itemlink.hasStartScene()) {
			date1 = itemlink.getStartScene().getDate();
		}
		Date date2 = null;
		if (itemlink.hasEndScene()) {
			date2 = itemlink.getEndScene().getDate();
		}
		if (date1 != null && date2 != null) {
			Model model = mainFrame.getBookModel();
			Session session = model.beginTransaction();
			SceneDAOImpl dao = new SceneDAOImpl(session);
			scenes = dao.findByPeriod(new Period(date1, date2));
			model.commit();
		}
		return (scenes);
	}

	public static List<Scene> findScenes(MainFrame mainFrame, Scene start, Scene end) {
		List<Scene> ret = new ArrayList<>();
		if (start == null) {
			return (ret);
		}
		List<Scene> scenes = findScenes(mainFrame);
		if (end == null) {
			int n = scenes.indexOf(start);
			if (n > 0) {
				ret.add(scenes.get(n - 1));
			}
			if (n < scenes.size() - 1) {
				ret.add(scenes.get(n + 1));
			}
			return ret;
		}
		return (scenes.subList(scenes.indexOf(start), scenes.indexOf(end)));
	}

	public static List<Scene> findScenes(MainFrame mainFrame, Tag tag) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		TaglinkDAOImpl dao = new TaglinkDAOImpl(session);
		List<Taglink> taglinks = dao.findByTag(tag);
		SceneDAOImpl daoScene = new SceneDAOImpl(session);
		List<Scene> scenes = new ArrayList<>();
		Date date1 = null, date2 = null;
		for (Taglink tl : taglinks) {
			if (tl.hasStartScene() && tl.getStartScene().hasScenets()) {
				date1 = tl.getStartScene().getDate();
			}
			if (tl.hasEndScene() && tl.getEndScene().hasScenets()) {
				date2 = tl.getEndScene().getDate();
			}
			if (date1 != null && date2 != null) {
				List<Scene> s = daoScene.findByDate(date1, date2);
				scenes.addAll(s);
			}
		}
		model.commit();
		return (scenes);
	}

	public static List<Scene> findScenes(MainFrame mainFrame, Taglink taglink) {
		List<Scene> scenes = new ArrayList<>();
		if (taglink.hasStartScene() && taglink.hasEndScene()) {
			Date date1 = taglink.getStartScene().getDate();
			Date date2 = taglink.getEndScene().getDate();
			if (date1 != null && date2 != null) {
				Model model = mainFrame.getBookModel();
				Session session = model.beginTransaction();
				SceneDAOImpl dao = new SceneDAOImpl(session);
				scenes = dao.findByPeriod(new Period(date1, date2));
				model.commit();
			}
		}
		return (scenes);
	}

	public static Person findPersonByName(MainFrame mainFrame, String name) {
		List list = findEntities(mainFrame, Book.TYPE.PERSON);
		if (list != null) {
			for (Object entity : list) {
				String s = ((Person) entity).getFirstname() + " " + ((Person) entity).getLastname();
				if (s.equals(name)) {
					return ((Person) entity);
				}
			}
		}
		return (null);
	}

	public static List<Relationship> findRelation(MainFrame mainFrame, AbstractEntity entity) {
		switch (Book.getTYPE(entity)) {
			case ITEM:
				return (findRelation(mainFrame, (Item) entity));
			case LOCATION:
				return (findRelation(mainFrame, (Location) entity));
			case PERSON:
				return (findRelation(mainFrame, (Person) entity));
			case SCENE:
				return (findRelation(mainFrame, (Scene) entity));
			default:
				break;
		}
		return (new ArrayList<>());
	}

	public static List<Relationship> findRelation(MainFrame mainFrame, Item item) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		RelationDAOImpl dao = new RelationDAOImpl(session);
		List<Relationship> links = dao.findByItems(item);
		model.commit();
		return (links);
	}

	public static List<Relationship> findRelation(MainFrame mainFrame, Location location) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		RelationDAOImpl dao = new RelationDAOImpl(session);
		List<Relationship> links = dao.findByLocations(location);
		model.commit();
		return (links);
	}

	public static List<Relationship> findRelation(MainFrame mainFrame, Person person) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		RelationDAOImpl dao = new RelationDAOImpl(session);
		List<Relationship> r = dao.findByPersons(person);
		model.commit();
		return r;
	}

	public static List<Relationship> findRelation(MainFrame mainFrame, Scene scene) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		RelationDAOImpl dao = new RelationDAOImpl(session);
		List<Relationship> links = dao.findByStartOrEndScene(scene);
		model.commit();
		return (links);
	}

	public static List<Itemlink> findItemlink(MainFrame mainFrame, Item item) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		ItemlinkDAOImpl dao = new ItemlinkDAOImpl(session);
		List<Itemlink> links = dao.findByItem(item);
		model.commit();
		return (links);
	}

	public static List<Itemlink> findItemlink(MainFrame mainFrame, Location item) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		ItemlinkDAOImpl dao = new ItemlinkDAOImpl(session);
		List<Itemlink> links = dao.findByLocation(item);
		model.commit();
		return (links);
	}

	public static List<Itemlink> findItemlink(MainFrame mainFrame, Person person) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		ItemlinkDAOImpl dao = new ItemlinkDAOImpl(session);
		List<Itemlink> links = dao.findByPerson(person);
		model.commit();
		return (links);
	}

	public static List<Itemlink> findItemlink(MainFrame mainFrame, Scene scene) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		ItemlinkDAOImpl dao = new ItemlinkDAOImpl(session);
		List<Itemlink> links = dao.findByScene(scene);
		model.commit();
		return (links);
	}

	public static List<Taglink> findTaglink(MainFrame mainFrame, Tag item) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		TaglinkDAOImpl dao = new TaglinkDAOImpl(session);
		List<Taglink> links = dao.findByTag(item);
		model.commit();
		return (links);
	}

	public static List<Taglink> findTaglink(MainFrame mainFrame, Location item) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		TaglinkDAOImpl dao = new TaglinkDAOImpl(session);
		List<Taglink> links = dao.findByLocation(item);
		model.commit();
		return (links);
	}

	public static List<Taglink> findTaglink(MainFrame mainFrame, Person person) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		TaglinkDAOImpl dao = new TaglinkDAOImpl(session);
		List<Taglink> links = dao.findByPerson(person);
		model.commit();
		return (links);
	}

	public static List<Taglink> findTaglink(MainFrame mainFrame, Scene scene) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		TaglinkDAOImpl dao = new TaglinkDAOImpl(session);
		List<Taglink> links = dao.findByScene(scene);
		model.commit();
		return (links);
	}

	public static AbstractEntity getSceneById(MainFrame mainFrame, Long id) {
		@SuppressWarnings("unchecked")
		List<Scene> scenes = findEntities(mainFrame, Book.TYPE.SCENE);
		for (Scene s : scenes) {
			if (s.getId().equals(id)) {
				return (s);
			}
		}
		return (null);
	}

	public static String getNames(List<AbstractEntity> entities) {
		if (entities == null) {
			return ("");
		}
		List<String> list = new ArrayList<>();
		for (AbstractEntity a : entities) {
			list.add(a.toString());
		}
		return (ListUtil.join(list, ","));
	}

	public static String getIds(List<AbstractEntity> entities) {
		if (entities == null) {
			return ("");
		}
		List<String> list = new ArrayList<>();
		for (AbstractEntity a : entities) {
			list.add(a.getId().toString());
		}
		return (ListUtil.join(list, ","));
	}

	public static AbstractEntity getEntityNext(MainFrame mainFrame, AbstractEntity entity) {
		List entities = EntityUtil.findEntities(mainFrame, entity.getObjType());
		if (entities != null) {
			for (int i = 0; i < entities.size(); i++) {
				AbstractEntity e = (AbstractEntity) entities.get(i);
				if (entity.getId().equals(e.getId())) {
					if (i > entities.size() - 2) {
						return (null);
					}
					return ((AbstractEntity) entities.get(i + 1));
				}
			}
		}
		return (null);
	}

	public static AbstractEntity getEntityPrior(MainFrame mainFrame, AbstractEntity entity) {
		if (entity == null) {
			return (null);
		}
		List entities = EntityUtil.findEntities(mainFrame, entity.getObjType());
		if (entities != null) {
			for (int i = 0; i < entities.size(); i++) {
				AbstractEntity e = (AbstractEntity) entities.get(i);
				if (entity.getId().equals(e.getId())) {
					if (i < 1) {
						return (null);
					}
					return ((AbstractEntity) entities.get(i - 1));
				}
			}
		}
		return (null);
	}

	public static Scene getScene(MainFrame m, Long id) {
		Model model = m.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes = dao.findAll();
		model.commit();
		for (Scene scene : scenes) {
			if (scene.getId().equals(id)) {
				return (scene);
			}
		}
		return (null);
	}

	public static List findLocations(MainFrame mainFrame, Chapter chapter) {
		List<Location> locations = new ArrayList<>();
		List<Scene> scenes = findScenes(mainFrame, chapter);
		for (Scene s : scenes) {
			for (Location l : s.getLocations()) {
				locations.add(l);
			}
		}
		return ListUtil.setUnique(locations);
	}

	@SuppressWarnings("unchecked")
	public static List<Date> findDates(MainFrame mainFrame, Chapter chapter) {
		List<Date> dates = new ArrayList<>();
		List<Scene> scenes = findScenes(mainFrame, chapter);
		for (Scene s : scenes) {
			dates.add(s.getDate());
		}
		dates = ListUtil.setUnique(dates);
		Collections.sort(dates, new ObjectComparator());
		return dates;
	}

	public static AbstractEntity createNewEntity(AbstractEntity entity) {
		AbstractEntity n = null;
		switch (Book.getTYPE(entity)) {
			case ATTRIBUTE:
				n = new Attribute();
				break;
			case CATEGORY:
				n = new Category();
				break;
			case CHAPTER:
				n = new Chapter();
				break;
			case ENDNOTE:
				n = new Endnote();
				break;
			case EVENT:
				n = new Event();
				break;
			case GENDER:
				n = new Gender();
				break;
			case IDEA:
				n = new Idea();
				break;
			case ITEM:
				n = new Item();
				break;
			case ITEMLINK:
				n = new Itemlink();
				break;
			case LOCATION:
				n = new Location();
				break;
			case MEMO:
				n = new Memo();
				break;
			case PART:
				n = new Part();
				break;
			case PERSON:
				n = new Person();
				break;
			case PLOT:
				n = new Plot();
				break;
			case RELATION:
				n = new Relationship();
				break;
			case SCENE:
				n = new Scene();
				break;
			case TAG:
				n = new Tag();
				break;
			case TAGLINK:
				n = new Taglink();
				break;
			default:
				break;
		}
		return (n);
	}

	public static AbstractEntity createNewEntity(Book.TYPE type) {
		AbstractEntity n = null;
		switch (type) {
			case ATTRIBUTE:
				n = new Attribute();
				break;
			case CATEGORY:
				n = new Category();
				break;
			case CHAPTER:
				n = new Chapter();
				break;
			case ENDNOTE:
				n = new Endnote();
				break;
			case EVENT:
				n = new Event();
				break;
			case GENDER:
				n = new Gender();
				break;
			case IDEA:
				n = new Idea();
				break;
			case ITEM:
				n = new Item();
				break;
			case ITEMLINK:
				n = new Itemlink();
				break;
			case LOCATION:
				n = new Location();
				break;
			case MEMO:
				n = new Memo();
				break;
			case PART:
				n = new Part();
				break;
			case PERSON:
				n = new Person();
				break;
			case PLOT:
				n = new Plot();
				break;
			case RELATION:
				n = new Relationship();
				break;
			case SCENE:
				n = new Scene();
				break;
			case STRAND:
				n = new Strand();
				break;
			case STATUS:
				n = new Status();
				break;
			case TAG:
				n = new Tag();
				break;
			case TAGLINK:
				n = new Taglink();
				break;
			default:
				break;
		}
		return n;
	}

	public static boolean checkEndnotes(Endnote endnote) {
		@SuppressWarnings("unchecked")
		Scene scene = endnote.getScene();
		if (scene == null) {
			return (false);
		}
		boolean b = endnote.isInText();
		/*App.trace("checkEndnote link="+endnote.getLinkTo()
				+ ", endnote=" + endnote.toString()
				+ ", scene=" + scene.toString()
				+", contains="+(b?"YES":"NO"));*/
		return (b);
	}

	@SuppressWarnings("unchecked")
	public static String getListId(List<?> entities) {
		if (entities == null || entities.isEmpty()) {
			return ("");
		}
		List<String> list = new ArrayList<>();
		for (AbstractEntity e : (List<AbstractEntity>) entities) {
			if (e.getId() != -1L) {
				list.add(e.getId().toString());
			}
		}
		return (String.join(",", list));
	}

	@SuppressWarnings("unchecked")
	public static String getListName(List<?> entities) {
		//App.trace("AbstractEntity.getListName(entities nb="+(entities==null?"null":entities.size())+")");
		if (entities == null || entities.isEmpty()) {
			return ("");
		}
		List<String> list = new ArrayList<>();
		for (AbstractEntity e : (List<AbstractEntity>) entities) {
			list.add(e.getName());
		}
		return (ListUtil.join(list));
	}

	public static String getNextIdeaNumber(MainFrame mainFrame) {
		List entities = findEntities(mainFrame, Book.TYPE.IDEA);
		Long id = 0L;
		for (Object entity : entities) {
			Idea idea = (Idea) entity;
			if (idea.getId() > id) {
				id = idea.getId();
			}
		}
		id++;
		return (id.toString());
	}

	public static List<Book.TYPE> getBookTypes() {
		List<Book.TYPE> list = new ArrayList<>();
		list.add(Book.TYPE.ATTRIBUTE);
		list.add(Book.TYPE.CHAPTER);
		list.add(Book.TYPE.CATEGORY);
		list.add(Book.TYPE.ENDNOTE);
		list.add(Book.TYPE.EVENT);
		list.add(Book.TYPE.GENDER);
		list.add(Book.TYPE.IDEA);
		list.add(Book.TYPE.INTERNAL);
		list.add(Book.TYPE.ITEM);
		list.add(Book.TYPE.ITEMLINK);
		list.add(Book.TYPE.LOCATION);
		list.add(Book.TYPE.MEMO);
		list.add(Book.TYPE.PART);
		list.add(Book.TYPE.PERSON);
		list.add(Book.TYPE.PLOT);
		list.add(Book.TYPE.RELATION);
		list.add(Book.TYPE.SCENE);
		list.add(Book.TYPE.STRAND);
		list.add(Book.TYPE.TAG);
		list.add(Book.TYPE.TAGLINK);
		return list;
	}

}
