/*
Storybook: Open Source software for novelists and authors.
Copyright (C) 2008 - 2012 Martin Mustun

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.model.hbn.dao;

import java.util.ArrayList;
import java.util.List;
import javax.persistence.NonUniqueResultException;
import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.Endnote;
import storybook.model.hbn.entity.Scene;
import storybook.ui.dialog.ExceptionDlg;

public class EndnoteDAOImpl extends SbGenericDAOImpl<Endnote, Long> implements EndnoteDAO {

	public EndnoteDAOImpl() {
		super();
	}

	public EndnoteDAOImpl(Session session) {
		super(session);
	}

	@Override
	public List<Endnote> findAll() {
		StringBuilder buf = new StringBuilder("from Endnote as s");
		buf.append(" order by number");
		Query query = session.createQuery(buf.toString());
		@SuppressWarnings("unchecked")
		List<Endnote> ret = (List<Endnote>) query.list();
		return ret;
	}

	public List<Endnote> findAllBySort(Integer type) {
		StringBuilder buf = new StringBuilder("from Endnote as s");
		buf.append(" where s.type=").append(type.toString());
		buf.append(" order by sort");
		Query query = session.createQuery(buf.toString());
		@SuppressWarnings("unchecked")
		List<Endnote> ret = (List<Endnote>) query.list();
		return ret;
	}

	@SuppressWarnings("unchecked")
	public List<Endnote> findAll(Scene scene) {
		StringBuilder buf = new StringBuilder("select Endnote from Endnote as s");
		if (scene != null) {
			buf.append(" where s.scene=:scene");
		}
		buf.append(" order by number");
		Query query = session.createQuery(buf.toString());
		if (scene != null) {
			query.setEntity(DAOutil.SCENE, scene);
		}
		List<Endnote> ret = (List<Endnote>) query.list();
		return ret;
	}

	@SuppressWarnings("unchecked")
	public List<Scene> findScenes(Scene scene) {
		List<Scene> scenes = new ArrayList<>();
		if (session.isOpen()) {
			Criteria crit = session.createCriteria(Scene.class);
			if (scene != null) {
				crit.add(Restrictions.eq(DAOutil.SCENE, scene));
			} else {
				crit.add(Restrictions.isNull(DAOutil.SCENE));
			}
			crit.addOrder(Order.asc(DAOutil.NUMBER));
			scenes = (List<Scene>) crit.list();
		}
		return scenes;
	}

	public int getNextNumber() {
		return getMaxNumber() + 1;
	}

	public int getMaxNumber() {
		Query query = session.createQuery("select max(number) from Endnote");
		if (query.uniqueResult() == null) {
			return 0;
		}
		return (Integer) query.uniqueResult();
	}

	@SuppressWarnings("unchecked")
	public boolean checkIfNumberExists(AbstractEntity entity) {
		try {
			Endnote newEndnote = (Endnote) entity;
			Integer newNumber = newEndnote.getNumber();

			if (!entity.isTransient()) {
				// update
				EndnoteDAOImpl dao = new EndnoteDAOImpl(session);
				Endnote oldEndnote = dao.find(entity.getId());
				Integer oldNumber = oldEndnote.getNumber();
				Criteria crit = session.createCriteria(Chapter.class);
				crit.add(Restrictions.eq(DAOutil.NUMBER, newNumber));
				List<Endnote> entities = (List<Endnote>) crit.list();
				List<Integer> numbers = new ArrayList<>();
				for (Endnote e : entities) {
					numbers.add(e.getNumber());
				}
				if (newNumber.equals(oldNumber)) {
					numbers.remove(newNumber);
				}
				return numbers.isEmpty();
			}

			// new
			Criteria crit = session.createCriteria(Endnote.class);
			crit.add(Restrictions.eq(DAOutil.NUMBER, newNumber));
			List<Endnote> entities = (List<Endnote>) crit.list();
			return entities.size() <= 0;
		} catch (NonUniqueResultException e) {
			ExceptionDlg.show(this.getClass().getName(), e);
			return true;
		}
	}

}
