/*
Storybook: Open Source software for novelists and authors.
Copyright (C) 2008 - 2012 Martin Mustun

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.model.hbn.entity;

import i18n.I18N;
import java.util.ArrayList;
import java.util.List;
import org.hibernate.Session;
import org.w3c.dom.Node;
import shef.ShefEditor;
import storybook.App;
import storybook.ctrl.Ctrl;
import storybook.exim.exporter.ExportBookToHtml;
import storybook.model.EntityUtil;
import storybook.model.Model;
import storybook.model.book.Book;
import storybook.model.hbn.dao.DAOutil;
import storybook.model.hbn.dao.EndnoteDAOImpl;
import storybook.toolkit.LOG;
import storybook.toolkit.StringUtil;
import storybook.toolkit.html.Html;
import storybook.ui.MainFrame;

public class Endnote extends AbstractEntity {

	public static final String TT = "Endnote";

	public static final String LIB_TYPE[] = {"endnote", "comment"};

	public static List find(MainFrame mainFrame, int type) {
		//LOG.trace(TT + ".find(mainFrame, type=" + type + ")");
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		List<Endnote> list = new EndnoteDAOImpl(session).findAllBySort(type);
		model.commit();
		return list;
	}

	public static List find(MainFrame mainFrame, int type, Scene scene) {
		List list = find(mainFrame, type);
		if (scene == null) {
			return list;
		}
		List<Endnote> endnotes = new ArrayList<>();
		for (Object obj : list) {
			Endnote en = (Endnote) obj;
			if (en.getType() == type
					&& scene.equals(en.getScene())) {
				endnotes.add(en);
			}
		}
		return endnotes;
	}

	public static Endnote find(MainFrame mainFrame, int type, Scene scene, int number) {
		for (Object obj : EntityUtil.findEntities(mainFrame, Book.TYPE.ENDNOTE)) {
			Endnote en = (Endnote) obj;
			if (en.getType() == type
					&& scene.equals(en.getScene())
					&& number == en.getNumber()) {
				return en;
			}
		}
		return null;
	}

	public static void removeEndnotes(MainFrame mainFrame, int type, Scene scene) {
		//LOG.trace(TT + ".removeEndnotes(mainFrame, scene=" + LOG.traceEntity(scene) + ")");
		Model model = mainFrame.getBookModel();
		for (Object obj : find(mainFrame, type, scene)) {
			model.deleteEntity((Endnote) obj);
		}
	}

	public static boolean renumber(MainFrame mainFrame, Integer type) {
		//LOG.trace(TT + ".renumber(mainFrame)");
		mainFrame.cursorSetWaiting();
		boolean rc = false;
		Ctrl ctrl = mainFrame.getBookController();
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		List<Endnote> endnotes = new EndnoteDAOImpl(session).findAllBySort(type);
		model.commit();
		int num = 1;
		List<Endnote> toRemove = new ArrayList<>();
		for (Endnote en : endnotes) {
			Scene scene = en.getScene();
			if (scene != null) {
				if (!scene.getSummary().contains(en.getLinkToEndnote(""))) {
					mainFrame.getBookModel().deleteEntity(en);
					continue;
				}
				if (en.getNumber() != num) {
					String oldLink = en.getLinkToEndnote("");
					en.setNumber(num);
					String newLink = en.getLinkToEndnote("");
					LOG.trace("replacing '" + oldLink + "' by '" + newLink + "'");
					String text = scene.getSummary().replace(oldLink, newLink);
					scene.setSummary(text);
					ctrl.updateEntity(scene);
					ctrl.updateEntity(en);
					rc = true;
				}
				num++;
			} else {
				toRemove.add(en);
			}
		}
		if (!toRemove.isEmpty()) {
			for (Endnote en : toRemove) {
				ctrl.deleteEntity(en);
			}
		}
		mainFrame.cursorSetDefault();
		return rc;
	}

	public static Endnote createEndnote(MainFrame mainFrame, int type,
			Scene scene, ShefEditor htTexte) {
		Endnote en;
		int num = 1;
		for (Object nt : find(mainFrame, type, scene)) {
			num = Math.max(num, ((Endnote) nt).getNumber() + 1);
		}
		en = new Endnote(type, scene, num);
		en.setSort(htTexte.wysEditorGet().getEditor().getCaretPosition());
		if (mainFrame.showEditorAsDialog(en)) {
			return null;
		}
		en = Endnote.find(mainFrame, type, scene, num);
		String link = linkTo("", en);
		htTexte.insertText(link);
		return en;
	}

	public static void removeAll(MainFrame mainFrame, Integer type) {
		LOG.trace(TT + ".removeAll(mainFrame, type=" + type);
		Ctrl ctrl = mainFrame.getBookController();
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		List<Endnote> endnotes = new EndnoteDAOImpl(session).findAllBySort(type);
		for (Endnote en : endnotes) {
			session.delete(en);
		}
		model.commit();
		mainFrame.setUpdated();
	}

	private Integer number = -1;
	private Scene scene;
	private String sort = "";
	private Integer type = 0;

	public Endnote() {
		super(Book.TYPE.ENDNOTE, "010");
		setNumber(-1);
		setScene(null);
	}

	public Endnote(int type, Scene scene, Integer number) {
		this();
		setType(type);
		setNumber(number);
		setScene(scene);
	}

	public Endnote(int type, Scene scene) {
		this();
		setType(type);
		setScene(scene);
	}

	public Endnote(Integer number, Scene scene, String notes) {
		this();
		setNumber(number);
		setScene(scene);
		setNotes(notes);
	}

	public Endnote(int type, Scene scene, Integer number, String notes) {
		this();
		setType(type);
		setScene(scene);
		setNumber(number);
		setNotes(notes);
	}

	public Endnote(int type, Scene scene, Integer number, String notes, int index) {
		this();
		setNumber(number);
		setScene(scene);
		setNotes(notes);
		setType(type);
		setSort(index);
	}

	public Integer getType() {
		return getMinMax(type, 0, 1);
	}

	public String getTypeLib() {
		return I18N.getMsg("endnote.type_" + LIB_TYPE[getType()]);
	}

	public void setType(Integer number) {
		this.type = (number == null ? 0 : number);
	}

	public Integer getNumber() {
		return number;
	}

	public void setNumber(Integer number) {
		this.number = (number == null ? -1 : number);
		setName(String.format("%s %02d", getTypeLib(), number));
	}

	public Scene getScene() {
		return scene;
	}

	public void setScene(Scene entity) {
		this.scene = entity;
	}

	public String getSort() {
		return sort;
	}

	public void setSort(String value) {
		this.sort = value;
	}

	/**
	 * compute default sort value format is: pp.cc.ss.ttttt where: - pp is part number, 99 if not -
	 * cc is chapter number, 99 if not - ss is scene number, mandatory - ttttt is text index of the
	 * endnote
	 *
	 * @param loc
	 */
	public void setSort(int loc) {
		int nchapter = 99, npart = 99;
		if (getScene().hasChapter()) {
			nchapter = getScene().getChapter().getChapterno();
			if (getScene().getChapter().hasPart()) {
				npart = getScene().getChapter().getPart().getNumber();
			}
		}
		this.sort = String.format("%02d.%02d.%03d.%05d",
				npart,
				nchapter,
				scene.getSceneno(),
				loc);
	}

	@Override
	public String toDetail(boolean detailed) {
		StringBuilder b = new StringBuilder();
		b.append(super.toHtmlDetailHeader(detailed));
		b.append(getInfo("endnote.type", getType() + " (" + getTypeLib() + ")"));
		b.append(getInfo(DAOutil.NUMBER, getClean(getNumber())));
		b.append(getInfo(DAOutil.SCENE, getClean(getScene())));
		if (App.isDev()) {
			b.append(getInfo("sort", getClean(getSort())));
		}
		b.append(super.toHtmlDetailFooter(detailed));
		return b.toString();
	}

	@Override
	public String toCsv(String quoteStart, String quoteEnd, String separator) {
		StringBuilder b = new StringBuilder();
		b.append(quoteStart).append(getClean(id)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getName())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(type)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(number)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(scene)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getNotes())).append(quoteStart).append("\n");
		return b.toString();
	}

	@Override
	public String toHtml() {
		return toCsv("<td>", "</td>", "\n");
	}

	@Override
	public String toText() {
		return toCsv("", "", "\t");
	}

	@Override
	public String toXml() {
		StringBuilder b = new StringBuilder();
		b.append(toXmlTab(1)).append("<").append(getObjType()).append(" \n");
		b.append(xmlCommon());
		b.append(xmlAttribute("type", getClean(type)));
		b.append(xmlAttribute(DAOutil.NUMBER, getClean(number)));
		b.append(xmlAttribute(DAOutil.SCENE, getClean(scene)));
		b.append(toXmlTab(2)).append(">\n");
		b.append(this.toXmlMeta(2, "notes", getClean(getNotes()), true));
		b.append(toXmlTab(1)).append("</").append(getObjType()).append(">\n");
		return b.toString();
	}

	public static Endnote fromXml(MainFrame mainFrame, Node node) {
		Endnote p = new Endnote();
		p.setId(getXmlLong(node, "id"));
		p.setType(getXmlInteger(node, "type"));
		p.setNumber(getXmlInteger(node, DAOutil.NUMBER));
		p.setSort(getXmlString(node, "sort"));
		p.setScene((Scene) EntityUtil.findEntity(mainFrame, Book.TYPE.SCENE, getXmlLong(node, DAOutil.SCENE)));
		p.setNotes(getXmlText(node, "notes"));
		return p;
	}

	@Override
	@SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
	public boolean equals(Object obj) {
		if (!super.equals(obj)) {
			return false;
		}
		Endnote test = (Endnote) obj;
		boolean ret = true;
		ret = ret && equalsIntegerNullValue(getNumber(), test.getNumber());
		ret = ret && equalsObjectNullValue(getScene(), test.getScene());
		ret = ret && equalsStringNullValue(getNotes(), test.getNotes());
		return ret;
	}

	@Override
	public int hashCode() {
		return hashPlus(super.hashCode(), number, scene, sort);
	}

	public static Integer getNextNumber(List endnotes) {
		int n = 0;
		for (Object c : endnotes) {
			if (((Endnote) c).number > n) {
				n = ((Endnote) c).number;
			}
		}
		return (n + 1);
	}

	public static int findFirst(MainFrame mainFrame, Scene scene, int type) {
		List endnotes = Endnote.find(mainFrame, type, scene);
		if (!endnotes.isEmpty()) {
			return ((Endnote) endnotes.get(0)).getNumber();
		}
		return find(mainFrame, type).size() + 1;
	}

	public static Endnote findNumber(List endnotes, int n, int type) {
		if (endnotes == null || endnotes.isEmpty()) {
			return (null);
		}
		for (Object c : endnotes) {
			if (((Endnote) c).type == type && ((Endnote) c).number == n) {
				return (Endnote) c;
			}
		}
		return (null);
	}

	/**
	 * get the link to the endnote from scene text
	 * <a href="#endnote_001" name="innote_001"><small><sup>&#160;(1)</sup></small></a>
	 *
	 * @param dir : for export, the directory of the endnote.html
	 * @return
	 */
	public String getLinkToEndnote(String dir) {
		return linkTo(dir, this);
	}

	/**
	 * get link from the endnote to the scene where it is
	 *
	 * @param dir
	 * @return
	 */
	public String getLinkToScene(String dir) {
		StringBuilder b = new StringBuilder();
		String snumber = String.format("_%03d", id);
		b.append(String.format("<a href=\"%s#innote%s\"", dir, snumber))
				.append(String.format(" name=\"endnote_%s\">", snumber))
				.append(String.format("%d. </a>", number));
		return b.toString();
	}

	/**
	 * Link to the endnote from text
	 * <a class="endnote" href="#endnote_001" name="innote_001"><small>sup>(1)</sup></small></a>
	 *
	 * @param file : real name of the endnote.html file
	 * @param endnote
	 * @return
	 */
	public static String linkTo(String file, Endnote endnote) {
		return linkTo(file, endnote.getId(), endnote.getNumber(), endnote.getType());
	}

	public static String linkTo(String file, Long id, int number, int type) {
		if (type < 0) {
			type = 0;
		}
		if (type > 1) {
			type = 1;
		}
		String snumber = String.format("_%03d", id);
		String ntype = "endnote";
		String enclose = "()";
		if (type == 1) {
			ntype = "comment";
			enclose = "[]";
		}
		String shref = String.format("%s#" + ntype + "%s", file, snumber);
		String sname = String.format("in" + ntype + "%s", snumber);
		String stext = String.format("%c%d%c", enclose.charAt(0), number, enclose.charAt(1));
		String sclass = " class=\"" + ntype + "\"";
		return Html.intoA(sname, shref, stext, sclass);
	}

	/**
	 * get the link from the endnote to scene text
	 * <a href="#endnote_001" name="innote_001">NN. text note</a>
	 *
	 * @param file : real name of the origin file
	 * @param endnote
	 * @return
	 */
	public static String linkFrom(String file, Endnote endnote) {
		return linkFrom(file, endnote.getId(), endnote.getNumber(), endnote.getType(), endnote.getNotes());
	}

	public static String linkFrom(String file, Long id, int number, int type, String notes) {
		if (type < 0) {
			type = 0;
		}
		if (type > 1) {
			type = 1;
		}
		String sid = String.format("_%03d", id);
		StringBuilder b = new StringBuilder();
		String shref = String.format("%s#innote%s", file, sid);
		String sname = String.format("endnote%s", sid);
		String text = String.format("%d. %s", number, notes);
		b.append(Html.intoA(sname, shref, text, ""));
		return b.toString();
	}

	public String getEndnote(ExportBookToHtml exp) {
		StringBuilder b = new StringBuilder();
		b.append(number.toString()).append(". ");
		if (getScene() != null) {
			b.append(exp.sceneLinkTo(scene, "")).append(" ");
		} else {
			b.append("no link to text ");
		}
		if (getNotes() != null && !getNotes().isEmpty()) {
			b.append(getNotes());
		} else {
			b.append("no endnote text ");
		}
		return (b.toString());
	}

	public boolean isInText() {
		return (scene == null ? false : scene.getSummary().contains(getLinkToEndnote("")));
	}

	public String getNotesToHtml() {
		return (StringUtil.escapeHtml(getNotes()));
	}

	public static List<String> getDefColumns() {
		List<String> list = AbstractEntity.getDefColumns(Book.TYPE.ENDNOTE);
		return list;
	}

	public static List<String> getTable() {
		List<String> ls = new ArrayList<>();
		String tableName = "endnote";
		AbstractEntity.getTable(tableName, ls);
		ls.add(tableName + ",number,Integer,0");
		ls.add(tableName + ",type,Integer,0");
		ls.add(tableName + ",sort,String,256");
		ls.add(tableName + ",scene_id,Integer,0");
		return ls;
	}

}
