/*
 * Copyright (C) 2022 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.toolkit;

import i18n.I18N;
import java.io.File;
import javax.swing.JButton;
import resources.icons.ICONS;
import resources.icons.IconUtil;
import shef.ShefEditor;
import shef.actions.HTMLInternalLinkAction;
import shef.actions.manager.ActionList;
import shef.editors.wys.WysiwygEditor;
import storybook.exim.exporter.ExportBookToDoc;
import storybook.exim.importer.ImportDocument;
import storybook.toolkit.file.IOUtil;
import storybook.ui.MainFrame;

/**
 *
 * @author favdb
 */
public class ShefUtil {

	private ShefUtil() {
		// empty
	}

	/**
	 * copy to file
	 *
	 * @param mainFrame
	 * @param editor
	 */
	public static void setToFileButton(MainFrame mainFrame, ShefEditor editor) {
		editor.wysEditorGet().getToolBar1().addSeparator();
		final JButton toFile = new JButton(IconUtil.getIconSmall(ICONS.K.F_EXPORT));
		toFile.setToolTipText(I18N.getMsg("shef.tofile"));
		toFile.addActionListener(e -> {
			ExportBookToDoc.createScene(mainFrame, editor.getText());
		});
		editor.wysEditorGet().getToolBar1().add(toFile);
	}

	/**
	 * past from file
	 *
	 * @param mainFrame
	 * @param editor
	 */
	public static void setFromFileButton(MainFrame mainFrame, ShefEditor editor) {
		//editor.getWysiwygEditor().getToolBar1().addSeparator();
		final JButton fromFile = new JButton(IconUtil.getIconSmall(ICONS.K.F_IMPORT));
		fromFile.setToolTipText(I18N.getMsg("shef.fromfile"));
		fromFile.addActionListener(e -> {
			File inFile = IOUtil.fileSelect(editor, "", "doc", "", "");
			if (inFile == null) {
				return;
			}
			ImportDocument doc = new ImportDocument(mainFrame, inFile);
			if (doc.openDocument()) {
				String str = doc.getDocument().body().html();
				if (!str.isEmpty()) {
					editor.setText(str);
				}
				doc.close();
			}
		});
		editor.wysEditorGet().getToolBar1().add(fromFile);
	}

	public static void setInternalLink(MainFrame mainFrame, ShefEditor editor) {
		WysiwygEditor wysEditor = editor.wysEditorGet();
		HTMLInternalLinkAction action = new HTMLInternalLinkAction(wysEditor, mainFrame);
		ActionList actionList = wysEditor.getActionList();
		actionList.add(action);
		JButton bt = new JButton(IconUtil.getIconSmall(ICONS.K.LINKINTERNAL));
		bt.setToolTipText(I18N.getMsg("link.internal_desc"));
		bt.addActionListener(action);
		wysEditor.getLinksBar().add(bt, 0);
	}

	public static void addButton(MainFrame mainFrame, ShefEditor editor, JButton button) {
		editor.wysEditorGet().getToolBar1().add(button);
	}

}
