/*
 * Copyright (C) 2017 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.ui;

import i18n.I18N;
import i18n.I18NDlg;
import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.List;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import resources.icons.ICONS.K;
import resources.icons.IconUtil;
import storybook.App;
import storybook.Pref;
import storybook.exim.exporter.ExportBookToHtml;
import storybook.model.EntityUtil;
import storybook.model.SceneUtil;
import storybook.model.book.Book;
import storybook.model.hbn.entity.Attribute;
import storybook.model.hbn.entity.Category;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.Endnote;
import storybook.model.hbn.entity.Event;
import storybook.model.hbn.entity.Gender;
import storybook.model.hbn.entity.Idea;
import storybook.model.hbn.entity.Item;
import storybook.model.hbn.entity.Itemlink;
import storybook.model.hbn.entity.Location;
import storybook.model.hbn.entity.Memo;
import storybook.model.hbn.entity.Part;
import storybook.model.hbn.entity.Person;
import storybook.model.hbn.entity.Plot;
import storybook.model.hbn.entity.Relationship;
import storybook.model.hbn.entity.Scene;
import storybook.model.hbn.entity.Strand;
import storybook.model.hbn.entity.Tag;
import storybook.model.hbn.entity.Taglink;
import storybook.model.oldmodel.ModelMigration;
import storybook.toolkit.DockUtil;
import storybook.toolkit.LOG;
import storybook.toolkit.TextTransfer;
import storybook.toolkit.net.Net;
import storybook.toolkit.net.Updater;
import storybook.toolkit.swing.SwingUtil;
import storybook.toolkit.swing.splash.WaitingSplash;
import storybook.ui.SbView.VIEWNAME;
import storybook.ui.dialog.AboutDlg;
import storybook.ui.dialog.CreateChaptersDlg;
import storybook.ui.dialog.CreatePartsDlg;
import storybook.ui.dialog.CreateScenesDlg;
import storybook.ui.dialog.EntityCopyDlg;
import storybook.ui.dialog.ExceptionDlg;
import storybook.ui.dialog.FoiDlg;
import storybook.ui.dialog.MessageDlg;
import storybook.ui.dialog.RenameDlg;
import storybook.ui.dialog.RenumScene;
import storybook.ui.dialog.ReplaceDlg;
import storybook.ui.dialog.ScenesLinks;
import storybook.ui.dialog.SearchDlg;
import storybook.ui.dialog.SpellDlg;
import storybook.ui.dialog.ToolBarDlg;
import storybook.ui.dialog.AssistantWizardDlg;
import storybook.ui.dialog.preferences.EpubCover;
import storybook.ui.dialog.preferences.PreferencesDlg;

/**
 *
 * @author favdb
 */
public class MainMenu implements MouseListener {

	private static final String CTRL = "ctrl ", ALT = "alt ", SHIFT = "shift ";

	private final MainFrame mainFrame;
	public boolean testFunction = false;

	public JToolBar toolBar;
	public JButton btFileNew, btFileOpen, btFileSave,
			btIdea, btMemo,
			btNewChapter, btNewGender, btNewItem, btNewItemlink, btNewLocation,
			btNewPart, btNewPerson, btNewRelation, btNewScene, btNewStrand,
			btNewTag, btNewTaglink,
			btTabChapter, btTabGender, btTabIdea, btTabItem, btTabItemlink,
			btTabLocation, btTabMemo, btTabPart, btTabPerson, btTabRelation,
			btTabScene, btTabStrand, btTabTag, btTabTaglink,
			btViewBook, btViewChrono, btViewManage, btViewMemoria,
			btViewReading, btViewStoryboard, btViewTypist;

	public JMenuBar menuBar;

	private JMenu menuFile;
	private JMenuItem fileNew, fileOpen,
			fileProperties, fileRename, fileClose, fileExit,
			fileSave, fileSaveAs, fileAssistant;
	private JMenu menuFileExport;
	private JMenuItem fileExportBOOK,
			fileExportOther,
			fileExportXml,
			//fileExportOptions,
			fileExportStoryboard;
	private JMenu menuFileImport;
	private JMenuItem fileImportXml;

	private JMenu menuFileBackup;
	private JMenuItem fileBackupNew, fileBackupRest;

	private JMenu menuFileOpenRecent;
	private JMenuItem menuFileCheckDb;

	//menu for new entity
	private JMenu menuNewEntity;
	private JMenuItem newAttribute,
			newCategory, newChapter, newChapters,
			newEvent,
			newFOI,
			newGender,
			newIdea, newItem, newItemLink,
			newLocation,
			newMemo,
			newPart, newPerson, newPlot,
			newRelation,
			newScene, newStrand,
			newTag, newTagLink;
	private JMenu menuPrimaryObjects;
	private JMenu menuSecondaryObjects;

	//menu for tables
	private JMenu menuTables;
	private JMenuItem tabAttribute,
			tabCategory, tabChapter,
			tabEndnote, tabEvent,
			tabGender,
			tabIdea, tabInternal, tabItem, tabItemLink,
			tabLocation,
			tabMemo,
			tabPart, tabPerson, tabPlot,
			tabRelation,
			tabScene, tabStrand,
			tabTag, tabTagLink;

	//menu for charts
	private JMenu menuCharts;
	private JMenuItem chartItems, chartLocations,
			chartPersons, chartPersonsByDate, chartPersonsByScene,
			chartStrands, chartWIWW, chartAttributes;

	//menu for help
	private JMenu menuHelp;
	private JMenuItem helpAbout, helpCheckUpdates, helpDoc, helpFaq, helpHome, helpReportBug,
			helpTranslate, menuToolsChaptersOrder;
	private JMenuItem devTest;

	//menu for edit
	private JMenu menuEdit;
	private JMenu editPaste;
	private JMenuItem editCopyBlurb, editCopyBook, editCopyEntity, editPreferences, editSpellChecker;

	//menu for tools
	private JMenu tools;
	private JMenuItem toolsPlan,
			toolsReplace,
			toolsScenesOrder,
			toolsPersonsLinks,
			toolsLocationsLinks,
			toolsItemsLinks,
			toolsSearch,
			toolsEpisodes,
			toolsNewScenes,
			toolsNewParts,
			toolsRenumberEndnotes,
			toolsEpubCover;
	private JMenu toolsRename;
	private JMenuItem renameCity,
			renameCountry,
			renameItemCategory,
			renameTagCategory;

	//menu for views
	private JMenu menuView;
	private JMenuItem vueBook, vueChrono, vueInfo, vueMemos, vueManageScene;
	private JMenuItem vueMemoria, vueNavigation, vueReading, vueStoryboard,
			vueTree, vueTypist, vueOptions,
			vueTimeline, vueStorymap;

	//menu for window
	private JMenu menuWindow;
	public JMenu windowPredefinedLayout;
	private JMenuItem windowDefaultLayout;
	public JMenu windowLoadLayout;
	private JMenuItem windowRefresh, windowResetLayout, windowSaveLayout;
	public JCheckBoxMenuItem allParts;

	//separators
	private JPopupMenu.Separator separator0, separator1, separator2, separator3, separator4;
	private JButton btViewTimeline;
	private JMenuItem editAssistant;

	public MainMenu(MainFrame m) {
		//LOG.trace("MainMenu(mainFrame)");
		mainFrame = m;
		init();
	}

	private void init() {
		//LOG.trace(TT+"init()");
		UIManager.put("Menu.font", App.getInstance().fontGetDefault());

		initToolbar();

		//create MenuBar
		menuBar = new JMenuBar();
		menuBar.setFont(App.getInstance().fontGetDefault());
		menuBar.setName("menuBar");
		initMenuFile();
		initMenuEdit();
		initMenuNew();
		initMenuTable();
		initMenuTools();
		initMenuView();
		initMenuCharts();
		initMenuWindow();
		initMenuHelp();
		setToolbar();
		enableSave(false);
	}

	public void setToolbar() {
		toolBar.removeMouseListener(this);
		JButton[] btn = {
			btFileNew, btFileOpen, btFileSave, //3
			btNewStrand, btNewPart, btNewChapter, btNewScene,//4
			btNewPerson, btNewGender, btNewRelation,//3
			btNewLocation,//1
			btNewItem, btNewItemlink,//2
			btNewTag, btNewTaglink, //2
			btMemo, btIdea, //2
			btTabStrand, btTabPart, btTabChapter, btTabScene,//4
			btTabPerson, btTabGender, btTabRelation, //3
			btTabLocation, //1
			btTabItem, btTabItemlink,//2
			btTabTag, btTabTaglink, //2
			btTabMemo, btTabIdea,//2
			btViewChrono, btViewTimeline, btViewBook, btViewManage, btViewReading, btViewMemoria, btViewStoryboard, btViewTypist //6
		}; // total 37
		//récupération du paramétrage sous forme d'un String
		String param = mainFrame.getPref().get(Pref.KEY.TOOLBAR.toString(), Pref.KEY.TOOLBAR.getValue());
		while (param.length() < btn.length) {
			param += "1";// default allParts button are visible
		}
		if (param.length() > btn.length) {
			param = param.substring(0, btn.length);
		}
		for (int i = 0; i < param.length(); i++) {
			btn[i].setVisible((param.charAt(i) == '1'));
		}
		toolBar.setFloatable(true);
		toolBar.addMouseListener(this);
		if (mainFrame.isBlank()) {
			hideToolbar();
		}
	}

	public void setTimelineMenu() {
		List<Scene> s = EntityUtil.findScenes(mainFrame);
		if (!s.isEmpty()) {
			vueTimeline.setEnabled(s.get(0).hasScenets());
		}
	}

	private JButton initButton(K icon, String tips, ActionListener... action) {
		JButton bt = SwingUtil.createButton("", icon.toString(), tips, true, action);
		bt.addMouseListener(this);
		return bt;
	}

	private void initToolbar() {
		if (toolBar == null) {
			toolBar = new JToolBar();
		}
		toolBar.setFloatable(false);
		toolBar.setName("MainToolbar");

		initTbFile();
		toolBar.add(new JToolBar.Separator());
		initTbNew();
		toolBar.add(new JToolBar.Separator());
		initTbTab();
		toolBar.add(new JToolBar.Separator());
		initTbView();
		toolBar.add(new JToolBar.Separator());
		initTbMemo();
	}

	private void initTbFile() {
		//LOG.trace(TT+"initTbFile()");
		btFileNew = initButton(K.F_NEW, "file.new",
				evt -> mainFrame.action.fileNew());
		toolBar.add(btFileNew);

		btFileOpen = initButton(K.F_OPEN, "file.open",
				evt -> mainFrame.action.fileOpen());
		toolBar.add(btFileOpen);

		btFileSave = initButton(K.F_SAVE, "file.save",
				evt -> mainFrame.action.fileSave());
		toolBar.add(btFileSave);

	}

	private void initTbNew() {
		//LOG.trace(TT+"initTbNew()");
		btNewStrand = initButton(K.NEW_STRAND, "strand.new", evt -> mainFrame.newEntity(new Strand()));
		toolBar.add(btNewStrand);

		btNewPart = initButton(K.NEW_PART, "part.new", evt -> mainFrame.newEntity(new Part()));
		toolBar.add(btNewPart);

		btNewChapter = initButton(K.NEW_CHAPTER, "chapter.new", evt -> mainFrame.newEntity(new Chapter()));
		toolBar.add(btNewChapter);

		btNewScene = initButton(K.NEW_SCENE, "scene.new", evt -> mainFrame.newEntity(new Scene()));
		toolBar.add(btNewScene);

		btNewPerson = initButton(K.NEW_PERSON, "person.new", evt -> mainFrame.newEntity(new Person()));
		toolBar.add(btNewPerson);

		btNewGender = initButton(K.NEW_GENDER, "gender.new", evt -> mainFrame.newEntity(new Gender()));
		toolBar.add(btNewGender);

		btNewRelation = initButton(K.ENT_RELATION, "relation.new", evt -> mainFrame.newEntity(new Relationship()));
		toolBar.add(btNewRelation);

		btNewLocation = initButton(K.NEW_LOCATION, "location.new", evt -> mainFrame.newEntity(new Location()));
		toolBar.add(btNewLocation);

		btNewItem = initButton(K.NEW_ITEM, "item.new", evt -> mainFrame.newEntity(new Item()));
		toolBar.add(btNewItem);

		btNewItemlink = initButton(K.ENT_ITEMLINK, "itemlink.new", evt -> mainFrame.newEntity(new Itemlink()));
		toolBar.add(btNewItemlink);

		btNewTag = initButton(K.NEW_TAG, "tag.new", evt -> mainFrame.newEntity(new Tag()));
		toolBar.add(btNewTag);

		btNewTaglink = initButton(K.ENT_TAGLINK, "taglink.new", evt -> mainFrame.newEntity(new Taglink()));
		toolBar.add(btNewTaglink);

	}

	private void initTbTab() {
		//LOG.trace(TT+"initTbTab()");
		btTabStrand = initButton(K.TABLE_STRANDS, "strands", evt -> mainFrame.showAndFocus(VIEWNAME.STRANDS));
		toolBar.add(btTabStrand);

		btTabPart = initButton(K.TABLE_PARTS, "parts", evt -> mainFrame.showAndFocus(VIEWNAME.PARTS));
		toolBar.add(btTabPart);

		btTabChapter = initButton(K.TABLE_CHAPTERS, "chapters", evt -> mainFrame.showAndFocus(VIEWNAME.CHAPTERS));
		toolBar.add(btTabChapter);

		btTabScene = initButton(K.TABLE_SCENES, "scenes", evt -> mainFrame.showAndFocus(VIEWNAME.SCENES));
		toolBar.add(btTabScene);

		btTabPerson = initButton(K.TABLE_PERSONS, "persons", evt -> mainFrame.showAndFocus(VIEWNAME.PERSONS));
		toolBar.add(btTabPerson);

		btTabGender = initButton(K.TABLE_GENDERS, "genders", evt -> mainFrame.showAndFocus(VIEWNAME.GENDERS));
		toolBar.add(btTabGender);

		btTabRelation = initButton(K.TABLE_RELATIONS, "relations", evt -> mainFrame.showAndFocus(VIEWNAME.RELATIONS));
		toolBar.add(btTabRelation);

		btTabLocation = initButton(K.TABLE_LOCATIONS, "locations", evt -> mainFrame.showAndFocus(VIEWNAME.LOCATIONS));
		toolBar.add(btTabLocation);

		btTabItem = initButton(K.TABLE_ITEMS, "items", evt -> mainFrame.showAndFocus(VIEWNAME.ITEMS));
		toolBar.add(btTabItem);

		btTabItemlink = initButton(K.TABLE_ITEMLINKS, "itemlinks", evt -> mainFrame.showAndFocus(VIEWNAME.ITEMLINKS));
		toolBar.add(btTabItemlink);

		btTabTag = initButton(K.TABLE_TAGS, "tags", evt -> mainFrame.showAndFocus(VIEWNAME.TAGS));
		toolBar.add(btTabTag);

		btTabTaglink = initButton(K.TABLE_TAGLINKS, "taglinks", evt -> mainFrame.showAndFocus(VIEWNAME.TAGLINKS));
		toolBar.add(btTabTaglink);

		btTabMemo = initButton(K.TABLE_MEMOS, "memos", evt -> mainFrame.showAndFocus(VIEWNAME.MEMOS));
		toolBar.add(btTabMemo);

		btTabIdea = initButton(K.TABLE_IDEAS, "ideas", evt -> mainFrame.showAndFocus(VIEWNAME.IDEAS));
		toolBar.add(btTabIdea);

	}

	private void initTbView() {
		//LOG.trace(TT+"initTbView()");
		btViewChrono = initButton(K.VW_CHRONO, "view.chrono", evt -> mainFrame.showAndFocus(VIEWNAME.CHRONO));
		btViewChrono.setEnabled(false);
		toolBar.add(btViewChrono);

		btViewTimeline = initButton(K.VW_TIMELINE, "view.timeline", evt -> mainFrame.showAndFocus(VIEWNAME.TIMELINE));
		toolBar.add(btViewTimeline);

		btViewBook = initButton(K.VW_BOOK, "view.book", evt -> mainFrame.showAndFocus(VIEWNAME.BOOK));
		toolBar.add(btViewBook);

		btViewManage = initButton(K.VW_MANAGE, "view.manage", evt -> mainFrame.showAndFocus(VIEWNAME.MANAGE));
		toolBar.add(btViewManage);

		btViewReading = initButton(K.VW_READING, "view.reading");
		btViewReading.addActionListener(evt -> mainFrame.showAndFocus(VIEWNAME.READING));
		toolBar.add(btViewReading);

		btViewMemoria = initButton(K.VW_MEMORIA, "view.memoria", evt -> mainFrame.showAndFocus(VIEWNAME.MEMORIA));
		toolBar.add(btViewMemoria);
		//btViewMemoria.setEnabled(App.isDev());

		btViewStoryboard = initButton(K.STAMP, "view.storyboard", evt -> mainFrame.showAndFocus(VIEWNAME.STORYBOARD));
		toolBar.add(btViewStoryboard);

		toolBar.addSeparator();

		btViewTypist = initButton(K.VW_TYPIST, "typist", evt -> mainFrame.typistActivate());
		toolBar.add(btViewTypist);

	}

	private void initTbMemo() {
		//LOG.trace(TT+"initTbMemo()");
		btMemo = initButton(K.ENT_MEMO, "memos");
		btMemo.addActionListener(evt -> mainFrame.showEditorAsDialog(new Memo()));
		toolBar.add(btMemo);

		btIdea = initButton(K.ENT_IDEA, "ideas");
		btIdea.addActionListener(evt -> mainFrame.showEditorAsDialog(new Idea()));
		toolBar.add(btIdea);
	}

	private JMenuItem initMenuItem(String title, ActionListener... action) {
		return (initMenuItem(K.EMPTY, "", ' ', title, "", action));
	}

	private JMenuItem initMenuItem(String title, String key, ActionListener... action) {
		JMenuItem m = initMenuItem(K.EMPTY, "", ' ', title, "");
		m.setFont(App.getInstance().fontGetDefault());
		if (!key.isEmpty()) {
			KeyStroke k = KeyStroke.getKeyStroke(key);
			if (k != null) {
				m.setAccelerator(k);
			}
		}
		if (action != null && action.length > 0) {
			m.addActionListener(action[0]);
		}
		return (m);
	}

	private JMenuItem initMenuItem(K icon, String key, char mnemonic, String text,
			String name, ActionListener... action) {
		if (icon == K.EMPTY) {
			return initMenuItem("", key, mnemonic, text, name, action);
		}
		return initMenuItem(icon.toString(), key, mnemonic, text, name, action);
	}

	private JMenuItem initMenuItem(String icon, String key, char mnemonic, String text,
			String name, ActionListener... action) {
		JMenuItem m = new JMenuItem();
		m.setFont(App.getInstance().fontGetDefault());
		if (!key.isEmpty()) {
			KeyStroke k = KeyStroke.getKeyStroke(key);
			if (k != null) {
				m.setAccelerator(k);
			}
		}
		if (!icon.isEmpty() && !icon.equals("empty")) {
			m.setIcon(IconUtil.getIconSmall(icon));
		}
		if (mnemonic != ' ') {
			m.setMnemonic(mnemonic);
		}
		if (text.startsWith("!")) {
			m.setText(text.substring(1));
		} else {
			m.setText(I18N.getMsg(text));
		}
		m.setActionCommand(name);
		m.setName(name);
		if (action != null && action.length > 0) {
			m.addActionListener(action[0]);
		}
		return (m);
	}

	private void initMenuFile() {
		//LOG.trace(TT+"initMenuFile()");
		menuFile = new JMenu();
		menuFile.setMnemonic(I18N.getMsg("file.mnemonic").charAt(0));
		menuFile.setText(I18N.getMsg("file"));

		JMenu menuFileNew = new JMenu();
		menuFileNew.setMnemonic(I18N.getMsg("project.create.mnemonic").charAt(0));
		menuFileNew.setText(I18N.getMsg("project.create"));
		menuFileNew.setIcon(IconUtil.getIconSmall(K.PLUS));

		fileNew = initMenuItem(K.F_NEW, CTRL + "N", 'N', "file.new",
				"new-command", evt -> mainFrame.action.fileNew());
		fileNew.setToolTipText(I18N.getMsg("file.new_tip"));
		menuFileNew.add(fileNew);

		JMenuItem fileNewDoc = initMenuItem(K.F_IMPORT, "", ' ', "file.from_doc", "import-doc",
				evt -> mainFrame.action.fileImportDoc());
		fileNewDoc.setToolTipText(I18N.getMsg("file.from_doc_tip"));
		menuFileNew.add(fileNewDoc);

		menuFile.add(menuFileNew);

		fileOpen = initMenuItem(K.F_OPEN, CTRL + "O", 'O', "project.open",
				"open-command", evt -> mainFrame.action.fileOpen());
		menuFile.add(fileOpen);

		menuFileOpenRecent = new JMenu();
		menuFileOpenRecent.setText(I18N.getMsg("file.open.recent"));
		menuFileOpenRecent.setActionCommand("recent-menu-command");
		menuFileOpenRecent.setName("recent-menu-command");
		menuFile.add(menuFileOpenRecent);

		fileSave = initMenuItem(K.F_SAVE, CTRL + "S", 'S', "project.save", "save-command");
		fileSave.addActionListener(evt -> mainFrame.action.fileSave());
		menuFile.add(fileSave);

		fileSaveAs = initMenuItem(K.F_SAVEAS, CTRL + "A", 'A', "project.save.as",
				"export-book-command", evt -> mainFrame.action.fileSaveAs());
		menuFile.add(fileSaveAs);

		fileRename = initMenuItem(K.RENAME, CTRL + "R", 'R', "project.rename",
				"rename-command", evt -> mainFrame.action.fileRename());
		menuFile.add(fileRename);

		fileClose = initMenuItem(K.CLOSE, CTRL + "F4", ' ', "project.close",
				"close-command", evt -> mainFrame.close(false));
		menuFile.add(fileClose);

		separator0 = new JPopupMenu.Separator();
		menuFile.add(separator0);

		menuFileBackup = new JMenu();
		menuFileBackup.setText(I18N.getMsg("file.backup"));
		menuFileBackup.setName("recent-menu-command");
		menuFile.add(menuFileBackup);
		fileBackupNew = initMenuItem(K.BK_SAVE, "", ' ', "file.backup_new",
				"backup_new", evt -> mainFrame.backupNew(true));
		menuFileBackup.add(fileBackupNew);
		fileBackupRest = initMenuItem(K.BK_REST, "", ' ', "file.backup_rest",
				"backup_rest", evt -> mainFrame.backupRest());
		menuFileBackup.add(fileBackupRest);

		separator1 = new JPopupMenu.Separator();
		menuFile.add(separator1);

		fileProperties = initMenuItem("file.properties");
		fileProperties.addActionListener(evt -> {
			if (mainFrame == null) {
				LOG.err("mainFrame is null");
				return;
			}
			if (mainFrame.action == null) {
				LOG.err("mainFrame.action is null");
				return;
			}
			mainFrame.action.fileProperties();
		});
		menuFile.add(fileProperties);

		menuFileCheckDb = initMenuItem("file.optimize");
		menuFileCheckDb.addActionListener(evt -> {
			if (MessageDlg.show(mainFrame, "file.optimize_info", "file.optimize")) {
				final ModelMigration old = ModelMigration.getInstance();
				if (old.open(mainFrame.getH2File())) {
					try {
						old.checkAndAlterModel(true);
					} catch (Exception e) {
						LOG.err("Optimizing File error: " + mainFrame.getH2File().getH2Name() + ")", e);
						//ExceptionDlg.show("", e);
					}
					old.closeConnection();
					mainFrame.getBook().setDbVersion();
				}
			}
		});
		menuFile.add(menuFileCheckDb);

		separator2 = new JPopupMenu.Separator();
		menuFile.add(separator2);

		initMenuFileImport();
		initMenuFileExport();

		separator4 = new JPopupMenu.Separator();
		menuFile.add(separator4);

		fileExit = initMenuItem(K.EXIT, ALT + "F4", ' ', "exit",
				"exit-command", evt -> {
					//mainFrame.close(true);
					App.getInstance().exit();
				});
		menuFile.add(fileExit);

		menuBar.add(menuFile);
	}

	private void initMenuFileImport() {
		menuFileImport = new JMenu(I18N.getMsg("file.import"));
		menuFile.add(menuFileImport);

		fileImportXml = initMenuItem("file.import.xml", evt -> mainFrame.action.importNew());
		menuFileImport.add(fileImportXml);

		/*fileImportDoc = initMenuItem(K.F_IMPORT, "", ' ',
				"file.import.doc", "import-doc",
				evt -> mainFrame.action.fileImportDoc());
		fileImportDoc.setToolTipText(I18N.getMsg("file.import.doc_tip"));*/
	}

	private void initMenuFileExport() {
		menuFileExport = new JMenu(I18N.getMsg("file.export"));
		menuFile.add(menuFileExport);

		fileExportStoryboard = initMenuItem(K.VW_STORYBOARD, "", ' ',
				"view.storyboard", "exportStoryboard",
				evt -> mainFrame.action.exportStoryboard());
		menuFileExport.add(fileExportStoryboard);

		fileExportBOOK = initMenuItem(K.VW_BOOK, "", ' ',
				"book", "exportBOOK",
				evt -> mainFrame.action.exportBook());
		menuFileExport.add(fileExportBOOK);

		fileExportXml = initMenuItem(K.COLUMNS, "", ' ',
				"file.export.base", "exportXml",
				evt -> mainFrame.action.exportXml());
		menuFileExport.add(fileExportXml);

		fileExportOther = initMenuItem(K.EXPAND, "", ' ',
				"file.export.other", "exportOther",
				evt -> mainFrame.action.exportOther());
		menuFileExport.add(fileExportOther);
	}

	private void initMenuEdit() {
		//LOG.trace(TT+"initMenuEdit()");
		menuEdit = new JMenu(I18N.getMsg("edit"));
		menuEdit.setMnemonic(I18N.getMsg("edit.mnemonic").charAt(0));
		menuEdit.setName("edit-menu-command");

		editCopyBook = initMenuItem(K.EDIT_COPY, "", 'C', "book.copy.text", "bookCopy",
				evt -> ExportBookToHtml.toClipboard(mainFrame, (Part) null));
		menuEdit.add(editCopyBook);

		editCopyBlurb = initMenuItem(K.EDIT_COPY, "", 'P', "book.copy.blurb", "blurbCopy");
		editCopyBlurb.addActionListener(evt -> {
			TextTransfer tf = new TextTransfer();
			tf.setClipboardContents(mainFrame.getBook().getBlurb() + "\n");
		});
		menuEdit.add(editCopyBlurb);

		editCopyEntity = initMenuItem("copy.menu");
		editCopyEntity.addActionListener(evt -> EntityCopyDlg.show(mainFrame));
		menuEdit.add(editCopyEntity);

		editPaste = new JMenu(I18N.getMsg("edit.paste_as"));
		editPaste.add(initMenuItem("part", evt -> {
			mainFrame.pasteAsPart();
		}));
		editPaste.add(initMenuItem("chapter", evt -> {
			mainFrame.pasteAsChapter();
		}));
		editPaste.add(initMenuItem("scene", evt -> {
			mainFrame.pasteAsScene();
		}));
		editPaste.setVisible(App.getInstance().hasPasted);
		menuEdit.add(editPaste);

		JMenuItem initEditor = initMenuItem("editor.reset", CTRL + ALT + "R");
		initEditor.addActionListener(evt -> SwingUtil.resetDlgPosition());
		menuEdit.add(initEditor);

		menuEdit.add(new JPopupMenu.Separator());

		editPreferences = initMenuItem("preferences.title");
		editPreferences.setIcon(IconUtil.getIconSmall(K.PREFERENCES));
		editPreferences.addActionListener(evt -> {
			PreferencesDlg dlg = new PreferencesDlg(mainFrame);
			SwingUtil.showModalDialog(dlg, mainFrame);
		});
		menuEdit.add(editPreferences);

		menuBar.add(menuEdit);
	}

	private void initMenuNew() {
		//LOG.trace(TT+"initMenuNew()");
		menuNewEntity = new JMenu(I18N.getMsg("new"));

		newStrand = initMenuItem(K.ENT_STRAND, CTRL + SHIFT + "S", 'S', "strand",
				"newStrand", evt -> mainFrame.newEntity(new Strand()));
		menuNewEntity.add(newStrand);

		newPart = initMenuItem(K.ENT_PART, "", 'P', "part",
				"newPart", evt -> mainFrame.newEntity(new Part()));
		menuNewEntity.add(newPart);

		newChapter = initMenuItem(K.ENT_CHAPTER, CTRL + SHIFT + "C", 'C', "chapter",
				"newChapter", evt -> mainFrame.newEntity(new Chapter()));
		menuNewEntity.add(newChapter);

		newScene = initMenuItem(K.ENT_SCENE, CTRL + SHIFT + "S", 'S', "scene",
				"newScene", evt -> mainFrame.newEntity(new Scene()));
		menuNewEntity.add(newScene);

		JMenu multi = new JMenu(I18N.getMsg("new.multi"));
		menuNewEntity.add(multi);
		toolsNewParts = initMenuItem(K.ENT_PART, "", 'M', "parts",
				"newParts", evt -> {
					CreatePartsDlg dlg = new CreatePartsDlg(mainFrame);
					dlg.setVisible(true);
					if (!dlg.isCanceled()) {
						mainFrame.getBookModel().refreshParts();
						mainFrame.setUpdated();
					}
				});
		multi.add(toolsNewParts);

		newChapters = initMenuItem(K.ENT_CHAPTER, "", 'M', "chapters",
				"newChapters", evt -> {
					CreateChaptersDlg dlg = new CreateChaptersDlg(mainFrame);
					dlg.setVisible(true);
					if (!dlg.isCanceled()) {
						mainFrame.getBookModel().refreshChapters();
						mainFrame.setUpdated();
					}
				});
		multi.add(newChapters);

		toolsNewScenes = initMenuItem(K.ENT_SCENE, "", 'M', "scenes",
				"newScenes", evt -> {
					CreateScenesDlg dlg = new CreateScenesDlg(mainFrame);
					dlg.setVisible(true);
					if (!dlg.isCanceled()) {
						mainFrame.getBookModel().refreshScenes();
						mainFrame.setUpdated();
					}
				});
		multi.add(toolsNewScenes);

		newPlot = initMenuItem(K.ENT_PLOT, "", ' ', "plot",
				"newPlot", evt -> mainFrame.newEntity(new Plot()));
		menuNewEntity.add(newPlot);
		menuNewEntity.add(new JPopupMenu.Separator());

		newPerson = initMenuItem(K.ENT_PERSON, CTRL + SHIFT + "P", 'P', "person",
				"newPerson", evt -> mainFrame.newEntity(new Person()));
		menuNewEntity.add(newPerson);

		newGender = initMenuItem(K.ENT_GENDER, CTRL + SHIFT + "G", ' ', "genders",
				"newGender", evt -> mainFrame.newEntity(new Gender()));
		menuNewEntity.add(newGender);

		newCategory = initMenuItem(K.ENT_CATEGORY, "", ' ', "persons.category",
				"newCategory", evt -> mainFrame.newEntity(new Category()));
		menuNewEntity.add(newCategory);

		newRelation = initMenuItem(K.ENT_RELATION, CTRL + SHIFT + "R", ' ', "relationship",
				"newRelation", evt -> {
					mainFrame.newEntity(new Relationship());
				});
		menuNewEntity.add(newRelation);

		newAttribute = initMenuItem(K.EMPTY, "", ' ', "attribute",
				"newAttribute", evt -> mainFrame.newEntity(new Attribute()));
		if (App.isDev()) {
			menuNewEntity.add(newAttribute);
		}

		newEvent = initMenuItem(K.ENT_EVENT, CTRL + SHIFT + "E", ' ', "event",
				"newEvent", evt -> mainFrame.newEntity(new Event()));
		menuNewEntity.add(newEvent);

		menuNewEntity.add(new JPopupMenu.Separator());

		newLocation = initMenuItem(K.ENT_LOCATION, CTRL + SHIFT + "L", 'L', "location",
				"newLocation", evt -> mainFrame.newEntity(new Location()));
		menuNewEntity.add(newLocation);
		menuNewEntity.add(new JPopupMenu.Separator());

		newItem = initMenuItem(K.ENT_ITEM, CTRL + SHIFT + "I", 'I', "item",
				"newItem", evt -> mainFrame.newEntity(new Item()));
		menuNewEntity.add(newItem);

		newItemLink = initMenuItem(K.LINK, "", ' ', "itemlink",
				"newItemLink", evt -> mainFrame.newEntity(new Itemlink()));
		menuNewEntity.add(newItemLink);
		menuNewEntity.add(new JPopupMenu.Separator());

		newTag = initMenuItem(K.ENT_TAG, CTRL + SHIFT + "T", 'T', "tag",
				"newTag", evt -> mainFrame.newEntity(new Tag()));
		menuNewEntity.add(newTag);

		newTagLink = initMenuItem(K.LINK, "", ' ', "taglink",
				"newTagLink", evt -> mainFrame.newEntity(new Taglink()));
		menuNewEntity.add(newTagLink);
		menuNewEntity.add(new JPopupMenu.Separator());

		newFOI = initMenuItem(K.ENT_IDEA, CTRL + ALT + "F", ' ', "foi.title",
				"newFOI", evt -> FoiDlg.show(mainFrame));
		menuNewEntity.add(newFOI);

		newIdea = initMenuItem(K.ENT_IDEA, CTRL + SHIFT + "F", ' ', "idea.table.idea",
				"newIdea", evt -> mainFrame.newEntity(new Idea()));
		menuNewEntity.add(newIdea);

		newMemo = initMenuItem(K.ENT_MEMO, CTRL + SHIFT + "M", ' ', "memo",
				"newMemo", evt -> mainFrame.newEntity(new Memo()));
		menuNewEntity.add(newMemo);

		menuBar.add(menuNewEntity);
	}

	private void initMenuTable() {
		//LOG.trace(TT+"initMenuTable()");
		menuTables = new JMenu(I18N.getMsg("tables"));

		menuPrimaryObjects = new JMenu(I18N.getMsg("primary.objects"));

		tabInternal = initMenuItem(K.EMPTY, "", ' ',
				"internal", "tabInternal",
				evt -> mainFrame.showAndFocus(VIEWNAME.INTERNALS));
		menuPrimaryObjects.add(tabInternal);
		if (!App.isDev()) {
			tabInternal.setVisible(false);
		}
		tabStrand = initMenuItem(K.TABLE_STRANDS, CTRL + ALT + "D", 'D',
				"strand", "tabStrand",
				evt -> mainFrame.showAndFocus(VIEWNAME.STRANDS));
		menuPrimaryObjects.add(tabStrand);

		tabPart = initMenuItem(K.TABLE_PARTS, CTRL + ALT + "T", 'T',
				"part", "tabPart",
				evt -> mainFrame.showAndFocus(VIEWNAME.PARTS));
		menuPrimaryObjects.add(tabPart);

		tabChapter = initMenuItem(K.TABLE_CHAPTERS, CTRL + ALT + "C", 'C',
				"chapter", "tabChapter",
				evt -> mainFrame.showAndFocus(VIEWNAME.CHAPTERS));
		menuPrimaryObjects.add(tabChapter);

		tabScene = initMenuItem(K.TABLE_SCENES, CTRL + ALT + "S", 'S',
				"scene", "tabScene",
				evt -> mainFrame.showAndFocus(VIEWNAME.SCENES));
		menuPrimaryObjects.add(tabScene);

		tabPlot = initMenuItem(K.TABLE_PLOTS, "", ' ',
				"plot", "tabPlot",
				evt -> mainFrame.showAndFocus(VIEWNAME.PLOTS));
		menuPrimaryObjects.add(tabPlot);

		tabPerson = initMenuItem(K.TABLE_PERSONS, CTRL + ALT + "P", 'P',
				"person", "tabPerson",
				evt -> mainFrame.showAndFocus(VIEWNAME.PERSONS));
		menuPrimaryObjects.add(tabPerson);

		tabLocation = initMenuItem(K.TABLE_LOCATIONS, CTRL + ALT + "L", 'L',
				"location", "tabLocation",
				evt -> mainFrame.showAndFocus(VIEWNAME.LOCATIONS));
		menuPrimaryObjects.add(tabLocation);

		tabItem = initMenuItem(K.TABLE_ITEMS, CTRL + ALT + "I", 'I',
				"item", "tabItem",
				evt -> mainFrame.showAndFocus(VIEWNAME.ITEMS));
		menuPrimaryObjects.add(tabItem);

		menuTables.add(menuPrimaryObjects);

		menuSecondaryObjects = new JMenu(I18N.getMsg("secondary.objects"));

		tabGender = initMenuItem(K.TABLE_GENDERS, CTRL + ALT + "G", 'G',
				"genders", "tabGender",
				evt -> mainFrame.showAndFocus(VIEWNAME.GENDERS));
		menuSecondaryObjects.add(tabGender);

		tabCategory = initMenuItem(K.TABLE_CATEGORIES, "", ' ',
				"persons.category", "tabCategory",
				evt -> mainFrame.showAndFocus(VIEWNAME.CATEGORIES));
		menuSecondaryObjects.add(tabCategory);

		tabRelation = initMenuItem(K.TABLE_RELATIONS, "", ' ',
				"relations", "tabRelation",
				evt -> mainFrame.showAndFocus(VIEWNAME.RELATIONS));
		menuSecondaryObjects.add(tabRelation);

		tabAttribute = initMenuItem(K.EMPTY, "", ' ',
				"attributes", "tabAttribute",
				evt -> mainFrame.showAndFocus(VIEWNAME.ATTRIBUTES));
		if (App.isDev()) {
			menuSecondaryObjects.add(tabAttribute);
		}

		tabEvent = initMenuItem(K.ENT_EVENT, "", ' ',
				"events", "tabEvent",
				evt -> mainFrame.showAndFocus(VIEWNAME.EVENTS));
		menuSecondaryObjects.add(tabEvent);

		menuSecondaryObjects.add(new JPopupMenu.Separator());

		tabItemLink = initMenuItem(K.TABLE_ITEMLINKS, "", ' ',
				"itemlink", "tabItemLink",
				evt -> mainFrame.showAndFocus(VIEWNAME.ITEMLINKS));
		menuSecondaryObjects.add(tabItemLink);

		menuSecondaryObjects.add(new JPopupMenu.Separator());

		tabTag = initMenuItem(K.TABLE_TAGS, CTRL + ALT + "T", 'T',
				"tags", "tabTag",
				evt -> mainFrame.showAndFocus(VIEWNAME.TAGS));
		menuSecondaryObjects.add(tabTag);

		tabTagLink = initMenuItem(K.TABLE_TAGLINKS, "", ' ',
				"taglink", "tabTagLink",
				evt -> mainFrame.showAndFocus(VIEWNAME.TAGLINKS));
		menuSecondaryObjects.add(tabTagLink);
		menuSecondaryObjects.add(new JPopupMenu.Separator());

		tabEndnote = initMenuItem(K.CHAR_ENDNOTE, "", ' ',
				"endnote", "tabEndnote",
				evt -> mainFrame.showAndFocus(VIEWNAME.ENDNOTES));
		menuSecondaryObjects.add(tabEndnote);

		menuSecondaryObjects.add(new JPopupMenu.Separator());

		tabMemo = initMenuItem(K.TABLE_MEMOS, "", ' ',
				"memos", "tabMemo",
				evt -> mainFrame.showAndFocus(VIEWNAME.MEMOS));
		menuSecondaryObjects.add(tabMemo);

		tabIdea = initMenuItem(K.TABLE_IDEAS, "", ' ',
				"ideas", "tabIdea",
				evt -> mainFrame.showAndFocus(VIEWNAME.IDEAS));
		menuSecondaryObjects.add(tabIdea);

		menuTables.add(menuSecondaryObjects);

		menuBar.add(menuTables);
	}

	private void initMenuTools() {
		//LOG.trace(TT+"initMenuTools()");
		tools = new JMenu(I18N.getMsg("tools"));

		toolsEpisodes = initMenuItem("episodes",
				evt -> mainFrame.showAndFocus(VIEWNAME.EPISODES));
		tools.add(toolsEpisodes);
		tools.add(new JSeparator());

		toolsSearch = initMenuItem("search", CTRL + "F",
				evt -> SearchDlg.show(mainFrame));
		tools.add(toolsSearch);

		toolsReplace = initMenuItem("replace", CTRL + "H",
				evt -> ReplaceDlg.show(mainFrame));
		tools.add(toolsReplace);

		menuToolsChaptersOrder = initMenuItem("chapters.order",
				evt -> mainFrame.action.chaptersOrderAction());
		tools.add(menuToolsChaptersOrder);

		toolsScenesOrder = initMenuItem("scenes.renumber",
				evt -> {
					if (RenumScene.show(mainFrame)) {
						App.getInstance().setWaitCursor();
						final WaitingSplash dlg = new WaitingSplash(mainFrame, I18N.getMsg("scenes.renumber"));
						dlg.setVisible(true);
						SwingUtilities.invokeLater(() -> {
							try {
								SceneUtil.renumber(mainFrame);
							} catch (Exception e) {
								ExceptionDlg.show("", e);
							}
							dlg.dispose();
						});
						App.getInstance().setDefaultCursor();
					}
				});
		tools.add(toolsScenesOrder);

		JMenu menuLinks = new JMenu(I18N.getMsg("links"));
		toolsPersonsLinks = initMenuItem("links.person",
				evt -> ScenesLinks.show(mainFrame, Book.TYPE.PERSON));
		menuLinks.add(toolsPersonsLinks);
		toolsLocationsLinks = initMenuItem("links.location",
				evt -> ScenesLinks.show(mainFrame, Book.TYPE.LOCATION));
		menuLinks.add(toolsLocationsLinks);
		toolsItemsLinks = initMenuItem("links.item",
				evt -> ScenesLinks.show(mainFrame, Book.TYPE.PERSON));
		menuLinks.add(toolsItemsLinks);
		tools.add(menuLinks);

		toolsRenumberEndnotes = initMenuItem("endnotes.renumber",
				evt -> {
					if (Endnote.renumber(mainFrame, 0)) {
						mainFrame.setUpdated();
						mainFrame.getBookModel().fireAgain();
					}
				});
		tools.add(toolsRenumberEndnotes);

		toolsRename = new JMenu(I18N.getMsg("tools.rename"));
		toolsRename.setIcon(IconUtil.getIconSmall(K.RENAME));

		renameCity = initMenuItem("location.rename.city",
				evt -> {
					RenameDlg dlg = new RenameDlg(mainFrame, "city");
					SwingUtil.showModalDialog(dlg, mainFrame);
				});
		toolsRename.add(renameCity);

		renameCountry = initMenuItem("location.rename.country",
				evt -> {
					RenameDlg dlg = new RenameDlg(mainFrame, "country");
					SwingUtil.showModalDialog(dlg, mainFrame);
				});
		toolsRename.add(renameCountry);

		renameTagCategory = initMenuItem("tag.rename.category",
				evt -> {
					RenameDlg dlg = new RenameDlg(mainFrame, "tag");
					SwingUtil.showModalDialog(dlg, mainFrame);
				});
		toolsRename.add(renameTagCategory);

		renameItemCategory = initMenuItem("item.rename.category",
				evt -> {
					RenameDlg dlg = new RenameDlg(mainFrame, "item");
					SwingUtil.showModalDialog(dlg, mainFrame);
				});
		toolsRename.add(renameItemCategory);

		tools.add(toolsRename);

		tools.add(new JSeparator());
		toolsEpubCover = initMenuItem("epub.cover.create",
				evt -> EpubCover.show(mainFrame));
		tools.add(toolsEpubCover);

		menuBar.add(tools);

	}

	private void initMenuView() {
		//LOG.trace(TT+"initMenuView()");
		menuView = new JMenu(I18N.getMsg("view"));

		vueTypist = initMenuItem(K.VW_TYPIST, "", ' ',
				"typist", "",
				evt -> mainFrame.typistActivate());
		menuView.add(vueTypist);
		menuView.addSeparator();

		vueChrono = initMenuItem(K.VW_CHRONO, "", ' ',
				"view.chrono", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.CHRONO));
		menuView.add(vueChrono);

		vueTimeline = initMenuItem(K.VW_TIMELINE, "", ' ',
				"view.timeline", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.TIMELINE));
		menuView.add(vueTimeline);

		vueBook = initMenuItem(K.VW_BOOK, "", ' ',
				"view.book", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.BOOK));
		menuView.add(vueBook);

		vueReading = initMenuItem(K.VW_READING, "", ' ',
				"view.reading", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.READING));
		menuView.add(vueReading);

		vueManageScene = initMenuItem(K.VW_MANAGE, "", ' ',
				"view.manage", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.MANAGE));
		menuView.add(vueManageScene);

		vueMemoria = initMenuItem(K.VW_MEMORIA, "", ' ',
				"view.memoria", "",
				evt -> {
					mainFrame.showAndFocus(VIEWNAME.MEMORIA);
				});
		menuView.add(vueMemoria);

		vueStoryboard = initMenuItem(K.STAMP, "", ' ',
				"view.storyboard", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.STORYBOARD));
		menuView.add(vueStoryboard);

		vueStorymap = initMenuItem(K.VW_STORYMAP, "", ' ',
				"view.storymap", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.STORYMAP));
		menuView.add(vueStorymap);

		menuView.add(new JPopupMenu.Separator());

		toolsPlan = initMenuItem(K.VW_PLAN, "", ' ',
				"view.plan", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.PLAN));
		menuView.add(toolsPlan);

		vueTree = initMenuItem(K.VW_TREE, "", ' ', "tree", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.TREE));
		menuView.add(vueTree);

		vueInfo = initMenuItem(K.INFO, "", ' ',
				"info.title", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.INFO));
		menuView.add(vueInfo);

		vueNavigation = initMenuItem(K.COMPASS, "", ' ',
				"navigation", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.NAVIGATION));
		menuView.add(vueNavigation);

		vueMemos = initMenuItem(K.ENT_MEMO, "", ' ',
				"memo", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.MEMOS));
		menuView.add(vueMemos);

		/*menuView.addSeparator();
		vueOptions = initMenuItem(K.OPTIONS, "", ' ', "options", "",
				evt -> OptionsDlg.show(mainFrame, null));
		menuView.add(vueOptions);*/
		menuBar.add(menuView);
	}

	private void initMenuCharts() {
		//LOG.trace(TT+"initMenuCharts()");
		menuCharts = new JMenu(I18N.getMsg("charts"));

		chartAttributes = initMenuItem(K.COLUMNS, "", ' ',
				"attribute.list", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.ATTRIBUTESLIST));
		if (App.isDev()) {
			menuCharts.add(chartAttributes);
		}

		chartPersonsByDate = initMenuItem(K.VW_CHART, "", ' ',
				"tools.charts.overall.character.date", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.CHART_PERSONS_BY_DATE));
		menuCharts.add(chartPersonsByDate);

		chartPersonsByScene = initMenuItem(K.VW_CHART, "", ' ',
				"tools.charts.part.character.scene", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.CHART_PERSONS_BY_SCENE));
		menuCharts.add(chartPersonsByScene);

		chartWIWW = initMenuItem(K.VW_CHART, "", ' ',
				"tools.charts.overall.whoIsWhereWhen", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.CHART_WIWW));
		menuCharts.add(chartWIWW);

		chartStrands = initMenuItem(K.VW_CHART, "", ' ',
				"tools.charts.overall.strand.date", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.CHART_STRANDS_BY_DATE));
		menuCharts.add(chartStrands);
		menuCharts.add(new JPopupMenu.Separator());

		chartPersons = initMenuItem(K.VW_CHART, "", ' ',
				"tools.charts.overall.character.occurrence", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.CHART_OCCURRENCE_OF_PERSONS));
		menuCharts.add(chartPersons);

		chartLocations = initMenuItem(K.VW_CHART, "", ' ',
				"tools.charts.overall.location.occurrence", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.CHART_OCCURRENCE_OF_LOCATIONS));
		menuCharts.add(chartLocations);

		chartItems = initMenuItem(K.VW_CHART, "", ' ',
				"tools.charts.overall.item.occurrence", "",
				evt -> mainFrame.showAndFocus(VIEWNAME.CHART_OCCURRENCE_OF_ITEMS));
		menuCharts.add(chartItems);

		menuBar.add(menuCharts);
	}

	private void initMenuWindow() {
		//LOG.trace(TT+"initMenuWindow()");
		menuWindow = new JMenu(I18N.getMsg("window"));

		windowLoadLayout = new JMenu(I18N.getMsg("docking.load.layout"));
		menuWindow.add(windowLoadLayout);

		windowSaveLayout = initMenuItem("docking.save.layout", evt -> mainFrame.windowSaveLayoutAction());
		menuWindow.add(windowSaveLayout);

		windowPredefinedLayout = new JMenu(I18N.getMsg("docking.predefined"));
		DockUtil.layoutGetMenu(mainFrame, windowPredefinedLayout);
		menuWindow.add(windowPredefinedLayout);

		windowDefaultLayout = initMenuItem("layout.default");
		windowDefaultLayout.addActionListener(evt -> DockUtil.layoutSet(mainFrame, "default"));

		windowResetLayout = initMenuItem("docking.reset.layout", evt -> {
			SwingUtil.setWaitingCursor(mainFrame);
			mainFrame.setDefaultLayout();
			SwingUtil.setDefaultCursor(mainFrame);
		});
		menuWindow.add(windowResetLayout);

		windowRefresh = initMenuItem(K.EMPTY, "F5", ' ',
				"refresh", "",
				evt -> mainFrame.refresh());
		menuWindow.add(windowRefresh);

		menuBar.add(menuWindow);
	}

	private void initMenuHelp() {
		//LOG.trace(TT+"initMenuHelp()");
		menuHelp = new JMenu(I18N.getMsg("help"));

		helpDoc = initMenuItem(K.EMPTY, "F1", ' ',
				"help.doc", "helpDoc",
				evt -> Net.openBrowser(Net.URL.DOC.toString()));
		menuHelp.add(helpDoc);

		helpFaq = initMenuItem("help.faq",
				evt -> Net.openBrowser(Net.URL.FAQ.toString()));
		menuHelp.add(helpFaq);

		helpHome = initMenuItem("help.homepage",
				evt -> Net.openBrowser(Net.URL.HOME.toString()));
		menuHelp.add(helpHome);

		helpReportBug = initMenuItem("help.bug",
				evt -> Net.openBrowser(Net.URL.REPORTBUG.toString()));
		menuHelp.add(helpReportBug);

		helpAbout = initMenuItem("help.about",
				evt -> AboutDlg.show(mainFrame));
		menuHelp.add(helpAbout);
		menuHelp.add(new JPopupMenu.Separator());

		editSpellChecker = initMenuItem("preferences.spelling");
		editSpellChecker.setIcon(new ImageIcon(getClass().getResource("/jortho/icon.png")));
		editSpellChecker.addActionListener(evt -> {
			SpellDlg dlg = new SpellDlg(mainFrame);
			dlg.setVisible(true);
		});
		menuHelp.add(editSpellChecker);

		editAssistant = initMenuItem(K.ASSISTANT, "", ' ', "assistant",
				"assistant", e -> {
					AssistantWizardDlg dlg = new AssistantWizardDlg(mainFrame);
					dlg.setVisible(true);
				});
		menuHelp.add(editAssistant);

		menuHelp.add(new JPopupMenu.Separator());

		helpCheckUpdates = initMenuItem("help.update",
				evt -> {
					if (Updater.checkForUpdate(true)) {
						JOptionPane.showMessageDialog(mainFrame,
								I18N.getMsg("update.no.text"),
								I18N.getMsg("update.no.title"),
								JOptionPane.INFORMATION_MESSAGE);
					}
				});
		menuHelp.add(helpCheckUpdates);

		JCheckBoxMenuItem helpTrace = new JCheckBoxMenuItem();
		helpTrace.setFont(App.getInstance().fontGetDefault());
		helpTrace.setMnemonic('T');
		helpTrace.setText(I18N.getMsg("help.trace"));
		helpTrace.addActionListener(evt -> {
			LOG.setTrace(!LOG.getTrace());
			helpTrace.setSelected(LOG.getTrace());
		});
		menuHelp.add(helpTrace);

		devTest = initMenuItem("!Dev-test", evt -> {
			testFunction = (devTest.getText().contains("Dev"));
			devTest.setText((testFunction ? "Test in progress" : "Dev-Test"));
			//function to test

			//restore dev menu
			devTest.setText((testFunction ? "Test in progress" : "Dev-Test"));
		});
		if (App.isDev()) {
			menuHelp.add(devTest);
		}

		helpTranslate = initMenuItem("!Translate UI", evt -> I18NDlg.show(mainFrame));
		menuHelp.add(helpTranslate);

		menuBar.add(menuHelp);
	}

	public void setMenuForBlank() {
		//LOG.trace(TT+"initMenuForBlank()");
		// hide menus from MenuBar
		JMenu menus[] = {
			menuNewEntity, menuTables, menuPrimaryObjects, menuSecondaryObjects,
			menuCharts, menuCharts, menuView, menuWindow, tools
		};
		for (javax.swing.JMenu m : menus) {
			if (m != null) {
				m.setVisible(false);
			}
		}
		JMenuItem submenus[] = {
			editCopyBlurb, editCopyBook, editCopyEntity,
			fileClose, menuFileExport,
			fileProperties, fileRename, fileSave, fileSaveAs, menuFileExport, fileImportXml,
			fileBackupNew, fileAssistant
		};
		for (JMenuItem m : submenus) {
			if (m != null) {
				m.setVisible(false);
			}
		}
		JSeparator separators[] = {
			separator0, separator1, separator2//, separator3
		};
		for (JSeparator sep : separators) {
			sep.setVisible(false);
		}
		for (Component comp : toolBar.getComponents()) {
			if (comp instanceof JButton) {
				((JButton) comp).setVisible(false);
			}
		}
		btFileNew.setVisible(true);
		btFileOpen.setVisible(true);
		devTest.setVisible(App.isDev());
	}

	public void enableSave(boolean b) {
		//LOG.trace(TT+"enableSave()");
		if (fileSave != null) {
			fileSave.setEnabled(b);
		}
		if (btFileSave != null) {
			btFileSave.setEnabled(b);
		}
	}

	public JMenu getMenuFileOpenRecent() {
		return menuFileOpenRecent;
	}

	public void reloadToolbar() {
		if (toolBar != null) {
			toolBar.removeAll();
		}
		initToolbar();
		setToolbar();
		if (fileExportStoryboard != null && mainFrame.getBook() != null) {
			fileExportStoryboard.setEnabled(mainFrame.getBook().isScenario());
		}
	}

	@Override
	public void mouseClicked(MouseEvent evt) {
		//LOG.trace("mouseClicked(evt=" + evt.toString());
		if (SwingUtilities.isRightMouseButton(evt)) {
			ToolBarDlg dlg = new ToolBarDlg(mainFrame);
			dlg.setVisible(true);
			if (!dlg.isCanceled()) {
				setToolbar();
			}
			evt.consume();
			//LOG.trace(" after mouseClicked evt is=" + evt.toString());
		}
	}

	@Override
	public void mousePressed(MouseEvent e) {
		// empty
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// empty
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// empty
	}

	@Override
	public void mouseExited(MouseEvent e) {
		// empty
	}

	void setLinks() {
		if (mainFrame == null || toolsPersonsLinks == null) {
			return;
		}
		toolsPersonsLinks.setEnabled(!EntityUtil.findEntities(mainFrame, Book.TYPE.PERSON).isEmpty());
		toolsLocationsLinks.setEnabled(!EntityUtil.findEntities(mainFrame, Book.TYPE.LOCATION).isEmpty());
		toolsItemsLinks.setEnabled(!EntityUtil.findEntities(mainFrame, Book.TYPE.ITEM).isEmpty());
	}

	public void enablePaste() {
		editPaste.setVisible(true);
	}

	public void hideToolbar() {
		toolBar.setVisible(false);
	}

}
