/*
Storybook: Scene-based software for novelists and authors.
Copyright (C) 2008 - 2012 Martin Mustun

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.ui.panel.tree;

import java.awt.Component;
import javax.swing.Icon;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import storybook.App;
import i18n.I18N;
import storybook.model.EntityUtil;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.Person;
import storybook.model.hbn.entity.Scene;
import storybook.model.hbn.entity.Status;
import storybook.toolkit.TextUtil;
import storybook.toolkit.html.Html;

@SuppressWarnings("serial")
class EntityTreeCellRenderer extends DefaultTreeCellRenderer {

	public EntityTreeCellRenderer() {
		super();
	}

	@Override
	public Component getTreeCellRendererComponent(JTree tree, Object value,
			boolean sel, boolean expanded, boolean leaf, int row,
			boolean hasFocus) {
		DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
		Object object = node.getUserObject();
		if (leaf) {
			if (object instanceof Person && ((Person) object).getGender() != null) {
				setLeafIcon(((Person) object).getGender().getIcon());
			} else if (object instanceof Scene) {
				setLeafIcon(Status.getStatusIcon(((Scene) object).getStatus()));
			} else if (object instanceof AbstractEntity) {
				Icon icon = EntityUtil.getEntityIcon((AbstractEntity) object);
				setLeafIcon(icon);
			} else {
				setLeafIcon(null);
			}
		}
		super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
		if (!leaf && object instanceof AbstractEntity) {
			Icon icon = EntityUtil.getEntityIcon((AbstractEntity) object);
			setIcon(icon);
			this.setText(((AbstractEntity) object).getName());
		}
		if (leaf && object instanceof AbstractEntity) {
			AbstractEntity entity = (AbstractEntity) object;
			if (entity.getId() == -1L && object instanceof Chapter) {
				this.setText(I18N.getMsg("scenes.unassigned"));
			} else {
				String txt = (entity.hasNotes() ? "*" : "") + entity.getName();
				if (App.getInstance().preferences.treeviewGetTrunc()) {
					txt = TextUtil.ellipsize(txt, App.getInstance().preferences.treeviewGetChar());
				}
				this.setText(txt);
			}
		}
		setNotes(object);
		return this;
	}

	private void setNotes(Object object) {
		String texte = null;
		if (object instanceof AbstractEntity) {
			AbstractEntity entity = (AbstractEntity) object;
			texte = entity.getNotes();
		}
		if (!Html.htmlToText(texte).equals("")) {
			setToolTipText(Html.HTML_B + texte + Html.HTML_E);
		} else {
			setToolTipText(null);
		}

	}
}
