/*
 * Copyright (C) 2023 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.ui.panel.typist;

import i18n.I18N;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeEvent;
import java.io.File;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.miginfocom.swing.MigLayout;
import resources.icons.ICONS;
import resources.icons.IconUtil;
import shef.ShefEditor;
import storybook.exim.exporter.ExportComments;
import storybook.exim.importer.ImportComments;
import storybook.model.EntityUtil;
import storybook.model.book.Book;
import storybook.model.hbn.entity.Endnote;
import storybook.model.hbn.entity.Scene;
import storybook.toolkit.LOG;
import storybook.toolkit.file.IOUtil;
import storybook.toolkit.html.Html;
import storybook.toolkit.swing.SwingUtil;
import storybook.ui.MIG;
import storybook.ui.MainFrame;
import storybook.ui.panel.AbstractPanel;
import storybook.ui.renderer.lcr.LCREntity;

/**
 *
 * @author favdb
 */
public class CommentsPanel extends AbstractPanel {

	private static final String TT = "CommentPanel";

	private ShefEditor htTexte;
	private JList leftList;
	private final Scene scene;
	private List<Endnote> comments;
	private final TypistPanel typist;
	private JButton btExport, btImport;

	public CommentsPanel(MainFrame mainFrame, TypistPanel typist) {
		super(mainFrame);
		this.typist = typist;
		this.scene = typist.getScene();
		initAll();
	}

	@Override
	@SuppressWarnings("unchecked")
	public void init() {
		comments = Endnote.find(mainFrame, 1, scene);
	}

	@Override
	public void initUi() {
		//LOG.trace(TT + ".initUi()");
		setBorder(BorderFactory.createTitledBorder(I18N.getColonMsg("comments")));
		setLayout(new MigLayout(MIG.get(MIG.INS1, MIG.WRAP), "[10%][]"));
		Dimension dim = SwingUtil.getScreenSize();
		dim.height = 16;
		dim.width = dim.width / 3;
		setPreferredSize(dim);
		setMinimumSize(dim);
		Dimension maxS = Toolkit.getDefaultToolkit().getScreenSize();
		maxS.width = dim.width;
		setMaximumSize(maxS);
		this.add(initTB(), MIG.SPAN);
		this.add(initLeftPanel(), MIG.GROWY);
		htTexte = new ShefEditor("none, nothing", "");
		JScrollPane scroller = new JScrollPane(htTexte);
		SwingUtil.setUnitIncrement(scroller);
		scroller.setPreferredSize(SwingUtil.getScreenSize());
		this.add(scroller, MIG.GROW);
		refresh();
	}

	@SuppressWarnings("unchecked")
	private JScrollPane initLeftPanel() {
		leftList = new JList();
		leftList.setModel(new DefaultListModel());
		leftList.setCellRenderer(new LCREntity());
		leftList.addMouseListener(new CommentButton());
		JLabel label = new JLabel("[00]");
		FontMetrics fm = label.getFontMetrics(label.getFont());
		int minW = fm.charWidth('W'), minH = fm.getHeight();
		Dimension dimLeft = new Dimension(3 * minW, 4 * minH);
		JScrollPane scroll = new JScrollPane(leftList);
		scroll.setBorder(BorderFactory.createLineBorder(Color.RED));
		scroll.setMinimumSize(dimLeft);
		scroll.setMaximumSize(Toolkit.getDefaultToolkit().getScreenSize());
		SwingUtil.setMaxPreferredSize(scroll);
		return scroll;
	}

	private JToolBar initTB() {
		JToolBar tb = new JToolBar();
		JButton bten = new JButton(IconUtil.getIconSmall(ICONS.K.CHAR_COMMENT));
		bten.setToolTipText(I18N.getMsg("comment.create"));
		bten.addActionListener(e -> commentAdd());
		tb.add(bten);
		btExport = new JButton(IconUtil.getIconSmall(ICONS.K.F_EXPORT));
		btExport.setToolTipText(I18N.getMsg("comments.export"));
		btExport.addActionListener(e -> commentsExport());
		tb.add(btExport);
		btImport = new JButton(IconUtil.getIconSmall(ICONS.K.F_IMPORT));
		btImport.setToolTipText(I18N.getMsg("comments.import"));
		btImport.addActionListener(e -> commentsImport());
		tb.add(btImport);
		return tb;
	}

	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		//empty
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		//empty
	}

	private void commentAdd() {
		Endnote en = Endnote.createEndnote(mainFrame, 1, scene, typist.getHtTexte());
		if (en != null) {
			typist.setModified();
		}
		refresh();
	}

	private void commentRemove(Endnote comment) {
		//LOG.trace(TT + ".commentRemove(comment=" + LOG.traceEntity(comment) + ")");
		if (comment != null) {
			String str = Endnote.linkTo("", comment);
			typist.getHtTexte().wysEditorGet().removeTag(str);
			mainFrame.getBookModel().deleteEntity(comment);
			refresh();
		}
	}

	/**
	 * popup menu to select the format to export
	 */
	private void commentsExport() {
		JPopupMenu menu = new JPopupMenu();
		menu.add(new JLabel(I18N.getMsg("comments.export_to")));
		JMenuItem item;
		item = new JMenuItem("TXT");
		item.addActionListener(e -> ExportComments.exec(mainFrame, this, "txt"));
		menu.add(item);
		item = new JMenuItem("HTML");
		item.addActionListener(e -> ExportComments.exec(mainFrame, this, "html"));
		menu.add(item);
		item = new JMenuItem("XML");
		item.addActionListener(e -> ExportComments.exec(mainFrame, this, "xml"));
		menu.add(item);
		menu.show(btExport, btExport.getX(), btExport.getY());
	}

	/**
	 * select the file to import and select if existing coments	are to be remove
	 */
	private void commentsImport() {
		//select a file to import (XML only)
		File file = IOUtil.fileSelect(this, "", "xml",
				I18N.getMsg("file.type.xml") + " (*.xml)", "import");
		// ask if add imported Comments or Replace all
		ImportComments.exec(mainFrame, file);
	}

	@Override
	@SuppressWarnings("unchecked")
	public void refresh() {
		comments = Endnote.find(mainFrame, 1, scene);
		int idx = leftList.getSelectedIndex();
		@SuppressWarnings("unchecked")
		DefaultListModel<Endnote> model = (DefaultListModel) leftList.getModel();
		model.removeAllElements();
		StringBuilder b = new StringBuilder();
		for (Endnote en : comments) {
			model.addElement(en);
			b.append("<a class=\"comment\">")
					.append(String.format("[%d] ", en.getNumber()))
					.append("</a>");
			b.append("<div id=\"").append(en.getId().toString()).append("\">")
					.append(en.getNotes())
					.append("</div>");
			b.append(Html.HR);
		}
		htTexte.setText(b.toString());
		if (idx < 0) {
			if (!model.isEmpty()) {
				leftList.setSelectedIndex(0);
			}
		} else {
			if (idx < model.getSize()) {
				leftList.setSelectedIndex(idx);
			}
		}
		leftList.revalidate();
	}

	public void save() {
		//LOG.trace(TT + ".save()");
		String str = htTexte.getText();
		Document doc = Jsoup.parse(str);
		Elements divs = doc.getElementsByTag("div");
		for (int i = 0; i < divs.size(); i++) {
			Element elem = divs.get(i);
			Long id = Long.valueOf(elem.id());
			Endnote endnote = (Endnote) EntityUtil.findEntity(mainFrame, Book.TYPE.ENDNOTE, id);
			if (endnote == null) {
				//LOG.trace("endnote not find, it's to create");
				endnote = new Endnote(1, scene);
			}
			//set the text of the comment
			endnote.setNotes(elem.html());
			//LOG.trace("endnote text:\n" + endnote.getNotes());
			//update the comment
			mainFrame.getBookController().updateEntity(endnote);
		}
	}

	private class CommentButton implements MouseListener {

		public CommentButton() {
		}

		@Override
		public void mouseClicked(MouseEvent evt) {
			//LOG.trace("CommentButton.mouseClicked()");
			if (SwingUtilities.isRightMouseButton(evt)) {
				Endnote endnote = (Endnote) leftList.getSelectedValue();
				if (endnote != null) {
					JPopupMenu pop = new JPopupMenu();
					JMenuItem item = new JMenuItem(I18N.getMsg("delete"));
					item.setIcon(IconUtil.getIconSmall(ICONS.K.REMOVE));
					item.setToolTipText(I18N.getMsg("comment.remove"));
					item.addActionListener(e -> commentRemove(endnote));
					pop.add(item);
					pop.show(leftList, evt.getX(), evt.getY());
				}
			} else {
				LOG.trace("TODO normal click for comment");
			}
			evt.consume();
		}

		@Override
		public void mousePressed(MouseEvent e) {
			// empty
		}

		@Override
		public void mouseReleased(MouseEvent e) {
			// empty
		}

		@Override
		public void mouseEntered(MouseEvent e) {
			// empty
		}

		@Override
		public void mouseExited(MouseEvent e) {
			// empty
		}
	}

}
