/*
 * Copyright (C) 2017 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.ui.panel.typist;

import i18n.I18N;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.border.EtchedBorder;
import org.hibernate.Session;
import org.miginfocom.swing.MigLayout;
import resources.icons.ICONS;
import resources.icons.IconUtil;
import shef.ShefEditor;
import storybook.App;
import storybook.Const;
import storybook.Pref;
import storybook.edit.Editor;
import storybook.edit.Intensity;
import storybook.model.EntityUtil;
import storybook.model.Model;
import storybook.model.book.Book;
import storybook.model.hbn.dao.SceneDAOImpl;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.Endnote;
import storybook.model.hbn.entity.Idea;
import storybook.model.hbn.entity.Scene;
import storybook.model.hbn.entity.Strand;
import storybook.toolkit.ShefUtil;
import storybook.toolkit.StringUtil;
import storybook.toolkit.file.TempUtil;
import storybook.toolkit.swing.ColorIcon;
import storybook.toolkit.swing.ColorLabel;
import storybook.toolkit.swing.SwingUtil;
import storybook.ui.MIG;
import storybook.ui.MainFrame;
import storybook.ui.Ui;
import storybook.ui.combobox.ComboModelSceneState;
import storybook.ui.dialog.ChapterSelectDlg;
import storybook.ui.dialog.EntitiesListDlg;
import storybook.ui.dialog.ListEndnotesDlg;
import storybook.ui.panel.AbstractPanel;
import storybook.ui.renderer.lcr.LCRSceneState;

public class TypistPanel extends AbstractPanel {

	private static final String TT = "TypistPanel";

	private int origin;
	private Scene scene = null;
	private Dimension screenSize;
	private int defaultHeight, saveSplit;
	private ShefEditor htTexte;
	private TypistInfo panelInfo;
	private JComboBox cbScenes, cbStatus;
	private JButton btNewScene,
			btFirst, btPrior, btNext, btLast,
			btScreen, btChapter, btTitle, btShowInfo,
			btStrand, btHide, btIgnore, btSave;
	private JPanel statusbar;
	private Intensity intensity;
	private JPanel strandsList, tb11, tb2;
	private boolean toRefresh, hiden = false, modified;
	private JButton btEndnotes;
	private List<Endnote> endnotes;
	private JSplitPane splitPane;
	private JScrollPane scrollComments;
	private CommentsPanel commentsPanel;
	private JCheckBox ckReread;

	public TypistPanel(MainFrame mainFrame) {
		super(mainFrame);
		this.scene = getFirstScene();
		initAll();
	}

	public TypistPanel(MainFrame mainFrame, Scene scene) {
		super(mainFrame);
		if (scene == null) {
			this.scene = getFirstScene();
		} else {
			this.scene = (Scene) TempUtil.check(mainFrame, scene, !Const.IGNORE);
		}
		initAll();
	}

	@Override
	public void init() {
		this.withPart = false;
		this.removeAll();
		toRefresh = false;
		App.getInstance().preferences.setBoolean(Pref.KEY.TYPIST_USE, true);
		hiden = App.getInstance().preferences.getBoolean(Pref.KEY.TYPIST_SHOWBAR);
	}

	@Override
	public void initUi() {
		//LOG.trace(TT + ".initUi()");
		setLayout(new MigLayout(MIG.get(MIG.INS1, MIG.HIDEMODE2, MIG.FILL)));
		screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		defaultHeight = 32;
		add(initToolbar(), MIG.get(MIG.SPAN, MIG.GROWX));
		htTexte = new ShefEditor("lang_all, allow, colored", scene.getSummary());
		ShefUtil.setToFileButton(mainFrame, htTexte);
		ShefUtil.setFromFileButton(mainFrame, htTexte);
		panelInfo = new TypistInfo(this, null);
		splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, htTexte, panelInfo);
		splitPane.setResizeWeight(0.85);
		splitPane.setPreferredSize(Toolkit.getDefaultToolkit().getScreenSize());
		add(splitPane, MIG.get(MIG.GROW));
		commentsInit();
		add(commentsPanel, MIG.GROWY);
		commentsPanel.setVisible(book.getReread());
		add(initStatusbar(), MIG.SPAN);
		if (!App.getInstance().preferences.getBoolean(Pref.KEY.TYPIST_SHOWINFO)) {
			infosShowHide();
		}
		endnoteInit(this);
		setScene(getFirstScene());
		setHiden(hiden);
		commentsPanel.setVisible(App.isDev());
		ckReread.setVisible(App.isDev());
	}

	@Override
	@SuppressWarnings("unchecked")
	public JToolBar initToolbar() {
		//super.initToolbar();
		toolbar = new JToolBar();
		toolbar.setLayout(new MigLayout(MIG.get(MIG.FILLX, MIG.INS0, MIG.HIDEMODE3)));
		toolbar.setName("TypistToolbar");
		toolbar.add(initTB1(), MIG.GROWX);
		tb11 = initTB11();
		toolbar.add(tb11, MIG.get(MIG.SPAN, MIG.RIGHT));
		tb2 = initTB2();
		toolbar.add(tb2, MIG.get(MIG.GROWX, MIG.NEWLINE));
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		dim.height = IconUtil.getDefSize() * 3;
		toolbar.setPreferredSize(dim);
		return (toolbar);
	}

	private JPanel initTB1() {
		JPanel tb1 = new JPanel(new MigLayout());
		btHide = Ui.initButton("btHide", "", ICONS.K.DP_UP, "typist.show_bar", (evt -> setHiden(!hiden)));
		tb1.add(btHide);
		cbScenes = Ui.initComboBox("cbScenes", "",
				EntityUtil.findEntities(mainFrame, Book.TYPE.SCENE), scene, !EMPTY, !ALL, this);
		cbScenes.setMaximumRowCount(15);
		tb1.add(cbScenes);
		btNewScene = Ui.initButton("btNew", "", ICONS.K.ADD, "scene.new", this);
		tb1.add(btNewScene);
		tb1.add(new JLabel(" "));
		//navigation dans les scènes
		btFirst = Ui.initButton("btFirst", "", ICONS.K.NAV_FIRST, "export.nav.first", this);
		tb1.add(btFirst);
		btPrior = Ui.initButton("btPrior", "", ICONS.K.NAV_PREV, "export.nav.prior", this);
		tb1.add(btPrior);
		btNext = Ui.initButton("btNext", "", ICONS.K.NAV_NEXT, "export.nav.next", this);
		tb1.add(btNext);
		btLast = Ui.initButton("btLast", "", ICONS.K.NAV_LAST, "export.nav.last", this);
		tb1.add(btLast);
		tb1.add(new JLabel(" "));
		//ignorer les modifications
		btIgnore = Ui.initButton("btIgnore", "", ICONS.K.CANCEL, "discard.changes", this);
		tb1.add(btIgnore);
		//enregistrer les modifications
		btSave = Ui.initButton("btSave", "", ICONS.K.F_SAVE, "file.save", this);
		tb1.add(btSave);
		tb1.add(new JLabel(" "));
		btShowInfo = Ui.initButton("btInfo", "", ICONS.K.INFO_HIDE, "typist.show_infos", this);
		tb1.add(btShowInfo);
		ckReread = Ui.initCheckBox("cbComment",
				"comments",
				book.getReread(),
				e -> changeCkComment());
		tb1.add(ckReread);
		return tb1;
	}

	private JPanel initTB11() {
		JPanel tb = new JPanel(new MigLayout(MIG.INS0));
		tb.setBorder(BorderFactory.createEmptyBorder());
		// idea
		tb.add(Ui.initButton("btIdea", "", ICONS.K.ENT_IDEA, "foi.new", this));
		tb.add(new JSeparator(), MIG.GROWX);
		//sortie du mode typist
		btScreen = Ui.initButton("btScreen", "", ICONS.K.SCREEN_NORMAL, "screen.normal", this);
		tb.add(btScreen, MIG.RIGHT);
		// exit
		tb.add(Ui.initButton("btExit", "", ICONS.K.EXIT, "exit", this));
		return tb;
	}

	@SuppressWarnings("unchecked")
	private JPanel initTB2() {
		JPanel tb = new JPanel(new MigLayout(MIG.get(MIG.INS0)));
		//titre du chapitre
		tb.add(initLabel("chapter", true));
		btChapter = Ui.initButton("btChapter", "chapter", ICONS.K.ENT_CHAPTER, "change", this);
		tb.add(btChapter);
		//titre de la scène
		tb.add(initLabel("scene", true));
		btTitle = Ui.initButton("btTitle", "", ICONS.K.ENT_SCENE, "change", this);
		tb.add(btTitle);
		cbStatus = new JComboBox(new ComboModelSceneState(false));
		cbStatus.setName(TypistScenario.CMD.CB_STATUS.toString());
		cbStatus.setSelectedIndex(scene.getStatus());
		cbStatus.setRenderer(new LCRSceneState());
		tb.add(initLabel("status", true));
		tb.add(cbStatus);
		intensity = new Intensity(Intensity.LIST, scene.getIntensity());
		tb.add(initLabel("intensity", true));
		tb.add(intensity);
		tb.add(new JLabel(""), MIG.GROWX);
		tb.add(initLabel("strands", true));
		btStrand = new JButton(new ColorIcon(scene.getStrand().getJColor(), IconUtil.getDefSize()));
		btStrand.setName("btStrand");
		btStrand.setToolTipText(scene.getStrand().getName());
		btStrand.addActionListener(this);
		tb.add(btStrand);
		strandsList = new JPanel(new MigLayout(MIG.INS0));
		strandsList.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
		strandsList.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				strandsAction();
			}
		});
		strandsRefresh();
		//tb.add(initLabel("strands.other", true));
		tb.add(strandsList);
		return tb;
	}

	private JPanel initStatusbar() {
		statusbar = new JPanel();
		statusbar.setName("TypistStatusbar");
		statusbar.setLayout(new MigLayout(MIG.FILLX));
		statusbar.setSize(screenSize.width, defaultHeight);
		return (statusbar);
	}

	public Scene getScene() {
		return this.scene;
	}

	public ShefEditor getHtTexte() {
		return this.htTexte;
	}

	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		// empty
	}

	@Override
	@SuppressWarnings("unchecked")
	public void refresh() {
		if (scene == null) {
			return;
		}
		commentsPanel.refresh();
		if (scene.getSummary() != null) {
			htTexte.setText(scene.getSummary());
			htTexte.setCaretPosition(0);
		} else {
			htTexte.setText("");
		}

		if (scene.hasChapter()) {
			btChapter.setText(scene.getChapter().getName());
		} else {
			btChapter.setText(I18N.getMsg("scene.hasnochapter"));
		}
		if (scene.getTitle() == null || scene.getTitle().isEmpty()) {
			btTitle.setText(I18N.getMsg("scene.hasnotitle"));
		} else {
			btTitle.setText(scene.getTitle());
		}
		cbStatus.setSelectedIndex(scene.getStatus());
		intensity.setValue(scene.getIntensity());
		strandRefresh(scene.getStrand());
		strandsRefresh();
		panelInfo.refreshInfo(scene);
		cbScenes.removeActionListener(this);
		cbScenes.setSelectedItem(scene);
		cbScenes.addActionListener(this);
		origin = htTexte.getText().hashCode();
		mainFrame.sceneSetLast(scene);
		setNav(EntityUtil.findScenes(mainFrame));
		repaint();
	}

	private void setNav(List<Scene> scenes) {
		if (scenes == null || scenes.isEmpty() || toolbar == null) {
			return;
		}
		btFirst.setEnabled(true);
		btPrior.setEnabled(true);
		btNext.setEnabled(true);
		btLast.setEnabled(true);
		if (scene.equals(scenes.get(0))) {
			btFirst.setEnabled(false);
			btPrior.setEnabled(false);
		} else if (scenes.size() > 1 && scene.equals(scenes.get(scenes.size() - 1))) {
			btNext.setEnabled(false);
			btLast.setEnabled(false);
		}
	}

	private Scene getFirstScene() {
		if (scene != null) {
			TempUtil.remove(mainFrame, scene, !Const.FORCE);
		}
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes = dao.findAll();
		if (scenes.isEmpty()) {
			return (null);
		}
		model.commit();
		Scene s = scenes.get(0);
		return (s);
	}

	private Scene getLastScene() {
		if (scene != null) {
			TempUtil.remove(mainFrame, scene, !Const.FORCE);
		}
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes = dao.findAll();
		if (scenes.isEmpty()) {
			return (null);
		}
		Scene s = scenes.get(scenes.size() - 1);
		return (s);
	}

	private boolean textToLarge(String topText) {
		if (topText.length() > 32767) {
			String str = I18N.getMsg("editor.text_too_large");
			JOptionPane.showMessageDialog(this, str,
					I18N.getMsg("editor"), JOptionPane.ERROR_MESSAGE);
			return (true);
		}
		return (false);
	}

	public int askModified() {
		//LOG.trace(TT+".askModified()");
		if (scene != null) {
			String topText = htTexte.getText().replace("--", "—&nbsp;");
			int nhash = htTexte.getText().hashCode();
			if (origin != nhash) {
				if (textToLarge(topText)) {
					return (JOptionPane.CANCEL_OPTION);
				}
				setModified();
			}
		}
		if (modified) {
			final Object[] options = {
				I18N.getMsg("save.changes"),
				I18N.getMsg("discard.changes"),
				I18N.getMsg("cancel")
			};
			String str = I18N.getMsg("save.or.discard.changes") + "\n\n"
					+ EntityUtil.getEntityTitle(scene) + ": "
					+ scene.toString() + "\n\n";
			int n = JOptionPane.showOptionDialog(this, str, I18N.getMsg("save.changes.title"),
					JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.QUESTION_MESSAGE, null, options, options[2]);
			switch (n) {
				case JOptionPane.CANCEL_OPTION:
					return (JOptionPane.CANCEL_OPTION);
				case JOptionPane.YES_OPTION:
					save();
					break;
				case JOptionPane.NO_OPTION:
					break;
				default:
					break;
			}
		}
		return (JOptionPane.YES_OPTION);
	}

	private void newScene() {
		Scene s = new Scene();
		s.setChapter(scene.getChapter());
		JDialog dlg = new JDialog(mainFrame, true);
		dlg.setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
		Editor editor = new Editor(mainFrame, s, dlg);
		dlg.setTitle(I18N.getMsg("editor"));
		Dimension pos = SwingUtil.getDlgPosition(s);
		dlg.add(editor);
		Dimension size = SwingUtil.getDlgSize(s);
		if (size != null) {
			dlg.setSize(size.height, size.width);
		} else {
			dlg.setSize(this.getWidth() / 2, 680);
		}
		if (pos != null) {
			dlg.setLocation(pos.height, pos.width);
		} else {
			dlg.setLocationRelativeTo(this);
		}
		dlg.setVisible(true);
		if (editor.canceled) {
			return;
		}
		TempUtil.remove(mainFrame, s, Const.FORCE);
		if (editor.entity != null) {
			scene = (Scene) editor.entity;
			refreshCbScenes();
			resetModified();
			cbScenes.setSelectedItem(scene);
		}
		SwingUtil.saveDlgPosition(dlg, s);
		refresh();
	}

	@SuppressWarnings("unchecked")
	private void refreshCbScenes() {
		cbScenes.removeActionListener(this);
		DefaultComboBoxModel combo = (DefaultComboBoxModel) cbScenes.getModel();
		combo.removeAllElements();
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Scene> scenes = dao.findAll();
		for (Scene entity : scenes) {
			combo.addElement(entity);
		}
		cbScenes.addActionListener(this);
	}

	private void strandChange(String name) {
		String s[] = name.split("_");
		if (s.length > 1 && StringUtil.isNumeric(s[1])) {
			Long id = Long.valueOf(s[1]);
			Strand strand = (Strand) EntityUtil.findEntity(mainFrame, Book.TYPE.STRAND, id);
			if (strand != null) {
				scene.setStrand(strand);
				btStrand.setIcon(new ColorIcon(strand.getJColor(), IconUtil.getDefSize()));
				btStrand.setToolTipText(strand.getName());
				setModified();
			}
		}
	}

	private void chapterChange() {
		ChapterSelectDlg dlg = new ChapterSelectDlg(mainFrame, scene.getChapter());
		dlg.setVisible(true);
		if (!dlg.isCanceled()) {
			Chapter chapter = dlg.getSelectedChapter();
			scene.setChapter(chapter);
			btChapter.setText(chapter.getName());
			setModified();
		}
	}

	private void changeSceneTitle() {
		String s = (String) JOptionPane.showInputDialog(
				this, "", I18N.getMsg("title"),
				JOptionPane.PLAIN_MESSAGE, null, null, scene.getTitle());
		if ((s != null) && (s.length() > 0)) {
			scene.setTitle(s);
			btTitle.setText(s);
			setModified();
		}
	}

	private boolean save() {
		//LOG.trace(TT + ".save()");
		String topText = htTexte.getText();
		topText = topText.replace("--", "—&nbsp;");
		if (textToLarge(topText)) {
			return (false);
		}
		scene.setSummary(topText);
		scene.setStatus(cbStatus.getSelectedIndex());
		scene.setIntensity(intensity.getValue());
		mainFrame.getBookController().updateEntity(scene);
		if (ckReread.isSelected()) {
			commentsPanel.save();
			Endnote.renumber(mainFrame, 1);
		}
		toRefresh = true;
		resetModified();
		setNav(EntityUtil.findScenes(mainFrame));
		return (true);
	}

	public void setButtons() {
		if (origin != htTexte.getText().hashCode()) {
			modified = true;
		}
		if (toolbar != null) {
			btSave.setEnabled(modified);
			btIgnore.setEnabled(modified);
		}
	}

	public void setModified() {
		modified = true;
		setButtons();
	}

	public void resetModified() {
		modified = false;
		this.origin = htTexte.getText().hashCode();
		setButtons();
	}

	public void closeScene() {
		if (scene != null) {
			TempUtil.remove(mainFrame, scene, Const.FORCE);
		}
	}

	private void close() {
		closeScene();
		mainFrame.typistActivate();
	}

	public void tempSave() {
		Scene svScene = new Scene();
		EntityUtil.copyEntityProperties(mainFrame, scene, svScene);
		scene.setSummary(htTexte.getText());
		TempUtil.write(mainFrame, scene);
		EntityUtil.copyEntityProperties(mainFrame, svScene, scene);
	}

	@Override
	public void actionPerformed(ActionEvent evt) {
		//LOG.trace(TT + ".actionPerformed(evt=" + evt.toString() + ")");
		JComponent source = (JComponent) evt.getSource();
		if (source.getName().startsWith("strand_")) {
			strandChange(source.getName());
			return;
		}
		switch (source.getName()) {
			case "btChapter":
				chapterChange();
				break;
			case "btExit":
				if (!save()) {
					break;
				}
				mainFrame.saveFile(true);
				SwingUtilities.invokeLater(() -> {
					mainFrame.close(true);
				});
				break;
			case "btIdea":
				mainFrame.showEditorAsDialog(new Idea());
				break;
			case "btIgnore":
				refresh();
				break;
			case "btNewPerson":
				break;
			case "btNewLocation":
				break;
			case "btNewItem":
				break;
			case "btNewPlot":
				break;
			case "btFirst":
				if (askModified() == JOptionPane.CANCEL_OPTION) {
					return;
				}
				setScene(getFirstScene());
				break;
			case "btLast":
				if (askModified() == JOptionPane.CANCEL_OPTION) {
					return;
				}
				setScene(getLastScene());
				break;
			case "btPrior":
				if (askModified() == JOptionPane.CANCEL_OPTION) {
					return;
				}
				setNextScene(-1);
				break;
			case "btNext":
				if (askModified() == JOptionPane.CANCEL_OPTION) {
					return;
				}
				setNextScene(1);
				break;
			case "btNew":
				if (askModified() == JOptionPane.CANCEL_OPTION) {
					return;
				}
				newScene();
				break;
			case "btSave":
				save();
				break;
			case "btStrand":
				strandSelect(evt);
				break;
			case "strand.new":
				strandNew();
				break;
			case "btInfo":
				infosShowHide();
				App.getInstance().preferences.setBoolean(Pref.KEY.TYPIST_SHOWINFO, panelInfo.isVisible());
				mainFrame.setUpdated();
				break;
			case "btScreen":
				if (askModified() == JOptionPane.CANCEL_OPTION) {
					return;
				}
				if (toRefresh) {
					SwingUtil.setWaitingCursor(this);
					mainFrame.refresh();
					SwingUtil.setDefaultCursor(this);
				}
				App.getInstance().preferences.setBoolean(Pref.KEY.TYPIST_USE, false);
				if (modified) {
					mainFrame.setUpdated();
				}
				close();
				break;
			case "btStage":
				break;
			case "btTitle":
				changeSceneTitle();
				break;
			case "cbScenes":
				if (askModified() == JOptionPane.CANCEL_OPTION) {
					return;
				}
				if (cbScenes.getSelectedItem() != null) {
					Scene s = (Scene) cbScenes.getSelectedItem();
					if (!s.equals(scene)) {
						setScene(s);
					}
				}
				break;
			case "cbStage":
				break;
			case "cbStatus":
				setModified();
				break;
			default:
				break;
		}
	}

	private void infosShowHide() {
		if (panelInfo.isVisible()) {
			saveSplit = splitPane.getDividerLocation();
		}
		panelInfo.setVisible(!panelInfo.isVisible());
		if (panelInfo.isVisible()) {
			btShowInfo.setIcon(IconUtil.getIconSmall(ICONS.K.INFO_HIDE));
			splitPane.setDividerLocation(saveSplit);
		} else {
			btShowInfo.setIcon(IconUtil.getIconSmall(ICONS.K.INFO));
		}
	}

	private void setScene(Scene s) {
		scene = s;
		cbScenes.removeActionListener(this);
		cbScenes.setSelectedItem(s);
		cbScenes.addActionListener(this);
		refresh();
		resetModified();
	}

	private void setNextScene(int i) {
		int n1 = cbScenes.getSelectedIndex() + i;
		if (n1 > -1 && n1 < cbScenes.getItemCount()) {
			cbScenes.setSelectedItem(cbScenes.getItemAt(n1));
		}
	}

	private JMenuItem getMenuItem(String name) {
		JMenuItem m = new JMenuItem(I18N.getMsg(name));
		m.setName(name);
		m.setText(I18N.getMsg(name));
		m.setIcon(IconUtil.getIconSmall(ICONS.K.NEW));
		m.addActionListener(this);
		return m;
	}

	private JMenuItem getMenuItem(Strand strand) {
		JMenuItem m = new JMenuItem(strand.getName());
		m.setName("strand_" + strand.getId().toString());
		m.setIcon(new ColorIcon(strand.getJColor(), IconUtil.getDefSize()));
		m.addActionListener(this);
		return m;
	}

	private void strandsAction() {
		EntitiesListDlg dlg = new EntitiesListDlg(this, mainFrame, scene, "strand");
		dlg.setModal(true);
		dlg.setVisible(true);
		if (!dlg.isCanceled()) {
			scene.setStrands(dlg.getStrands());
			strandsRefresh();
			setModified();
		}
	}

	private void strandsRefresh() {
		strandsList.removeAll();
		if (!scene.getStrands().isEmpty()) {
			for (Strand str : scene.getStrands()) {
				strandsList.add(new ColorLabel(str.getJColor()));
			}
			strandsList.setToolTipText(scene.getStrandsHtml());
		} else {
			strandsList.add(new JLabel("..."));
			strandsList.setToolTipText("");
		}
		strandsList.revalidate();
	}

	private void strandSelect(ActionEvent evt) {
		//LOG.trace(TT + ".strandSelect()");
		JPopupMenu menu = new JPopupMenu();
		JButton bt = (JButton) evt.getSource();
		for (Strand strand : EntityUtil.findStrands(mainFrame)) {
			menu.add(getMenuItem(strand));
		}
		menu.add(new JSeparator());
		JMenuItem addstrand = getMenuItem("strand.new");
		menu.add(addstrand);
		menu.show(bt, 0, bt.getBounds().height);
	}

	private void strandRefresh(Strand strand) {
		//LOG.trace(TT + ".strandRefresh(strand=" + LOG.traceEntity(strand) + ")");
		scene.setStrand(strand);
		btStrand.setIcon(new ColorIcon(scene.getStrand().getJColor(), IconUtil.getDefSize()));
		btStrand.setToolTipText(scene.getStrand().getName());
	}

	private void strandNew() {
		//LOG.trace(TT + ".strandNew()");
		Strand strand = new Strand();
		boolean r = mainFrame.showEditorAsDialog(strand);
		if (!r) {
			strandRefresh(strand);
			setModified();
		}
	}

	/**
	 * set the deploy button
	 */
	private void setHiden(boolean b) {
		//LOG.trace(TT + ".setHiden(b=" + (b ? "true" : "false") + ") hiden=" + (hiden ? "true" : "false"));
		hiden = b;
		tb2.setVisible(!hiden);
		htTexte.wysEditorGet().setHide(!hiden);
		btHide.setIcon(IconUtil.getIconSmall((!hiden ? ICONS.K.DP_UP : ICONS.K.DP_DOWN)));
		App.getInstance().preferences.setBoolean(Pref.KEY.TYPIST_SHOWBAR, hiden);
	}

	private void commentsInit() {
		//LOG.trace(TT + ".commentsInit()");
		htTexte.wysEditorGet().getToolBar2().addSeparator();
		commentsPanel = new CommentsPanel(mainFrame, this);
		commentsPanel.setVisible(book.getReread());
	}

	/**
	 * initialize the endnote functions
	 *
	 * @return a JButton for activate the liste of Endnotes
	 */
	@SuppressWarnings("unchecked")
	private void endnoteInit(JPanel panel) {
		//LOG.trace(TT + ".initEndnote(textPanel)");
		endnotes = Endnote.find(mainFrame, 0, scene);
		// add endnote button in Shef toolbar
		htTexte.wysEditorGet().getToolBar2().addSeparator();
		JButton bten = new JButton(IconUtil.getIconSmall(ICONS.K.CHAR_ENDNOTE));
		bten.setToolTipText(I18N.getMsg("shef.endnote"));
		bten.addActionListener(e -> {
			Endnote.createEndnote(mainFrame, 0, scene, htTexte);
			endnotes = Endnote.find(mainFrame, 0, scene);
			btEndnotes.setEnabled(!endnotes.isEmpty());
		});
		htTexte.wysEditorGet().getToolBar2().add(bten);
		// define endnotes button at bottom texte panel
		btEndnotes = Ui.initButton("endnote",
				"endnote",
				ICONS.K.SORT,
				"endnote",
				e -> ListEndnotesDlg.showDlg(mainFrame, 0, scene));
		btEndnotes.setEnabled(!endnotes.isEmpty());
		panel.add(btEndnotes, MIG.SPAN);
	}

	private void changeCkComment() {
		book.setReread(!book.getReread());
		book.save();
		commentsPanel.setVisible(book.getReread());
	}

}
